/*
    putconf.c - save configuration table
    Copyright (C) 1993,1994 Uwe Ohse, 1991, 1992 Wolfram Roesler

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


	For questions contact:
	e-mail: (prefered)
		Uwe_Ohse@me.maus.de
		uwe@tirka.gun.de
	snail-mail:
	    Uwe Ohse, Drosselstrasse 2, 47055 Duisburg, Germany

*/

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif
#include <stdio.h>
#include <stdlib.h>
#include "conflib.h"
#include "intern.h"
	
void cl_writeconf(FILE *Fp,const conf_t *K)
{
	if (!Fp || !K)
		return;
	
	while(K->varname)
	{
		switch(K->typ)
		{
			case CL_STRING:
			case CL_ISTRING:
				fprintf(Fp,"%s=%s\n",K->varname,*(char**)K->adr);
				break;
			case CL_NUM:
				fprintf(Fp,"%s=%ld\n",K->varname,*(long*)K->adr);
				break;
			case CL_NUMSHORT:
					fprintf(Fp,"%s=%d\n",K->varname,*(short*)K->adr);
				break;
			case CL_BOOLEAN:
				fprintf(Fp,"%s=%s\n",K->varname,*(int*)K->adr? "true":"false");
				break;
			case CL_ENUM:
				{
					conf_enum_t *p;
					long numvalue;
					char *strvalue;
					numvalue=*(long *)K->adr;
					strvalue=NULL;

					for(p = (conf_enum_t *)K->secdata;p->magic;p++)
					{
						if (p->value==numvalue)
						{
							strvalue=p->magic;
							break;
						}
					}
					if (!strvalue)
						strvalue=((conf_enum_t *)K->secdata)->magic;
					fprintf(Fp,"%s=%s\n",K->varname,strvalue);
				}
				break;
			case CL_ARRAY:	/* Stringarray */
			case CL_IARRAY:	/* mit Env-Expand */
				{
					char **p;
					for(p = (char**)K->adr;*p;p++)
						fprintf(Fp,"%s=%s\n",K->varname,*p);
					break;
				}
			case CL_DUMMY:
			case CL_ALIAS:
				break;
			case CL_LIST:
				{
					conf_list_t *akt;
					char *sep;
					akt = *(conf_list_t **) K->adr;
					sep = (char *) K->secdata;
					if (!sep)
						sep=":";
					fprintf(Fp,"%s=%s",K->varname,
						akt ? "" : "\n");
					while(akt)
					{
						fprintf(Fp,"%s%c",akt->inhalt,
							(akt->next) ? *sep : '\n');
						akt=akt->next;
					}
				}
				break;
		}
		K++;
	}
	return;
}
	
	
