/*
    tinterp.c - textinterpretations
    Copyright (C) 1993  Uwe Ohse,  1991, 1992 Wolfram Roesler

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


	For questions contact:
	e-mail: (prefered)
		Uwe_Ohse@me.maus.de
		uwe@tirka.gun.de
	snail-mail:
	    Uwe Ohse, Drosselstrasse 2, 47055 Duisburg, Germany

*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#include "intern.h"
#endif
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>

#ifdef HAVE_UNISTD_H
#  include <unistd.h>
#endif

#ifdef HAVE_PWD_H
#  include <pwd.h>
#endif

#ifndef HAVE_CONFIG_H
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <pwd.h>
#endif

#include "tinterp.h"

char *tip_errtext;
static int pw_used;

typedef struct tabliststruct 
{
	tip_t *tab;
	char first_magic; /* paranoider check */
	int pw_used;
	struct tablisttruct *next;
} tablist_t;
tablist_t *anker;


#define LENCHECK(l) if (l>=maxlen) { tip_errtext="buffer shortage"; return TIP_ERROR; }
#define ADD_CHAR(c) { *to=c; to[1]='\0'; return 1; } 

/*
    $ABC    the value of env var ABC
    $(ABC)  dito
	$$      a single $
*/
size_t 
tip_dollar(const char *akt, const char **next, char *to, size_t maxlen)
{
	size_t l;
	if (*akt!='$') /* ENV - Var beginnt. */
	{
		tip_errtext="tip_dollar called without $";
		return TIP_ERROR;
	}
	if (akt[1]=='$') /* $$ -> $ */
	{
		LENCHECK(1);
		*next=akt+2;
		ADD_CHAR('$');
	}
	else
	{
		int bracket;
		const char *end;
		const char *start;

		if (akt[1]=='(')
		{
			start=akt+2;
			bracket=1;
		}
		else
		{
			start=akt+1;
			bracket=0;
		}
		end=start;
		while (*end!='\0' 
			&& ( (bracket==0 && (isalnum(*end) || *end=='_')) 
				|| (bracket==1 && *end!=')') 
			))
			end++;
		if (end!=start)
		{
			char *var;
			char varname[256];
			strncpy(varname,start,end-start);
			varname[end-start]='\0';

			*next=end+bracket; /* Schon mal setzen */

			var=getenv(varname);
			if (var)
			{
				l=strlen(var);				
				LENCHECK(l);
				strcpy(to,var);
				return l;
			}
			else
				return 0; /* Durch nichts ersetzt */
		}
		else
		{
			if (*end=='\0')
				*next=end;
			else
				*next=end+1;
			return 0; /* durch nichts ersetzt */
		}
	}
	return TIP_NOTDONE; /* try next method */
}

/*
 * ~name
 * ~~
 * ~(name)
 */
size_t 
tip_tilde(const char *akt, const char **next, char *to, size_t maxlen)
{
	size_t l;

	if (*akt!='~') 
	{
		tip_errtext="tip_tilde called without ~";
		return TIP_ERROR;
	}
	if (akt[1]=='~') /* ~~ -> ~ */
	{
		LENCHECK(1);
		*next=akt+2;
		ADD_CHAR('~'); /* returns 1 */
	}
	else 
	{
		const char *end=akt+1;
		const char *start;
		struct passwd *pw;
		int bracket=0;
		pw_used++;
		if (*end=='(')
		{
			bracket++;
			end++;
		}
		start=end;
		while (isalnum(*end) || *end=='_' 
				|| (bracket && *end!=')') )
			end++;
		if (start==end)
		{
			/* ~/ oder ~; oder ~ */
			pw=getpwuid(getuid());
			*next=akt+1;
		}
		else
		{
			/* ~name/ */
			char un[255];
			strncpy(un,start,end-start);
			un[end-start]='\0';
			pw=getpwnam(un);
			*next=end+bracket;
		}
		if (pw)
		{
			l=strlen(pw->pw_dir);
			LENCHECK(l);
			strcpy(to,pw->pw_dir);
			return l;

		}
		else
		{
			l=end-start+1+bracket*2;
			LENCHECK(l);
			strncpy(to,start-1-bracket,end-start+1+bracket*2);
			to[l]='\0';
			return l;
		}
	}
	return TIP_NOTDONE;
}

/*
 * ~name
 * ~~
 * ~(name)
 */
size_t 
tip_backslash(const char *akt, const char **next, char *to, size_t maxlen)
{
	char nz;

	if (*akt!='\\') 
	{
		tip_errtext="tip_backslash called without \\";
		return TIP_ERROR;
	}
	switch(akt[1])
	{
		case '0': /* oktal oder hex (0x..) */
			if (akt[2]=='x' || akt[2]=='X')
				nz=strtol(akt+1, (char **) next,16);
			else
				nz=strtol(akt+1, (char **) next,8);
			LENCHECK(1);
			ADD_CHAR(nz); /* returns 1 */
			break;
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			nz=strtol(akt+1, (char **) next,10);
			LENCHECK(1);
			ADD_CHAR(nz); /* returns 1 */ 
			break;										
		case '\\':
			LENCHECK(1);
			*next=akt+2;
			ADD_CHAR('\\');
			break;
		case 'a':	/* bell */
			LENCHECK(1);
			*next=akt+2;
			ADD_CHAR('\a');
			break;
		case 'b':   /* backspace */
			LENCHECK(1);
			*next=akt+2;
			ADD_CHAR('\b');
			break;
		case 'f':   /* formfeed */
			LENCHECK(1);
			*next=akt+2;
			ADD_CHAR('\f');
			break;
		case 'n':   /* newline */
			LENCHECK(1);
			*next=akt+2;
			ADD_CHAR('\n');
			break;
		case 'r':   /* carrige return */
			LENCHECK(1);
			*next=akt+2;
			ADD_CHAR('\r');
			break;
		case 't':   /* hor. tab. */
			LENCHECK(1);
			*next=akt+2;
			ADD_CHAR('\t');
			break;
		case 'v':   /* vert tab. */
			LENCHECK(1);
			*next=akt+2;
			ADD_CHAR('\v');
			break;
		case '~':
		case '$':
			LENCHECK(1);
			*next=akt+2;
			ADD_CHAR(akt[1]);
			break;
	} /* switch */
	return TIP_NOTDONE;
}
#undef ADD_CHAR
#undef LENCHECK

size_t
tip_interpret(tip_t *tab, const char *format, char *buffer, size_t buflen)
{
	tip_t *akt;
	char *to;
	size_t done;
	size_t restlen;

	to=buffer;
	restlen=buflen;

	while (*format)
	{
		akt=tab;
		done=TIP_NOTDONE;
		while (akt->magic!='\0')
		{
			if (akt->magic==*format)
			{
				const char *next;
				done=akt->interp(format,&next,to,restlen);
				if (done==TIP_ERROR)
					return TIP_ERROR;
				if (done!=TIP_NOTDONE)
				{
					format=next;
					to+=done;
					restlen-=done;
					break;
				}
				/* Pech, hat wohl nicht gepasst */
			}
			akt++;
		}
		if (done==TIP_NOTDONE)
		{
			/* nichts hat darauf gepasst, also ein Zeichen anhaengen */
			if (1<restlen)
			{
				*to++=*format;
				*to='\0';
				restlen--;
				format++;
			}
			else
			{
				tip_errtext="buffer shortage"; 
				return TIP_ERROR; 
			}
		}
	} /* while (*format) */
	return buflen-restlen+1; /* +1 wg. \0 */
}

int
tip_init(void)
{
	return 0;
}

int 
tip_deinit(void)
{
	if (pw_used)
	{
		endpwent();
		pw_used=0;
	}
	return 0;
}

