/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1993 1994 Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* lib/arg.c: Kommandozeilenhandhabung fuer die Programme */

#include "ownstd.h"
#include "mausconlib.h"
#include <getopt.h>

#define ALL (MAUS2RFC | RFC2MAUS | MROUTE | GATECONFTEST \
		| CALL | GETMAUSVAL | MCOM )

char *conffilename = DEFAULT_CNFNAME;
char *libdirname = DEFAULT_LIBDIR;
char *transferidfilename = NULL;
char *outputfilename = NULL;
char *inputfilename = NULL;
char *argument_from = NULL;
char *argument_value = NULL;
char *mail_to = NULL;
int verbose_flag = 0;
int this_is_mail_flag = 0;
int argument_is_rnews = 0;
int dont_use_nntp = 0;
int dont_use_smtp = 0;
int work_only=ONLY_ALL;
int argument_route_create;
int argument_route_show;
int argument_route_test;
int argument_test;


/* ------------------------------------------------------------------------- */
static void
usage (PROGRAM prog)
{
	typedef struct
	{
		PROGRAM prog;
		char *shortopt;
		char *longopt;
		char *text;
	}
	PARAS;
	PARAS pars[] =
	{
		{
			ALL,
			"-c <configfilename>",
			"--conffile <configfilename>",
			"read config data out of this file (which has to be in <libdir>)"
		},
		{
			MROUTE,
			"-f <from>",
			"--from <sender>",
			"the sender of this message (REQUIRED!)"
		},
		{
			MROUTE,
			"-m",
			"--mail",
			"it's mail, not news (the later is default!)"
		},
		{
			GETMAUSVAL,
			"-V <var>",
			"--val <var>",
			"get a value"
		},
		{
			ALL,
			"--help",
			"-h",
			"give help (this text)"
		},
		{
			MHIST,
			NULL,
			"--id <id>",
			"Show history entry for <id>"
		},
		{
			MROUTE | MAUS2RFC,
			"--input <inputfilename>",
			"-i <inputfilename>",
			"read from this file instead of stdin"
		},
		{
			ALL,
			"--libdir <gatelibrarydirectory>",
			"-l <libdir>",
			"library directory instead of " DEFAULT_LIBDIR
		},
		{
			RFC2MAUS,
			"--output <outputfilename>",
			"-o <outputfilename>",
			"write to this file instead of stdout"
		},
		{
			MAUS2RFC,
			NULL,
			"--only-mail",
			"work mail only"
		},
		{
			MAUS2RFC,
			NULL,
			"--only-news",
			"work news only"
		},
		{
			MAUS2RFC,
			NULL,
			"--only-special",
			"work special messages only"
		},
		{
			MAUS2RFC,
			NULL,
			"--dont-use-nntp",
			"do not use nntp"
		},
		{
			RFC2MAUS,
			"-r",
			"--rnews",
			"work as rnews"
		},
		{
			MROUTE,
			NULL,
			"--route-create",
			"create routing database"
		},
		{
			MROUTE,
			NULL,
			"--route-show",
			"show routing database"
		},
		{
			MROUTE,
			NULL,
			"--route-test",
			"enter test mode (dont send anything)"
		},
		{
			RFC2MAUS,
			"-t <transferidfilename>",
			"--t <transferidfilename>",
			"write transfer-ids to this file instead of /dev/null"
		},
		{
			MROUTE,
			"-T <to-adress>",
			"--to <to-adress>",
			"adress of recipient (mail only)"
		},
		{
			ALL,
			"-v",
			"--verbose",
			"give verbose output (on stderr!)"
		},
		{0, NULL, NULL, NULL}
	};

	PARAS *p = pars;

	fprintf(stderr,"%s\n",global_help_text);

	fprintf (stderr, "Optionen:\n");
	while (p->prog != 0)
	{
		if (prog == GATECONFTEST | (prog & p->prog))
		{
			if (p->shortopt)
				fprintf (stderr, "  %s,\n", p->shortopt);
			if (p->longopt)
				fprintf (stderr, "  %s:\n", p->longopt);
			fprintf (stderr, "    %s\n", p->text);
		}
		p++;
	}
	fprintf (stderr, "action is specified in the configuration file\n");
}

/* gemeinsame argv[]- und configfile-Auswertung aller Programme des
   Pakets
 */
void
configure (PROGRAM prog, int argc, char **argv, int *remainder)
{
	char *this;
	struct option opts[] =
	{
		/* Optionen mit Shortcuts */
		{"from", 1, NULL, 'f'},
		{"help", 0, NULL, 'h'},
		{"input", 1, NULL, 'i'},
		{"libdir", 1, NULL, 'l'},
		{"mail", 0, NULL, 'm'},
		{"output", 1, NULL, 'o'},
		{"rnews", 0, NULL, 'r'},
		{"transfer", 1, NULL, 't'},
		{"to", 1, NULL, 'T'},
		{"val", 1, NULL, 'V'},
		{"verbose", 0, NULL, 'v'},
		/* Optionen mit Flagcharakter ohne Shortopts */
		{"dont-use-nntp", 0, &dont_use_nntp, 1},
		{"dont-use-smtp", 0, &dont_use_smtp, 1},
		{"only-mail", 0, &work_only, ONLY_MAIL},
		{"only-news", 0, &work_only, ONLY_NEWS},
		{"only-special", 0, &work_only, ONLY_SPECIAL},
		{"route-show", 0, &argument_route_show, 1},
		{"route-create", 0, &argument_route_create, 1},
		{"route-test", 0, &argument_route_test, 1},
		{"test", 0, &argument_test, 1},
		{NULL, 0, NULL, 0}
	};
	int option_index = 0;
	int libdir_given = 0;

	while (1)
	{
		int c;

		c = getopt_long (argc, argv, "f:hi:l:mo:rt:T:vV:",
						 opts, &option_index);

		/* Detect the end of the options.   */
		if (c == -1)
			break;
		switch (c)
		{
		default: 
			break; 
		case 'f':
			argument_from = optarg;
			break;
		case 'h':
			usage (prog);
			exit (0);
			break;
		case 'i':
			inputfilename = optarg;
			break;
		case 'l':
			libdir_given = 1;
			libdirname = optarg;
			break;
		case 'm':
			this_is_mail_flag = 1;
			break;
		case 'o':
			outputfilename = optarg;
			break;
		case 'r':
			argument_is_rnews = 1;
			break;
		case 't':
			transferidfilename = optarg;
			break;
		case 'T':
			mail_to = optarg;
			break;
		case 'v':
			verbose_flag = 1;
			break;
		case 'V':
			argument_value = optarg;
			break;
		case '?':
			log (LOG_ERR, "ungltiger Parameter %s", argv[optind - 1]);
			usage (prog);
			exit (2);
			break;
		}
	}							/* while(1) */

	if (argv[optind])
	{
		if (!strcasecmp(argv[optind],"help")
			|| !strcasecmp(argv[optind],"hilf")
			|| !strcasecmp(argv[optind],"hilfe"))
		{
			fprintf(stderr,"%s\n",global_help_text);
			exit(0);
		}
		optind++;
		this=argv[argc-1];
	}
	else if (prog==MROUTE)
		this="route";
	else if (prog==MCOM)
		this="mcom";
	else
	{
		log (LOG_ERR, "Aktion fehlt");
		usage (prog);
		exit (2);
	}
	if (argv[optind])
	{
		if (!remainder)
		{
			log (LOG_ERR, "Zu viele Parameter");
			usage (prog);
			exit (2);
		}
		*remainder=optind;
	}
	else if (remainder)
		*remainder=optind;

	if (!libdir_given)
	{
		char *p;

		p = getenv ("MAUSGATELIBDIR");
		if (p)
			libdirname = p;
	}

	if (do_conf (argv[0], libdirname, conffilename, this))
		exit (1);
	return;
}
