/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1994 Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* 
	lib/cancel.c: Deal with cancel messages 
*/

#include "ownstd.h"
#include <errno.h>
#include <ctype.h>
#include <stdarg.h>
#include <errno.h>
#include "xmalloc.h"
#include "mausconlib.h"
#ifdef FEATURE_HISTORY
#   include "lib/idlib.h"
#endif

static cancel_t *anker;

cancel_t *
cancel_anker(void)
{
	return anker;
}

int
cancel_init(int keep_locked)
{
	char cfname[PATH_MAX];
	char lfname[PATH_MAX];
	FILE *f;
	int fatal_err=0;
	unsigned int sleeptime=1;
	char buf[512];
	char *p;
	cancel_t *new;

	sprintf(lfname,"%s/%s/cancel.lock",opts.spoolbase,action);
	sprintf(cfname,"%s/%s/cancel.list",opts.spoolbase,action);
	while (lock_create (lfname, &fatal_err))
	{
		if (fatal_err)
		{
			log(LOG_ERR,"cannot lock with %s",lfname);
			return 1;
		}
		sleep(sleeptime);
		if (sleeptime<120)
			sleeptime*=2;
	}
	f=fopen(cfname,"r");
	if (!f)
	{
		if (!keep_locked)
			lock_delete(lfname);
		return 0; /* dann gibt's die Datei halt nicht */
	}
	while(fgets(buf,sizeof(buf),f))
	{
		p=strpbrk(buf,"\r\n");
		if (p)
			*p='\0';
		p=strtrim(buf);
		new=XMALLOC(sizeof(cancel_t));
		new->id=XSTRDUP(p);
		new->done=0;
		new->next=anker;
		anker=new;
	}
	fclose(f);
	if (!keep_locked)
		lock_delete(lfname);
	return 0;
}

cancel_t * 
cancel_find_in_list(char *id)
{
	cancel_t *akt;
	char *id_at;
	size_t id_local_l;
	char *this_at;
	size_t this_local_l;
	akt=anker;
	id_at=strchr(id,'@');
	if (!id_at)
		return NULL;
	id_local_l=id_at-id;
	while (akt)
	{
		this_at=strchr(akt->id,'@');
		if (this_at)
		{
			this_local_l=this_at-akt->id;
			if (this_local_l==id_local_l)
			{
				if (!strncmp(akt->id,id,this_local_l)
					&& !strcasecmp(this_at+1,id_at+1))
					return akt;
			}
		}
		akt=akt->next;
	}
	return NULL;
}

void cancel_end(int delete, int rewrite)
{
	char lfname[PATH_MAX];
	sprintf(lfname,"%s/%s/cancel.lock",opts.spoolbase,action);
	if (delete)
	{
		char cfname[PATH_MAX];
		sprintf(cfname,"%s/%s/cancel.list",opts.spoolbase,action);
		unlink(cfname);
	}
	else if (rewrite)
	{
		char cfname[PATH_MAX];
		FILE *f;
		sprintf(cfname,"%s/%s/cancel.list",opts.spoolbase,action);
		f=fopen(cfname,"w");
		if (!f)
			log(LOG_ERR,"cannot open %s: %s", cfname,strerror(errno));
		else
		{
			cancel_t *akt;
			akt=anker;
			while (akt)
			{
				fprintf(f,"%s\n",akt->id);
				akt=akt->next;
			}
			fclose(f);
		}
	}
	lock_delete(lfname);
}

void cancel_add(char *id)
{
	cancel_t *new;
	if (cancel_find_in_list(id))
		return;
	new=XMALLOC(sizeof(cancel_t));
	new->id=XSTRDUP(id);
	new->next=anker;
	new->done=0;
	anker=new;
}

