#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>

#include "ownstd.h"

#include "cslib.h"

#ifdef __TOS__
#include "atarist.h"
#else
#include "iso-8859-1.h"
#endif

#ifdef __PUREC__
#define strcasecmp stricmp
#define strncasecmp strnicmp
#endif

static size_t
int_block_translate(cstl_t *cs, int to_foreign, unsigned char *to, 
	const unsigned char *from, size_t len);

	/* 
	 * Standardhandle.
	 * Null -> kein Zeichensatz eingestellt, Ausgaben entumlauten 
	 */
static cstl_t *tr_cs;

static unsigned char nativetable[256];

int (*ausprintf)(const char *, ... )=printf;

cstl_t *cslib_get_charsettable(void)
{
	return cstl_table;
}

void cslib_set_charset(cstl_t *cs)
{
	tr_cs=cs;
}

void cslib_unset_charset(void)
{
	tr_cs=cslib_get_charset_from_name("ascii");
}

void cslib_init(int (*out_func)(const char *, ... ))
{
	int i;
	cstl_t *p;
	if (out_func != (int (*)(const char *, ... )) 0)
		ausprintf=out_func;
	else 
		ausprintf=printf;

	/* bersetzungstabelle fr den native-charset */
	for (i=0;i<256;i++)
		nativetable[i]=i;

	/* Aliaspointer zuordnen */
	p=cstl_table;
	while (p && p->name)
	{
		if (p->flags & CSF_ALIAS)
		{
			/* Ist ein alias. Jetzt das Original suchen */
			cstl_t *p2;
			p2=cstl_table;
			while (p2->csid)
			{
				if (p2->csid==p->csid && !(p2->flags & CSF_ALIAS))
				{
					p->aliasp=p2;
					break;
				}
				p2++;
			}
		}
		else if (p->flags & CSF_NATIVE)
		{
			p->tr_to=nativetable;
			p->tr_from=nativetable;
		}
		p++;
	}
}

cstl_t *cslib_get_charset_from_name(const char *charsetname)
{
	cstl_t *p;

	p=cstl_table;
	while (p && p->name)
	{
		if (!strcasecmp(p->name, charsetname))
		{
			/* Gefunden. */
			cstl_t *p2;
			/* Wenn kein Alias */
			if (!(p->flags & CSF_ALIAS))
			{
				return p;
			}
			p2=cstl_table;
			while (p2->csid)
			{
				if (p2->csid==p->csid && !(p2->flags & CSF_ALIAS))
					return p2;
				p2++;
			}
			return NULL; /* kann eigentlich nicht passieren */
		}
		p++;
	}
	return NULL; /* nicht gefunden */
}

cstl_t *cslib_get_charset_from_nr(int charsetid)
{
	cstl_t *p;

	p=cstl_table;
	while (p && p->name)
	{
		if (charsetid==p->csid)
			return p;
		p++;
	}
	return NULL; /* nicht gefunden */
}

/*
	Basisroutine zur Umwandlung von Blcken.
	to_foreign: Aus eigenem in fremden Zeichensatz wandeln.
*/
static size_t
int_block_translate(cstl_t *cs, int to_foreign, unsigned char *to, 
	const unsigned char *from, size_t len)
{
	/* 
	 * Wenn kein Zeichensatz angegeben ist, dann den Standardzeichensatz
	 * verwenden.
	 */
	if (!cs)
		cs=tr_cs;

	/* 
	 * wenn auch kein Standardzeichensatz eingestellt oder entumlautet
	 * werden soll.
	 */

	if (!cs || cs->flags & CSF_ENTUMLAUTEN)
	{
		if (!to_foreign)
		{
			memcpy(to,from,len);
			return len;
		}
		else
		{
			/* entumlauten */
			size_t i;
			size_t l;
			const unsigned char *pf;
			unsigned char *pt;
			pf=from;
			pt=to;
			l=0;
			for (i=0;i<len; i++)
			{
				switch(*pf)
				{
				case (unsigned char) '':
					*pt++='u';
					*pt++='e';
					l+=2;
					break;
				case (unsigned char) '':
					*pt++='a';
					*pt++='e';
					l+=2;
					break;
				case (unsigned char) '':
					*pt++='o';
					*pt++='e';
					l+=2;
					break;
				case (unsigned char) '':
					*pt++='U';
					*pt++='e';
					l+=2;
					break;
				case (unsigned char) '':
					*pt++='A';
					*pt++='e';
					l+=2;
					break;
				case (unsigned char) '':
					*pt++='O';
					*pt++='e';
					l+=2;
					break;
				case (unsigned char) '':
					*pt++='s';
					*pt++='s';
					l+=2;
					break;
				default:
					*pt++=*pf;
					l++;
					break;
				} /* switch */
				pf++;
			} /* for */
			return l;
		} /* entumlauten? */
	}
	/*
	 * Wenn der Zeichensatz "native" ist
	 */
	else if (cs && cs->flags & CSF_NATIVE)
	{
		/* einfach kopieren */
		memcpy(to,from,len);
		return len;
	}
	/* 
	 * Wenn entumlautet werden soll
	 */
	else
	{
		const unsigned char *pf;
		unsigned char *pt;
		unsigned char const *tab;
		if (to_foreign)
			tab=cs->tr_to;
		else
			tab=cs->tr_from;
		pt=to;
		pf=from;
		while (*pf)
		{
			*pt++=tab[*pf++];
		}
		*pt='\0';
		return len;
	}
}

size_t cslib_block_to_native(cstl_t *cs, unsigned char *to,
	const unsigned char *from,size_t maxlen, size_t len)
{
	if (len>maxlen)	
		return (size_t) -1;
	return int_block_translate(cs,0,to,from,len);
}

size_t cslib_block_to_foreign(cstl_t *cs, unsigned char *to,
	const unsigned char *from,size_t maxlen, size_t len)
{
	if (len>maxlen)	
		return (size_t) -1;
	return int_block_translate(cs,1,to,from,len);
}

size_t cslib_string_to_native(cstl_t *cs, unsigned char *to,
	const unsigned char *from,size_t maxlen)
{
	size_t len;
	len=strlen(from)+1;
	if (len>maxlen)	
		return (size_t) -1;
	return int_block_translate(cs, 0,to,from,len)-1; /* -1 wg. \0 */
}

size_t cslib_string_to_foreign(cstl_t *cs, unsigned char *to,
	const unsigned char *from,size_t maxlen)
{
	size_t len;
	len=strlen(from)+1; /* +1 wg. \0 */
	if (len>maxlen)	
		return (size_t) -1;
	return int_block_translate(cs, 1,to,from,len)-1; /* -1 wg. \0 */
}

size_t cslib_char_to_native(cstl_t *cs, unsigned char *to, unsigned char from)
{
	/* Kein Zeichensatz angegeben? */
	if (!cs)
		cs=tr_cs;
	to[1]='\0';
	if (!cs || cs->flags & CSF_ENTUMLAUTEN)
	{
		/* dann kopieren */
#if 0
		to[0]=from;
#else
		to[0]=from & 0x7f;
#endif
	}
	else
		*to=cs->tr_from[from];
	return 1;
}

size_t cslib_char_to_foreign(cstl_t *cs, unsigned char *to, unsigned char from)
{
	to[1]='\0';
	/* Kein Zeichensatz angegeben? */
	if (!cs)
		cs=tr_cs;

	if (!cs || cs->flags & CSF_ENTUMLAUTEN)
	{
		/* dann kopieren */
		switch(from)
		{
		case '':
			*to='u';
			to[1]='e';
			to[2]='\0';
			break;
		case '':
			*to='a';
			to[1]='e';
			to[2]='\0';
			break;
		case '':
			*to='o';
			to[1]='e';
			to[2]='\0';
			break;
		case '':
			*to='U';
			to[1]='e';
			to[2]='\0';
			break;
		case '':
			*to='A';
			to[1]='e';
			to[2]='\0';
			break;
		case '':
			*to='O';
			to[1]='e';
			to[2]='\0';
			break;
		case '':
			*to='s';
			to[1]='s';
			to[2]='\0';
			break;
		default:
			*to=from;
			to[1]='\0';
			break;
		} /* switch */
		return 2;
	}
	else
		*to=cs->tr_from[from];
	return 1;
}


void cslib_list(int mit_alias)
{
	cstl_t *cs;
	cs=cstl_table;
	while (cs && cs->name)
	{
		if (cs->flags & CSF_ALIAS && !mit_alias)
		{
			cs++;
			continue;
		}
		ausprintf("%3d %-12.12s ",cs->csid, cs->name);
		if (cs->flags)
		{
			int cnt;
			ausprintf("[");
			cnt=0;
			if (cs->flags & CSF_NATIVE)
			{
				ausprintf("keine Wandlung ntig");
				cnt++;
			}
			if (cs->flags & CSF_ENTUMLAUTEN)
			{
				ausprintf("%s%s",cnt ? ", " : "", 
					"entumlauten bei Ausgabe");
				cnt++;
			}
			if (cs->flags & CSF_ALIAS)
			{
				ausprintf("%s%s%s",cnt ? ", " : "",
					"Alias fr ",cs->aliasp->name);
				cnt++;
			}
			ausprintf("]");
		} /* Wenn Flags angegeben */
		ausprintf("\r\n");
		cs++;
	} /* while */
}
