#ifndef CS_LIB_H
#define CS_LIB_H

/*
	Spezielle Zeichensatz-IDs:
	0: illegal.
	1: MausCode (z.Z. nicht implementiert, aber reserviert)
	1..99: reserviert.
*/
#define CSF_NATIVE 		1
#define CSF_ENTUMLAUTEN	2
#define CSF_ALIAS		4

typedef struct cstl_t_struct
{
	char *name;
	int flags;
	int csid;
	unsigned char const *tr_from;
	unsigned char const *tr_to;
	struct cstl_t_struct *aliasp;
} cstl_t; /* CharSetTransLate_Type */

cstl_t *cslib_get_charset_from_name(const char *charsetname);
cstl_t *cslib_get_charset_from_nr(int charsetid);

/* 
	Mu unbedingt aufgerufen werden. Bei Verwendung aus der Quark
	mu mbprintf bergeben werden. Sonst kann auch 0 bergeben werden,
	dann wird printf verwendet.
 */
void cslib_init(int (*out_func)(const char *, ... ));


	/*
	 * Standardzeichensatz einstellen.
	 */
void cslib_set_charset(cstl_t *cs);
	/*
	 * Standardzeichensatz deaktivieren.
	 */
void cslib_unset_charset(void);
cstl_t *cslib_get_charsettable(void);

void cslib_list(int mit_alias);

/* 
 * Wenn bei den folgenden Funktionen kein Zeichensatz angegeben ist,
 * wird der eingestellte Standardzeichensatz verwendet. Wenn dieser
 * nicht eingestellt ist, wird:
 * - bei Ausgabe entumlautet
 * - bei Eingabe einzelner Zeichen jedes Zeichen mit gesetztem 8. Bit
 *   verworfen (Mailboxonlinebetrieb)
 * - bei Wandlung vieler Zeichen kopiert.
 */

/* 
	maxlen gibt die maximale Lnge des Resultats an (= Lnge des 
	Ausgabepuffers)
	Wenn das Resultat lnger ist als maxlen, scheitert die Umwandlung,
	die Funktionen liefern (size_t) -1 zurck.
	Die Ausgabe kann durchaus lnger werden als als die Eingabe
	(keine Umlaute, Mauscode ...)
	Auf der sicheren Seite ist man, wenn der Ausgabepuffer viermal so
	gro ist wie die Eingabe.
*/
size_t cslib_block_to_native(cstl_t *cs, unsigned char *to, 
	const unsigned char *from, size_t max, size_t len);
size_t cslib_block_to_foreign(cstl_t *cs, unsigned char *to, 
	const unsigned char *from, size_t max, size_t len);
size_t cslib_string_to_native(cstl_t *cs, unsigned char *to, 
	const unsigned char *from, size_t max);
size_t cslib_string_to_foreign(cstl_t *cs, unsigned char *to, 
	const unsigned char *from, size_t max);
/* auch hier: to==NULL -> statischer Puffer. 
	to braucht hchstens 4 Byte zu sein 
*/
size_t cslib_char_to_native(cstl_t *cs, unsigned char *to, unsigned char from);
size_t cslib_char_to_foreign(cstl_t *cs, unsigned char *to, unsigned char from);

#endif
