/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1993 1994 Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* 
	lib/do_conf.c - Auslesen der Konfigurationsdaten fr maus2rfc 
	Diese Datei erledigt die Konfiguration fuer alle Programme des
	Pakets.
*/

#include "ownstd.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <conflib.h>

#if defined HAVE_SYSLOG_H && defined HAVE_SYSLOG
#define USE_SYSLOG
#include <syslog.h>
#endif

#include "mausconlib.h"
#include "xmalloc.h"
#define INIT(x,y)	y,
#include "mausconlib.h"
#undef INIT

char *gate_version = VERSION;
char *action;

static conf_enum_t workmodeenum[] =
{
	{"user", USER},
	{"gate", GATE},
	{NULL, 0}
};

static conf_enum_t rekillenum[] =
{
	{"full", REKILL_FULL},
	{"smart", REKILL_SMART},
	{"none", REKILL_NONE},
	{NULL, 0}
};
static conf_enum_t mailmodeenum[] =
{
	{"bsmtp", MAILMODE_BSMTP},
#ifdef FEATURE_SMTP
	{"smtp", MAILMODE_SMTP},
#endif
	{NULL, 0}
};

static conf_enum_t boxtypenum[] =
{
	{"quark 1", BOXTYP_QUARK_I},
	{"quark I", BOXTYP_QUARK_I},
	{"quark", BOXTYP_QUARK_I},
	{"quark 2", BOXTYP_QUARK_II},
	{"quark II", BOXTYP_QUARK_II},
	{"maus", BOXTYP_MAUS},
	{NULL, 0}
};

#ifdef USE_SYSLOG
conf_enum_t facenum[] =
{
	{"daemon", LOG_DAEMON},
	{"user", LOG_USER},
	{"uucp", LOG_UUCP},
	{"mail", LOG_MAIL},
	{"news", LOG_NEWS},
	{"local0", LOG_LOCAL0},
	{"local1", LOG_LOCAL1},
	{"local2", LOG_LOCAL2},
	{"local3", LOG_LOCAL3},
	{"local4", LOG_LOCAL4},
	{"local5", LOG_LOCAL5},
	{"local6", LOG_LOCAL6},
	{"local7", LOG_LOCAL7},
	{NULL, 0}
};
#endif
/* Die folgenden sind in mausconlib.h auf jeden Fall definiert worden */
conf_enum_t prioenum[] =
{
	{"debug", LOG_DEBUG},
	{"info", LOG_INFO},
	{"notice", LOG_NOTICE},
	{"warning", LOG_WARNING},
	{"err", LOG_ERR},
	{"crit", LOG_CRIT},
	{"alert", LOG_ALERT},
	{"emerg", LOG_EMERG},
	{NULL, 0}
};


conf_list_t *list_allowed;
conf_list_t *list_reach_sites;
conf_list_t *list_route_actions;

conf_t Tab[] =
{
	{"route-actions", NULL, CL_LIST, &list_route_actions, 0},
	{"pollsite", NULL, CL_STRING, &opts.pollsite, 0},
	{"mailuser", NULL, CL_STRING, &opts.mailuser, 0},
	{"mailuserdomain", NULL, CL_STRING, &opts.mailuserdomain, 0},
	{"allowed", " \t", CL_LIST, &list_allowed, 0},
	{"reach-sites", " \t", CL_LIST, &list_reach_sites, 0},
	{"reroute-host", NULL, CL_STRING, &opts.reroute_host, 0},
	{"boxtyp", &boxtypenum, CL_ENUM, &opts.boxtyp, 0},
	{"mode", &workmodeenum, CL_ENUM, &opts.workmode, 0},
	{"mailmode", &mailmodeenum, CL_ENUM, &opts.mailmode, 0},
	{"direct_smtp", NULL, CL_BOOLEAN, &opts.direct_smtp, 0},
	{"mailhost", NULL, CL_STRING, &opts.mailhost, 0},
	{"newshost", NULL, CL_STRING, &opts.newshost, 0},
	{"usenet-dist-local", NULL, CL_STRING, &opts.usenet_dist_local, 0},
	{"usenet-dist-mausnet", NULL, CL_STRING, &opts.usenet_dist_mausnet, 0},
	{"mausnet-dist-mausnet", NULL, CL_STRING, &opts.mausnet_dist_mausnet, 0},
	{"mausnet-dist-local", NULL, CL_STRING, &opts.mausnet_dist_local, 0},
	{"mausnet-dist-splitted", NULL, CL_STRING, &opts.mausnet_dist_splitted, 0},
	{"compress", NULL, CL_STRING, &opts.compress, 0},
#ifdef USE_SYSLOG
	{"sl-facility", &facenum, CL_ENUM, &opts.sl_facility, 0},
	{"sl-priority", &prioenum, CL_ENUM, &opts.sl_priority, 0},
#endif

	{"gatemaster", NULL, CL_STRING, &opts.gatemaster, 0},
	{"mysite", NULL, CL_STRING, &opts.mysite, 0},
	{"mysitedomain", NULL, CL_STRING, &opts.mysitedomain, 0},
 {"organization", "ORGANIZATION", CL_STRING, &opts.organization, CL_PREENV},
	{"path-entry", NULL, CL_STRING, &opts.path_entry, 0},
	{"gprefix", NULL, CL_STRING, &opts.gprefix, 0},

	{"mausgatedomain", NULL, CL_STRING, &opts.mausgatedomain, 0},
	{"mausdomain", NULL, CL_STRING, &opts.mausdomain, 0},

	{"spoolbase", NULL, CL_STRING, &opts.spoolbase, 0},
	{"tempdir", "TMP", CL_ISTRING, &opts.tempdir, CL_POSTENV},

	{"lock-tries", NULL, CL_NUMSHORT, &opts.lock_tries, 0},
	{"lock-sleep", NULL, CL_NUMSHORT, &opts.lock_sleep, 0},

	{"munge-mails", NULL, CL_BOOLEAN, &opts.munge_mails, 0},
	{"testmode", NULL, CL_BOOLEAN, &opts.testmode, 0},
	{"send-ag", NULL, CL_BOOLEAN, &opts.send_ag, 0},
	{"send-dzeile", NULL, CL_BOOLEAN, &opts.send_dzeile, 0},
	{"send-crosspostings", NULL, CL_BOOLEAN, &opts.send_crosspostings, 0},
	{"send-followup-to", NULL, CL_BOOLEAN, &opts.send_followup_to, 0},
	{"send-sender", NULL, CL_BOOLEAN, &opts.send_sender, 0},
	{"send-reply-to", NULL, CL_BOOLEAN, &opts.send_reply_to, 0},
	{"send-path", NULL, CL_BOOLEAN, &opts.send_path, 0},
	{"warn-news-routing", NULL, CL_BOOLEAN, &opts.warn_news_routing, 0},
	{"split-after", NULL, CL_NUM, &opts.split_after, 0},
	{"cancel-echo", NULL, CL_BOOLEAN, &opts.cancel_echo, 0},
	{"kill-crosspostings", NULL, CL_BOOLEAN, &opts.kill_crosspostings, 0},
	{"unlink-unallowed", NULL, CL_BOOLEAN, &opts.unlink_unallowed, 0},
	{"mime-coded-headers", NULL, CL_BOOLEAN, &opts.mime_coded_headers, 0},
	{"mime-coded-body", NULL, CL_BOOLEAN, &opts.mime_coded_body, 0},
	{"autoinfosubscribe", NULL, CL_BOOLEAN, &opts.autoinfosubscribe, 0},
	{"rekillmode", &rekillenum, CL_ENUM, &opts.rekillmode, 0},
	{"tabsize", NULL, CL_NUMSHORT, &opts.tabsize, 0},
	{"subjectlen", NULL, CL_NUMSHORT, &opts.subjectlen, 0},
	{"limit-references", NULL, CL_NUMSHORT, &opts.limit_references, 0},
	{"diffcmd", NULL, CL_ISTRING, &opts.diffcmd, 0},

	/* Kompatibilitaet zu frueherem Schreibfehler (<0.3.4)*/
	{"log.session", "log_session", CL_ALIAS, NULL, 0},
	{"log-session", NULL, CL_BOOLEAN, &opts.log_session, 0},
	{"dial-retry-time", NULL, CL_NUMSHORT, &opts.dial_retry_time, 0},
	{"dial-timeout", NULL, CL_NUMSHORT, &opts.dial_timeout, 0},
	{"uucplockdir", NULL, CL_STRING, &opts.uucplockdir, 0},
	{"uucplock-sleep", NULL, CL_NUMSHORT, &opts.uucplock_sleep, 0},
	{"modem-line", NULL, CL_ISTRING, &opts.modem_line, 0},
	{"modem-at", NULL, CL_ISTRING, &opts.modem_at, 0},
	{"modem-speed", NULL, CL_NUM, &opts.modem_speed, 0},
	{"modem-reset", NULL, CL_ISTRING, &opts.modem_reset, 0},
	{"modem-init", NULL, CL_ISTRING, &opts.modem_init, 0},
	{"modem-timeout", NULL, CL_NUMSHORT, &opts.modem_timeout, 0},
	{"modem-dial", NULL, CL_ISTRING, &opts.modem_dial, 0},

	{"fp-max", NULL, CL_NUMSHORT, &opts.fp_max, 0},
	{"fp-busy", NULL, CL_NUMSHORT, &opts.fp_busy, 0},
	{"fp-no-carrier", NULL, CL_NUMSHORT, &opts.fp_no_carrier, 0},
	{"fp-no-dial", NULL, CL_NUMSHORT, &opts.fp_no_dial, 0},
	{"fp-error", NULL, CL_NUMSHORT, &opts.fp_error, 0},
	{"fp-init", NULL, CL_NUMSHORT, &opts.fp_init, 0},
	{"fp-reset", NULL, CL_NUMSHORT, &opts.fp_reset, 0},
	{"fp-lock", NULL, CL_NUMSHORT, &opts.fp_lock, 0},
	{"fp-dev-init", NULL, CL_NUMSHORT, &opts.fp_dev_init, 0},
	{"fp-dial-timeout", NULL, CL_NUMSHORT, &opts.fp_dial_timeout, 0},
	{"fp-modem-timeout", NULL, CL_NUMSHORT, &opts.fp_dial_timeout, 0},


	{NULL, NULL, 0, NULL, 0}
};

static conf_t *backup;

char *
get_conf_value (char *s)
{
	conf_t *t = Tab;
	int found = 0;
	char buf[128];
	char *erg = NULL;

	while (t->varname != NULL)
	{
		if (!strcasecmp (t->varname, s))
		{
			found++;
			switch (t->typ)
			{
			case CL_ENUM:
				{
					conf_enum_t *p;
					int found = 0;

					p = (conf_enum_t *) t->secdata;
					while (p && p->magic)
					{
						if (*(long *) t->adr == p->value)
						{
							found = 1;
							erg = p->magic;
							break;
						}
						p++;
					}
					if (!found)
						erg = ((conf_enum_t *) t->secdata)->magic;
				}
				break;
			case CL_STRING:
			case CL_ISTRING:
				erg = *(char **) t->adr;
				break;
			case CL_NUM:
				sprintf (buf, "%ld", *(long *) t->adr);
				erg = buf;
				break;
			case CL_NUMSHORT:
				sprintf (buf, "%d", *(short *) t->adr);
				erg = buf;
				break;
			case CL_BOOLEAN:
				if (*(int *) t->adr)
					erg = "true";
				else
					erg = "false";
				break;
			case CL_ARRAY:
			case CL_IARRAY:
				erg = "?";
				break;
			case CL_ALIAS: 
				return get_conf_value((char *)t->secdata);
			case CL_LIST:
				erg= "?";
				break;
			case CL_DUMMY:
				erg = "?";
				break;
			}
		}						/* if */
		t++;
	}							/* while */
	if (found)
		return erg;
	else
		return "(null)";
}

void
dump_conf (FILE * f)
{
	fprintf (f, "action: %s\n", action);
	cl_writeconf (f, Tab);
}

static void 
list_convert(mc_list_t **mclist,conf_list_t *cllist)
{
	mc_list_t *new;
	mc_list_t *last=NULL;
	conf_list_t *akt;
	akt=cllist;
	while (akt)
	{
		if (akt->inhalt)
		{
			new=XMALLOC(sizeof(mc_list_t));
			new->data=XSTRDUP(akt->inhalt);
			new->ext=NULL;
			new->next=NULL;
			if (last)
				last->next=new;
			else
				*mclist=new;
			last=new;
		}
		akt=akt->next;
	}
}

static void 
list_free(mc_list_t **mclist)
{
	mc_list_t *akt;
	mc_list_t *next;
	akt=*mclist;
	while (akt)
	{
		if (akt->data)
			free(akt->data);
		if (akt->ext)
			free(akt->ext);
		next=akt->next;
		free(akt);
		akt=next;
	}
	*mclist=NULL;
}

int
do_conf (char *progname, char *libdir, char *conffilename, char *aktion)
{
	int erg;
	char fname[PATH_MAX];
	int is_routing;

	static int first=0;

	if (conffilename == NULL)
		conffilename = DEFAULT_CNFNAME;
	if (libdir == NULL)
		libdir = DEFAULT_LIBDIR;
	if (conffilename[0] == '/')
		strcpy (fname, conffilename);
	else
		sprintf (fname, "%s/%s", libdir, conffilename);

	action = aktion;

	is_routing= !strcasecmp(aktion,"route");

	if (cl_getversion () < 022)
	{
		log (LOG_CRIT, "inkompatible Version der Conflib %d", cl_getversion ());
		return 1;
	}
	if (!first)
	{
		backup=XMALLOC(sizeof(Tab));
		memcpy(backup,Tab,sizeof(Tab));
		first=1;
	}
	else
	{
		cl_clearconf(Tab);
		memcpy(Tab,backup,sizeof(Tab));
	}

	erg = cl_getconfstanza (fname, Tab, "default",1);
	if (erg)
	{
		/* tja, pech */
		log (LOG_ERR, "kann Defaults nicht aus %s lesen", fname);
		return 1;
	}
	if (!is_routing && aktion)
	{
		erg = cl_getconfstanza (fname, Tab, aktion,1);
		if (erg)
		{
			/* tja, pech */
			cl_closeconf();
			log (LOG_ERR, "kann %s nicht aus %s lesen", aktion, fname);
			return 1;
		}
	}
	cl_closeconf();
	if (!opts.mysite)
	{
		opts.mysite = XMALLOC(128);
		gethostname (opts.mysite, 127);
	};
	if (!opts.mysitedomain)
	{
		char *p;
		size_t l;

		opts.mysitedomain = XMALLOC(128);
		strcpy (opts.mysitedomain, opts.mysite);
		l = strlen (opts.mysitedomain);
		opts.mysitedomain[l]='.';
		l++;
		p = opts.mysitedomain + l;
		getdomainname (p, 127 - l);
		if (!*p || *p=='(')
			gethostname(opts.mysitedomain,127);
	};
#define MISSCHECK(x) \
if (!x || !*x) \
{ \
	fprintf(stderr,"%s: missing %s\n",progname,#x); \
	log(LOG_ERR,"%s fehlt",#x); \
	return 1; \
}
	MISSCHECK (opts.gatemaster);
	MISSCHECK (opts.mausgatedomain);
	MISSCHECK (opts.mausdomain);
	MISSCHECK (opts.mysite);
	MISSCHECK (opts.mysitedomain);
	MISSCHECK (opts.spoolbase);
	if (!is_routing)
	{
		MISSCHECK (opts.mailuser);
		if (!opts.mailuserdomain)
		{
			opts.mailuserdomain=XMALLOC(strlen(opts.mailuser)
				+strlen(opts.mysitedomain)+2);
			sprintf(opts.mailuserdomain,"%s@%s",opts.mailuser,opts.mysitedomain);
		}
		MISSCHECK (opts.pollsite);
		MISSCHECK (opts.gprefix);
#if 0
		MISSCHECK (opts.path_entry);
#endif
		MISSCHECK (opts.mailhost);
		MISSCHECK (opts.newshost);
#undef MISSCHECK
	/* Zwischen conflib-Listenformat und MC-Listenformat wandeln */
		list_free(&opts.allowed);
		list_convert(&opts.allowed,list_allowed);
		if (list_reach_sites)
		{
			list_free(&opts.reach_sites);
			list_convert(&opts.reach_sites,list_reach_sites);
		}
		else if (opts.workmode==USER)
		{
			conf_list_t li;
			li.inhalt="*";
			li.next=NULL;
			list_free(&opts.reach_sites);
			list_convert(&opts.reach_sites,&li);
		}
		else
			list_free(&opts.reach_sites);
	}
	list_convert(&opts.route_actions,list_route_actions);
	return 0;
}
