/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1993 1994 Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
	lib/errormail.c: Fehlermeldungen verschicken.
	Zustand zur Zeit: Uebel.
*/

#include "ownstd.h"

#include <stdarg.h>
#include <errno.h>

#include "mausconlib.h"
#include "xmalloc.h"

#define MAXEMP	10

/* ------------------------------------------------------------------------- */


void
errormail (FILE * input, char *subject, char *error,...)
{
	va_list ap;
	int zahl;
	int i;
	int end = 0;
	char *emp[MAXEMP];
	FILE *f[MAXEMP];
	char buf[1024];


	if (argument_test)
		return;

	zahl = 0;

	va_start (ap, error);

	while (zahl < MAXEMP && !end)
	{
		emp[zahl] = va_arg (ap, char *);

		end = (emp[zahl] == NULL);
		if (!end)
			zahl++;
	}
	va_end (ap);

	for (i = 0; i < zahl; i++)
	{
		char s[128];

		if (opts.testmode)
			sprintf (s, "mail -s '%s to %s' %s", subject, emp[i], opts.gatemaster);
		else
			sprintf (s, "mail -s '%s' %s", subject, emp[i] + 1);
		f[i] = popen (s, "w");
		if (!f[i])
		{
			int e;

			e = errno;
			fprintf (stderr, "FATAL MAUSGATE ERROR\n"
					 "MausConnect\n"
					 "could not send error messages\n"
					 "Message lost due to error:\n"
					 "%s\n%s\n",
					 subject, error);
			log (LOG_CRIT, "Message lost, couldn't send error message");
			log (LOG_CRIT, "could not popen to mail: %s",
				 strerror (e));
		}
	}
	for (i = 0; i < zahl; i++)
	{
		if (f[i])
		{
			fprintf (f[i], "Mausgate %s <-> %s (MausConnect version %s)\n",
					 opts.pollsite, opts.mysitedomain, gate_version);
			fprintf (f[i], "\n");
			fprintf (f[i], "Critical problem during delivering:\n");
			fprintf (f[i], "%s\n", subject);
			fprintf (f[i], "%s\n", error);
			if (emp[i][0] == '-' || emp[i][0] == '=')
				dump_conf (f[i]);
			if (emp[i][0] == '+' || emp[i][0] == '=')
				fprintf (f[i], "------ failed message follows ------\n");
		}
	}
	if (input)
		while (fgets (buf, sizeof (buf) - 1, input))
		{
			for (i = 0; i < zahl; i++)
				if (f[i])
					fprintf (f[i], "%s", buf);
		}
	else
	{
		for (i = 0; i < zahl; i++)
			if (f[i])
				fprintf (f[i], "message completely lost! Sorry!\n");
	}
	for (i = 0; i < zahl; i++)
		if (f[i])
			pclose (f[i]);
}


void
bounce (char *to, char *reason, char *mbuf)
{
	char buf[1024];

	FILE *pipef;

	if (argument_test)
		return;

	sprintf (buf, "mail -s 'message failed' '%s'", to);
	pipef = popen (buf, "w");
	if (!pipef)
	{
		char g[1024];
		char h[PATH_MAX * 2];
		int e;

		e = errno;
		sprintf (g, "=%s", opts.gatemaster);
		sprintf (h, "%s: %s", buf, strerror (errno));
		errormail (NULL, "error bouncing message",
				   h, g, NULL);
		log (LOG_CRIT, "could not bounce message: %s",
			 strerror (e));
		return;
	}
	fprintf (pipef, "%s\n", reason);
	if (mbuf)
	{
		fprintf(pipef,"failed message follows:\n");
		fprintf(pipef,"%s",mbuf);
	}
	else
		fprintf(pipef,"failed message completely lost\n");
	pclose (pipef);
}
