/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1992 1993 1994 Michael Kraehe, Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* 
	lib/names.c: Gruppennamenshandling in MausConnect.
	Zustand: wird wegen notwendigier Erweiterungen bald ueberarbeitet.
*/

#include "ownstd.h"

#include <assert.h>
#include <ctype.h>
#include <errno.h>

#include "mausconlib.h"
#include "xmalloc.h"
#include "groups.h"

static grp_data_t *anker = NULL;
static grp_data_t *last = NULL;

grp_data_t leer_grp;

BOOLEAN 
grp_lock(void)
{
	char lockfname[PATH_MAX];   /* Lockfile */
	int fatal;

	sprintf(lockfname,"%s/groups.LCK",opts.spoolbase);
	while (lock_create(lockfname,&fatal))
	{
		if (fatal)
		{
			log(LOG_ERR,"kann Lockfile fuer Gruppen nicht anlegen: %s\n",	
				strerror(errno));
			return FALSE;
		}
		fprintf(stderr,"Locke Gruppendatei\n");
		sleep(5);
	}
	return TRUE;
}

void 
grp_unlock(void)
{
	char lockfname[PATH_MAX];   /* Lockfile */
	sprintf(lockfname,"%s/groups.LCK",opts.spoolbase);
	unlink(lockfname);
}

grp_data_t * 
grp_get_anker(void)
{
	return anker;
}

BOOLEAN 
grp_delete(grp_data_t *grp)
{
	if (anker==grp)
		anker=grp->next;
	else
	{
		grp_data_t *akt;
		akt=anker;
		while (akt->next && akt->next!=grp)
			akt=akt->next;
		if (akt->next)
			akt->next=akt->next->next;
		else
			return FALSE;
	}
	if (grp->boxen)
		free(grp->boxen);
	if (grp->mausname)
		free(grp->mausname);
	if (grp->rfcname)
		free(grp->rfcname);
	if (grp->default_followup)
		free(grp->default_followup);
	if (grp->userinfo)
		free(grp->userinfo);
	if (grp->approve)
		free(grp->approve);
	return TRUE;
}

/*
 * Die Gruppennamenzuordnungsdatei einlesen. 
 * Format:
 * boxkuerzel<whitespace>name_in_rfc<whitespace>name_in_maus
 * Alles ab "#" ist Kommentar
 */
BOOLEAN
grp_load_list (void)
{
	FILE *f;
	char fname[PATH_MAX];

	char *p;
	grp_data_t g=LEER_GRP;

	char str[1024];				/* Puffer fr eine gelesene Zeile */
	grp_data_t *new;
	long linenumber = 0;

	leer_grp=g;

	sprintf(fname,"%s/groups",opts.spoolbase);

	if ((f = fopen (fname, "r")) == NULL)
	{
		int e;

		e = errno;
		/* Das ist kein allzu ernstzunehmendes Problem */
		/* Im Gatemodes *ist* es ernstzunehmen */
		log (LOG_ERR, "could'nt read %s: %s", fname, strerror (errno));
		return FALSE;
	}

	new=XMALLOC(sizeof(grp_data_t));
	*new=leer_grp;

	while (fgets (str, sizeof (str) - 1, f))
	{
		char *inhalt;
		linenumber++;

		/* 
		 * Zeilenende weg
		 */
		p=strpbrk(str,"\r\n");
		if (p)
			*p='\0';
		if (!*str || !strncmp(str,"---",3))
			continue;
		p=str;
		while (isspace(*p))
			p++;
		if (!*p)
			continue;
		inhalt=strchr(p,'=');
		if (!inhalt)
			continue;

		/* Variablennamen terminieren */

		*inhalt++='\0';
		strtrim(inhalt);
		strtrim(p);

		if (!strcasecmp(p,"group"))
		{
			if (new->rfcname && new->mausname)
			{
				if (!grp_append_new(new))
				{
					fclose(f);
					return FALSE;
				}
				new=XMALLOC(sizeof(grp_data_t));
				*new=leer_grp;
			}
			new->rfcname=XSTRDUP(inhalt);
		}
		else if (!strcasecmp(p,"mausname"))
			new->mausname=XSTRDUP(inhalt);
		else if (!strcasecmp(p,"approve") || !strcasecmp(p,"approved"))
			new->approve=XSTRDUP(inhalt);
		else if (!strcasecmp(p,"userinfo") || !strcasecmp(p,"info"))
			new->userinfo=XSTRDUP(inhalt);
		else if (!strcasecmp(p,"followup") || !strcasecmp(p,"followup-to"))
			new->default_followup=XSTRDUP(inhalt);
		else if (!strcasecmp(p,"usenet-distribution") 
			|| !strcasecmp(p,"usenet-dist"))
			new->usenet_distribution=XSTRDUP(inhalt);
		else if (!strcasecmp(p,"mausnet-distribution") 
			|| !strcasecmp(p,"mausnet-dist"))
			new->mausnet_distribution=XSTRDUP(inhalt);
		else if (!strcasecmp(p,"boxen") || !strcasecmp(p,"box"))
			new->boxen=XSTRDUP(inhalt);
		else if (!strcasecmp(p,"mime"))
		{
			new->mime_headers=TRI_FALSE;
			new->mime_body=TRI_FALSE;
			if (tolower(*inhalt)=='h')
				new->mime_headers=TRUE;
			else if (tolower(*inhalt)=='b')
			{
				new->mime_headers=TRUE;
				new->mime_body=TRUE;
			}
		}
		else if (!strcasecmp(p,"checkgroups"))
		{
			new->checkgroups=parse_bool(inhalt);
		}
		else if (!strcasecmp(p,"itg"))
		{
			new->itg=parse_bool(inhalt);
		}
		else
			fprintf(stderr,"groups: %s: unknown variable\n",p);
	} /* while */
	fclose(f);

	if (new->rfcname && new->mausname)
	{
		if (!grp_append_new(new))
			return FALSE;
	}
	return TRUE;
}

/* Einen neuen Mausnamen in die Liste eintragen */
BOOLEAN
grp_append_new(grp_data_t *new)
{
	if (!anker)
		anker=new;
	else
		last->next=new;
	new->next=NULL;
	last=new;
	return TRUE;
}


/* Liefert den mausinternen Namen der mit Rfcnamen gegebenen Gruppe zurck */
grp_data_t *
grp_suche_mausname(const char *mausname, const char *box)
{
	grp_data_t *akt;

	akt=anker;
	while (akt)
	{
		/* wenn na->box == wildcard-box oder box==na->box) */
		if (!box 
			|| ((akt->boxen[0] == '*' && akt->boxen[1] == '\0') 
				|| !strcasecmp (box, akt->boxen)))
		{
			if (!strcasecmp (mausname, akt->mausname))
				return akt;
		}
		akt = akt->next;
	}
	return NULL;
}

grp_data_t *
grp_suche_rfcname(const char *rfcname, const char *box)
{
	grp_data_t *akt;

	akt=anker;
	while (akt)
	{
		if (!box 
			|| ((akt->boxen[0] == '*' && akt->boxen[1] == '\0') 
				|| !strcasecmp (box, akt->boxen)))
		{
			if (!strcasecmp (rfcname, akt->rfcname))
				return akt;
		}
		akt = akt->next;
	}
	return NULL;
}

BOOLEAN
grp_save_list(void)
{
	FILE *f;
	char fname[PATH_MAX];
	grp_data_t *akt;

	sprintf(fname,"%s/groups",opts.spoolbase);

	f = fopen (fname, "w");
	if (!f)
	{
		log (LOG_ERR, "could'nt write %s: %s", fname, strerror (errno));
		return FALSE;
	}
	akt=anker;
	while (akt)
	{
		if (!akt->dont_save)
		{
			fprintf(f,"Group=%s\n",akt->rfcname);
			fprintf(f,"\tMausname=%s\n",akt->mausname);
			fprintf(f,"\tBoxen=%s\n",akt->boxen);
			if (akt->userinfo)
				fprintf(f,"\tUserinfo=%s\n",akt->userinfo);
			if (akt->approve)
				fprintf(f,"\tApprove=%s\n",akt->approve);
			if (akt->default_followup)
				fprintf(f,"\tFollowup=%s\n",akt->default_followup);
			if (akt->checkgroups)
				fprintf(f,"\tCheckgroups=true\n");
			if (akt->itg)
				fprintf(f,"\tITG=true\n");
			if (akt->mime_headers==TRI_TRUE)
				fprintf(f,"\tMIME=headers\n");
			else if (akt->mime_headers==TRI_FALSE)
				fprintf(f,"\tMIME=none\n");
			fprintf(f,"--------------------------------------\n");
		}
		akt=akt->next;
	}
	fclose(f);
	return TRUE;
}


/* Gruppennamenskonvertierung */

/* Was soll passieren, falls ein Namensteil zu lang (>14 Zeichen) ist?
 * #undef SKIP_TO_DOT  -> Punkt zwischenschieben.
 * #define SKIP_TO_DOT -> alles bis zum nchsten Punkt ignorieren.
 * Letzteres mte ausreichen, um eindeutige Namen zu erzeugen, erzeugt
 * aber eindeutig die schneren Namen.
 */
#define SKIP_TO_DOT

/*
 * Liste bekannter Gruppenhierarchienamen, fr die kein "maus." als
 * Prefix vorgehngt wird.
 */
char *known_prefix[] =
{
	"alt",
	"aus",
	"bionet",
	"bit",
	"biz",
	"bln",
	"clari",
	"comp",
	"de",
	"fido",
	"fj",
	"gnu",
	"k12",
	"maus",
	"misc",
	"news",
	"rec",
	"schule",
	"sci",
	"soc",
	"talk",
	"vmsnet",
	"zer",
	NULL
};

#define IS_SPECIAL(chr) (strchr("+-._",chr))
BOOLEAN 
grp_create_rfcname (unsigned char *unconverted, unsigned char *to)
{
	unsigned char *p;
	unsigned char *q;
	unsigned char *last_dot;
	unsigned char c;
	unsigned char origc;
	unsigned char last;
	unsigned char *buf;
	char **prefix;

#ifdef SKIP_TO_DOT
	int skip_to_dot = 0;
#endif

	buf = XMALLOC (strlen (unconverted) * 2 + 1);

	/* entumlauten */
	strcpy (buf, unconverted);
	convert_umlaute (buf);

	/* Feststellen, ob ein "maus."-Prefix vor den Namen gestellt werden
	 * mu.
	 */
	*to = '\0';
	prefix = known_prefix;
	while (*prefix)
	{
		size_t l;
		l=strlen(*prefix);
		if (!strncasecmp (buf, *prefix, l))
		{
			if (buf[ l ] == '.')
				break;
		}
		prefix++;
	}
	if (!*prefix)
	{
		if (!strcmp(buf,"maus"))
			sprintf (to, "maus.maus.");
		else
			sprintf (to, "%s.%s.", opts.gprefix, opts.pollsite);
	}

	q = to + strlen (to);
	last_dot = q - 1;

	for (p = buf; *p; p++)
	{
		if (p==buf)	
			last='\0';
		else 
			last=p[-1];

		/* 
		 * Zeichenmenge auf Kleinbuchstaben und Ziffern sowie "+-_."
		 * beschrnken.
		 */
		origc = *p;
		if (isupper (origc))
			c = tolower (origc);
		else if (origc=='&' || origc=='/')
			c = '+';
		else if (islower (origc) || isdigit (origc) || origc == '.'
			|| origc == '+' || origc == '-' || origc == '_' || origc == '=')
			c = origc;
		else
		{
			/* alles andere in Punkte verwandeln */
			c = '.';
		}

		/* 
		 * Das Anfangszeichen der Komponente prfen 
		 */
		if (q - last_dot == 1)
		{
			/* "subsequent components SHOULD begin with a letter" */
			if (!(islower (c)))
				continue;
		}
		if (q - last_dot == 14)
		{
			/* 14 Zeichen ist die Maximallnge fr Namensteile */
#ifndef SKIP_TO_DOT
			last_dot = q;
			*q++ = '.';
			last = '.';
#else
			skip_to_dot = 1;
#endif
		}

		/* Den Punkt behandeln */
		if (c == '.')
		{	
#ifdef SKIP_TO_DOT
			skip_to_dot = 0;
#endif
			/* "+" wg. c++ besondert behandeln */
			if (IS_SPECIAL(last) && last!='+')
			{
				q[-1]='.';
				last_dot=q-1;
			}
			else
			{
				*q++='.';
				last_dot=q;
			}
			continue;
		}
		/* 
		 * Auseinanderfolgende Sonderzeichen vermeiden, mit Ausnahme 
		 * von "++" wie in "c++" 
		 */
		if (IS_SPECIAL(c) && IS_SPECIAL(last) && c!='+' && last!='+')
			continue;
#ifdef SKIP_TO_DOT
		if (skip_to_dot)
				continue;
#endif
		*q++ = c;
	}
	while (q > to && q[-1] == '.')
		*(--q) = '\0';
	*q = '\0';
	free (buf);
	return TRUE;
}

#if 0

Alte Version

static void
convert_groupname (unsigned char *unconverted, unsigned char *to)
{
	unsigned char *p;
	unsigned char *q;

	if ((p = get_rfcname (opts.pollsite, unconverted)) != NULL)
	{
		strcpy (to, p);
	}
	else
	{
		sprintf (to, "%s.%s.", opts.gprefix, opts.pollsite);
		p = unconverted;
		q = to + strlen (to);

		while (*p)
		{
			if (isupper (*p))
				*q++ = tolower (*p);
			else if (islower (*p) || *p == '.')
				*q++ = *p;
			else
				*q++ = '_';
			p++;
		}
		put_mausname (unconverted, opts.pollsite, to, NULL);
	}
}
#endif
