/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1994 Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* 
	lib/growio.c: i/o to/from buffers of varying size 
	Vereinfacht die Handhabung von Mail und News durch selbstaendige
	Vergroesserung der Puffer. Einziges echtes Limit ist das
	32K-Limit in growprintf, das die Anzahl der Zeichen beschraenkt,
	die mit einem Mal geschrieben werden koennen.
*/

#include "ownstd.h"
#include <errno.h>
#include <ctype.h>
#include <stdarg.h>

#include "mausconlib.h"

/*
 * Einen Text an den Buffer anhaengen, diesen bei Bedarf vergroessern.
 * Maximal 32K koennen auf einmal geschrieben werden.
 */
int
growprintf (unsigned char **buffer, unsigned char **aktpos, size_t * bufsize,
			size_t grow, unsigned char *format,...)
{
	va_list ap;
	char buf[32768];
	int len;
	size_t used;

	va_start (ap, format);
	len = vsprintf (buf, format, ap);
	va_end (ap);

	used = *aktpos - *buffer;
	if (used + len >= *bufsize)
	{
		*buffer = realloc (*buffer, *bufsize + grow);
		if (!*buffer)
		{
			log (LOG_CRIT, "Out of memory in %s %u",
				 __FILE__, __LINE__);
			exit (1);
		}
		*aktpos = *buffer + used;
		*bufsize += grow;
	}
	strcpy (*aktpos, buf);
	*aktpos += len;
	return len;
}
