#include "ownstd.h"

#include <stdarg.h>
#include <errno.h>
#include <ctype.h>

#include "mausconlib.h"
#include "maus.h"

static minfo_t *anker;
static int dirty;

int info_read_list(void)
{
	char fname[PATH_MAX];
	char str[1024];
	minfo_t *new;
	FILE *f;
	sprintf(fname,"%s/%s/etc/info.list",opts.spoolbase,action);
	f=fopen(fname,"r");
	if (!f)
	{
		log(LOG_WARNING,"cannot load %s",fname);
		return 0; /* naja, nur ein kleinerer Fehler! */
	}
	new=NULL;
	while (fgets (str, sizeof(str), f))
	{
		char *p=strpbrk(str,"\r\n");
		if (p)
			*p='\0';
		switch(*str)
		{
		case '#':
			if (new)
			{
				info_add_new(new);
				new=NULL;
			}
			if (str[1])
			{
				new=calloc(1,sizeof(minfo_t));
				if (!new)
				{
					log(LOG_ERR,
						"out of memory while loading infofiles");
					fclose(f);
					return 1;
				}
				new->kuerzel=strdup(str+1);
				if (!new->kuerzel)
				{
					log(LOG_ERR,
						"out of memory while loading infofiles");
					fclose(f);
					return 1;
				}
			}
			break;
		case ':':
			if (!new)
				break;
			new->kurztext=strdup(str+1);
			if (!new->kurztext)
			{
				log(LOG_ERR,"out of memory while loading infofiles");
				fclose(f);
				return 1;
			}
			break;
		case 'F': /* Flagszeile */
			if (!new)
				break;
			if (maus_get_tauschflag(str+1,'C')=='+')
				new->needs_crc=1;
			else
				new->needs_crc=0;
			break;
		case 'f': /* lokale Flagzeile fr Mausconnect */
			if (!new)
				break;
			if (maus_get_tauschflag(str+1,'d')=='+') /* Diff senden? */
				new->send_diff=1;
			else
				new->send_diff=0;
			if (maus_get_tauschflag(str+1,'m')=='+') /* Mail senden? */
				new->send_mail=1;
			else
				new->send_mail=0;
			if (maus_get_tauschflag(str+1,'b')=='+') /* bestellt? */
				new->subscribed=1;
			else
				new->subscribed=0;
			break;
		case 'c': /* letzte CRC */
			if (!new)
				break;
			new->last_crc=strtol(str+1,NULL,10);
			break;
		case 'p': /* Pause in Tagen mal letzten Erhalten des Infofiles */
			if (!new)
				break;
			new->pause=strtol(str+1,NULL,10);
			break;
		case 'd': /* Datum des letzten Eintreffens */
			if (!new)
				break;
			new->last_date=(time_t) strtol(str+1,NULL,10);
			break;
		}
	}
	fclose(f);
	if (new)
	{
		info_add_new(new);
	}
	return 0;
}

minfo_t *info_get_anker(void)
{
	return anker;
}

int info_write_list(void)
{
	char fname[PATH_MAX];
	minfo_t *akt;
	FILE *f;
	if (!anker)
		return 0;
	if (!dirty)
		return 0;
	sprintf(fname,"%s/%s/etc/info.list",opts.spoolbase,action);
	f=fopen(fname,"w");
	if (!f)
	{
		log(LOG_WARNING,"cannot write %s",fname);
		return 0; /* naja, nur ein kleinerer Fehler! */
	}
	akt=anker;
	while(akt)
	{
		fprintf(f,"#%s\n",akt->kuerzel);
		if (akt->kurztext)
			fprintf(f,":%s\n",akt->kurztext);
		if (akt->needs_crc)
			fprintf(f,"FC+\n");
		else
			fprintf(f,"FC-\n");
		fprintf(f,"c%ld\n",akt->last_crc);
		fprintf(f,"p%ld\n",akt->pause);
		fprintf(f,"d%ld\n",akt->last_date);
		fprintf(f,"fm%cd%cb%c\n",
			(akt->send_mail)? '+':'-',
			(akt->send_diff)? '+':'-',
			(akt->subscribed)? '+':'-');
		akt=akt->next;
	}
	fclose(f);
	return 1;
}

void info_add_new(minfo_t *new)
{
	new->next=anker;
	anker=new;
	info_modified(new);
}

minfo_t *info_find(char *kuerzel)
{
	minfo_t *il;

	il=anker;
	while (il)
	{
		if (!strcasecmp(il->kuerzel,kuerzel))
			return il;
		il=il->next;
	}
	/* nicht gefunden */
	return NULL;
}

void info_new_crc(char *kuerzel, long crc)
{
	minfo_t *new,*il;

	il=info_find(kuerzel);
	if (il)
	{
		if (crc!=il->last_crc)
		{
			/* 
			 * wenn sich die CRC verndert hat, dann das Datum der
			 * letzten Bestellung ndern.
			 */
			il->last_crc=crc;
			il->last_date=time(NULL);
			dirty=1;
		}
		return;
	}
	/* nicht gefunden */

	/* 
	 * Also haben wir eine CRC fr ein unbekanntes und nicht bestelltes
	 * Infofile bekommen - merkwrdig, aber wahr. (denkbar ist, da
	 * jemand das Infile editiert hat)
	 */
	
	new=calloc(1,sizeof(minfo_t));
	if (!new)
		return;
	new->kuerzel=strdup(kuerzel);
	if (!new->kuerzel)
	{
		free(new);
		return;
	}
	new->last_crc=crc;    		/* crc auslesen */
	new->pause=0;			/* Annahme: Es soll bestellt werden */
	new->subscribed=1;
	new->last_date=time(NULL);	/* Datum des letzten Erhalts */
	info_add_new(new);
}

/*
 * dirty-Flag setzen, gewisse Defaults setzen
 */
void info_modified(minfo_t *mod)
{
	if (!mod->needs_crc)
	{
		mod->send_mail=1;
		mod->send_diff=0;
	}
	dirty=1;
}
