/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1994 Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   Die Verwendung der Routinen dieses Moduls im TOM Mailboxsystem in
   gelinkter Form ist ausdrcklich erlaubt.

*/

/* 
 	itb.c: Handhabung einer Box/Domainliste, Format ITB.
*/

#include "ownstd.h"
#include <errno.h>
#include <ctype.h>
#include <stdarg.h>

#include "mausconlib.h"
#include "itb.h"
#include "xmalloc.h"

static itb_entry_t *anker;
static itb_entry_t *last;


static BOOLEAN itb_verketten(void);

void 
itb_append_box(itb_entry_t *n)
{
	if (!anker)
		anker=n;
	else
		last->next=n;
	n->next=NULL;
	last=n;
}

BOOLEAN
itb_read(const char *fname, const char *gsliste)
{
	FILE *f;
	char buf[1024];
	struct itb_entry_t *entry=NULL;
	struct itb_entry_t leer_entry=LEER_ITB_ENTRY;
	BOOLEAN had_domains=FALSE;

	f=fopen(fname,"r");
	if (!f)
	{
		log(LOG_NOTICE,"no %s",fname);
		return -1;
	}

	while (fgets(buf,sizeof(buf),f))
	{
		char *p;
		p=strpbrk(buf,"\r\n");
		if (p)
			*p='\0';
		strtrim(buf);
		if (!entry && *buf!='*')
			/* mll */
			continue;

		switch(*buf)
		{
		case '*':
			if (entry)
				itb_append_box(entry);
			entry=XMALLOC(sizeof(itb_entry_t));
			*entry=leer_entry;
			entry->mausnetkuerzel=XSTRDUP(buf+1);
			entumlauten(entry->mausnetkuerzel,buf);
			entry->kuerzel_umlautfrei=XSTRDUP(buf);
			break;
		case '#':
			entry->mausnetnummer=atoi(buf+1);
			break;
		case 'N':
			entry->boxname=XSTRDUP(buf+1);
			break;
		case '-':
			entry->serverkuerzel=XSTRDUP(buf+1);
			break;
		case 'T':
		case 't':
			{
				char tmp[1024];
				itb_tel_nr_t *t;
				itb_tel_nr_t *last;
				t=XMALLOC(sizeof(itb_tel_nr_t));
				if (!strncmp(buf+1,"00",2))
				{
					*tmp='+';
					strcpy(tmp+1,buf+3);
				}
				else
					sprintf(tmp,"+49%s",buf+2);
				t->nummer=XSTRDUP(tmp);
				t->geheim=(*buf=='t') ? 1 : 0;
				t->next=NULL;
				last=entry->nummern;
				while (last && last->next)
					last=last->next;
				if (last)
					last->next=t;
				else
					entry->nummern=t;
			}
			break;
		case 'D': /* Primrdomain */
		case 'Z': /* Zusatzdomain */
			had_domains=TRUE; /* ein ITB mit Domains darin */
			{
				itb_domain_t *d;
				char tmp[1024];
				d=XMALLOC(sizeof(itb_domain_t));
				d->techdom=XSTRDUP(buf+1);
				if (buf[1]=='.')
					sprintf(tmp,"%s%s",entry->kuerzel_umlautfrei,buf+1);
				else
					strcpy(tmp,buf+1);
				d->fulldom=XSTRDUP(tmp);
				if (*buf=='D')
				{
					d->next=entry->domains;
					entry->domains=d;
				}
				else if (entry->domains)
				{
					d->next=entry->domains->next;
					entry->domains->next=d;
				}
				else
				{
					d->next=NULL;
					entry->domains=d;
				}
			}
			break;
		case 'G': /* Gateways */
			{
				itb_gateway_t *g;
				g=XMALLOC(sizeof(itb_gateway_t));
				g->gateway=XSTRDUP(buf+1);
				g->next=entry->gateways;
				entry->gateways=g;
			}
			break;
		}
	}
	fclose(f);
	if (entry)
		itb_append_box(entry);
	if (had_domains)
	{
		/* alles OK */
		return itb_verketten();
	}

	/*
	 * Eine ITB ohne Domains ... kranke Sache, jetzt kommt Gnter Stcks
	 * Liste der Exportdomains.
	 */
	f=fopen(gsliste,"r");
	if (!f)
		return FALSE;
	while (fgets(buf,sizeof(buf),f))
	{
		itb_entry_t *akt;
		char *p;
		p=strpbrk(buf,"\r\n");
		if (p)
			*p='\0';
		strtrim(buf);
		if (!*buf)
			/* mll */
			continue;

		p=buf;
		p=strchr(buf,' ');
		if (p)
		{
			*p='\0';
			while (isspace(*p))
				p++;
			if (!*p)
				p=NULL;
			else if (strstr(p,"keine Domain"))
				p=NULL;
		}
		akt=anker;
		while (akt && strcasecmp(akt->mausnetkuerzel,buf))
			akt=akt->next;
		if (akt)
		{
			if (!p)
				akt->noexport_flag=TRUE;
			else
			{
				itb_domain_t *last=NULL;
				itb_domain_t *d;
				char *q=strpbrk(p,"\t ");
				if (q)
				{
					*q++='\0';
					while (isspace(*q))
						q++;
				}
				d=XMALLOC(sizeof(itb_domain_t));
				d->techdom=XSTRDUP(p);
				d->fulldom=XSTRDUP(p);
				d->next=NULL;
				akt->domains=d;
				last=d;
				/* nun zustzliche Domains anhngen */
				while (q && *q)
				{
					p=q;
					q=strpbrk(p,"\t ");
					if (q)
					{
						*q++='\0';
						while (isspace(*q))
							q++;
					}
					d=XMALLOC(sizeof(itb_domain_t));
					d->techdom=XSTRDUP(p);
					d->fulldom=XSTRDUP(p);
					d->next=NULL;
					last->next=d;
					last=d;
				}
			}
		}
		/* else dirt */
	}
	fclose(f);

	/* und nun alle anderen */
	{
		itb_entry_t *akt;
		itb_domain_t *d;

		for(akt=anker;akt;akt=akt->next)
		{
			char tmp[256];
			if (akt->domains || akt->noexport_flag)
				continue;
			sprintf(tmp,"%s.maus.de",akt->kuerzel_umlautfrei);
			d=XMALLOC(sizeof(itb_domain_t));
			d->techdom=XSTRDUP(tmp);
			d->fulldom=XSTRDUP(tmp);
			akt->domains=d;
			sprintf(tmp,"%s.maus.sub.org",akt->kuerzel_umlautfrei);
			d=XMALLOC(sizeof(itb_domain_t));
			d->techdom=XSTRDUP(tmp);
			d->fulldom=XSTRDUP(tmp);
			akt->domains->next=d;
			d->next=NULL;
		}
	}
	return itb_verketten();
}

itb_entry_t *
itb_find_mauskuerzel(const char *kuerzel)
{
	char tmp[256];
	itb_entry_t *akt;
	entumlauten(kuerzel,tmp);
	for(akt=anker;akt;akt=akt->next)
	{
		if (!strcasecmp(akt->kuerzel_umlautfrei,tmp))
			return akt;
	}
	return NULL;
}

itb_entry_t *
itb_find_domainname(const char *domainname)
{
	char tmp[1024];
	itb_entry_t *akt;
	entumlauten(domainname,tmp);
	for(akt=anker;akt;akt=akt->next)
	{
		itb_domain_t *d;
		d=akt->domains;
		for(d=akt->domains;d;d=d->next)
		{
			if (!strcasecmp(d->fulldom,tmp))
				return akt;
		}
		if (!akt->domains)
		{
			/* Boxen ohne Domains dennoch in .maus.de und .maus.sub.org
			 * finden.
			 */
			size_t l=strlen(akt->kuerzel_umlautfrei);
			if (!strcasecmp(akt->kuerzel_umlautfrei,tmp))
			{
				if (!strcasecmp(tmp+l,".maus.de")
					|| !strcasecmp(tmp+l,".maus.sub.org"))
					return akt;
			}
		}
	}
	return NULL;
}

static BOOLEAN
itb_verketten(void)
{
	itb_entry_t *akt;
	for (akt=anker;akt;akt=akt->next)
	{
		if (akt->serverkuerzel)
			akt->uplink=itb_find_mauskuerzel(akt->serverkuerzel);
	}
	return TRUE;
}

BOOLEAN
itb_hopcount_berechnen(itb_entry_t *startbox)
{
	int i;
	itb_entry_t *akt;
	/* Alle Flags lschen */
	for (akt=anker;akt;akt=akt->next)
		akt->mp_flag=0;
	/* Und Flags aufwrts setzen */
	for (akt=startbox,i=1;akt;akt=akt->uplink,i++)
		akt->mp_flag=i; /* hopcount + 1 */
	/* Durch alle Boxen gehen */
	for (akt=anker;akt;akt=akt->next)
	{
		itb_entry_t *found=NULL;
		if (akt->mp_flag)
			found=akt;
		else
		{
			for (found=akt->uplink; found && !found->mp_flag; 
				found=found->uplink)
				; /* leer */
		}
		if (found)
		{
			if (akt->hops==0 || akt->hops>found->mp_flag)
			{
				akt->route_ueber=found;
				akt->hops=found->mp_flag;
			}
		}
	}
	return TRUE;
}
