#ifndef ITB_H
#define ITB_H

/*
 * Liste der Telefonnummern, unter denen die Box erreichbar ist.
 * "geheim" ist gesetzt bei Nummern, die nicht fr jedermann sind.
 */
typedef struct itb_tel_nr_t
{
	char *nummer;
	int geheim;
	struct itb_tel_nr_t *next;
} itb_tel_nr_t;

/*
 * Liste der Domains, unter denen die Box bekannt ist. 
 * Zwei Werte pro Box:
 * - tatschlicher Wert (ac.maus.de), entumlautet.
 * - technischer Wert (.maus.de)
 */
typedef struct itb_domain_t 
{
	char *fulldom;
	char *techdom;
	struct itb_domain_t *next;
} itb_domain_t;

typedef struct itb_gateway_t 
{
	char *gateway;
	struct itb_gateway_t *next;
} itb_gateway_t;

typedef struct itb_entry_t
{
	char *primaername;            /* abgeleitet aus D und * */

	char *mausnetkuerzel;         /* * */
	char *kuerzel_umlautfrei;     /* * */
	char *serverkuerzel;          /* - */
	unsigned int mausnetnummer;   /* # */ 
	char *boxname;                /* N */
		/* 
		 * Erste ist Primaerdomain! 
		 */
	itb_domain_t *domains;        /* D, Z */
	itb_tel_nr_t *nummern;        /* tT */
	itb_gateway_t *gateways;      /* G */

		/*
		 * Nachrichten aus dieser Box knnen nicht exportiert werden 
		 */
	BOOLEAN noexport_flag;      

		/* 
		 * Zeiger auf die Box darber.
		 */
	struct itb_entry_t *uplink;

	struct itb_entry_t *route_ueber;
	unsigned int hops;

	int mp_flag; /* multipurpose-flag */

	struct itb_entry_t *next;
} itb_entry_t;

#define LEER_ITB_ENTRY { \
	NULL,NULL,NULL,NULL,0,NULL, \
	NULL,NULL,NULL, \
	FALSE,NULL,NULL,0,0,NULL \
	} 

void itb_append_box(itb_entry_t *n);
BOOLEAN itb_read(const char *fname, const char *gsliste);
itb_entry_t *itb_find_domainname(const char *domainname);
itb_entry_t *itb_find_mauskuerzel(const char *kuerzel);
/* BOOLEAN itb_verketten(void); */
BOOLEAN itb_hopcount_berechnen(itb_entry_t *startbox);



#endif
