/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1992 1993 1994 Michael Kraehe, Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
	lib/lib.c: Allgemeine Hilfsfunktionen.
*/

#include "ownstd.h"

#include <assert.h>
#include <ctype.h>
#include <fnmatch.h>
#include <errno.h>

#include "mausconlib.h"

int
strhead (const char *s1, const char *s2)
{
	return (!memcmp (s1, s2, strlen (s2)));
}

char *
strtrim (char *s)
{
	char *start;
	char *p;

	start = s;
	while (isspace (*s))
		s++;
	if (!*s)
	{
		*start = '\0';
		return start;
	}
	p = s + strlen (s) - 1;
	while (isspace (*p))
		*p-- = '\0';
	memmove (start, s, p - s + 1);
	return start;
}


int
is_allowed (const char *sender)
{
	char lcsender[1024];
	char *p;
	mc_list_t *akt;

	for (p=lcsender; *sender; sender++)
		*p++=tolower(*sender);
	*p='\0';

	if (!strhalfcasecmp(opts.mailuser,lcsender))
		return 1;
	if (!strhalfcasecmp(opts.mailuserdomain,lcsender))
		return 1;

	akt=opts.allowed;
	while (akt && akt->data)
	{
		/* naechster Versuch */
		if (!fnmatch (akt->data, lcsender, FNM_CASEFOLD))
			return 1;
		akt=akt->next;
	}
	return 0;
}

/* Optimiertes strcasecmp fr den Fall, da der zweite String
 * schon in Kleinschrift ist. Aus der GNUlib geklaut und modifiziert.
 */

int 
strhalfcasecmp(const char *uplow, const char *low)
{
	register const unsigned char *p1 = (const unsigned char *) uplow;
	register const unsigned char *p2 = (const unsigned char *) low;
	register int ret;
	unsigned char c1;

	if (p1 == p2)
		return 0;

	for (; !(ret = (c1 = tolower(*p1)) - (*p2)); p1++, p2++)
		if (c1 == '\0') break;
	return ret;
}

/* Optimiertes strncasecmp fr den Fall, da der zweite String
 * schon in Kleinschrift ist. Aus der GNUlib geklaut und modifiziert.
 */

int 
strnhalfcasecmp(const char *uplow, const char *low, size_t n)
{
	register const unsigned char *p1 = (const unsigned char *) uplow;
	register const unsigned char *p2 = (const unsigned char *) low;
	register int ret;
	unsigned char c1;

	if (p1 == p2)
		return 0;

	for (; n--; p1++, p2++) 
	{
		c1=tolower(*p1);
		ret=(c1 - *p2);
		if (ret) 
			return ret;
		if (c1 == '\0') break;
	}
	return (0);

}

void
convert_umlaute (unsigned char *s)
{
	unsigned char *sb = malloc (strlen (s) + 1);
	unsigned char *p;

	if (!sb)
	{
		p = s;
		while (*p)
		{
			if (!isascii (*p))
				*p = '_';
			p++;
		}
		return;
	}
	strcpy (sb, s);
	while (*sb)
	{
		switch ((int) *(char *) sb)
		{
		case '':
			*s++ = 'a';
			*s++ = 'e';
			break;
		case '':
			*s++ = 'A';
			*s++ = 'e';
			break;
		case '':
			*s++ = 'o';
			*s++ = 'e';
			break;
		case '':
			*s++ = 'O';
			*s++ = 'e';
			break;
		case '':
			*s++ = 'u';
			*s++ = 'e';
			break;
		case '':
			*s++ = 'U';
			*s++ = 'e';
			break;
		case '':
			*s++ = 's';
			*s++ = 's';
			break;
		default:
			*s++ = *sb;
			break;
		}
		sb++;
	}
	*s = '\0';
}

int parse_bool(char *s)
{
    char c=tolower(*s);
    if (c=='y' || c=='j' || c=='t')
        return 1;
    else if (c=='n' || c=='f')
        return 0;
    else
        return (strtol(s+1,NULL,10)!=0);
}


BOOLEAN entumlauten(const char *from, char *to)
{
	BOOLEAN umlaut=FALSE;
	while (*from)
	{
		switch(*from)
		{
		case '': 
			umlaut=TRUE;
			*(to++)='U';
			if (isupper(from[1]))
				*(to++)='E';
			else
				*(to++)='e';
			break;
		case '':
			umlaut=TRUE;
			*(to++)='u';
			*(to++)='e';
			break;
		case '': 
			umlaut=TRUE;
			*(to++)='A';
			if (isupper(from[1]))
				*(to++)='E';
			else
				*(to++)='e';
			break;
		case '':
			umlaut=TRUE;
			*(to++)='a';
			*(to++)='e';
			break;
		case '':
			umlaut=TRUE;
			*(to++)='O';
			if (isupper(from[1]))
				*(to++)='E';
			else
				*(to++)='e';
			break;
		case '':
			umlaut=TRUE;
			*(to++)='o';
			*(to++)='e';
			break;
		case '':
			umlaut=TRUE;
			*(to++)='s';
			*(to++)='s';
			break;
		default:
			*(to++)=*from;
			break;
		}
		from++;
	}
	*to='\0';
	return umlaut;
}

FILE *
xfopen(const char *fname, const char *mode)
{
	FILE *f;
	f=fopen(fname,mode);
	if (!f)
	{
		/* es knnte jemand an der Datei herumbasteln, z.B. 
		 * mv datei old
		 * mv neu datei
		 */
		sleep(2);
		f=fopen(fname,mode);
	}
	if (!f)
	{
		static BOOLEAN no_recurse=FALSE;
		if (no_recurse)
			fprintf(stderr,"kann %s nicht ffnen: %s\n",fname,strerror(errno));
		else
		{
			no_recurse=TRUE;
			log(LOG_ERR,"kann %s nicht ffnen: %s",fname,strerror(errno));
		}
		exit (1);
	}
	return f;
}	

