/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1993 1994 Michael Kraehe, Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
	lib/log.c: Logmeldungen schreiben.
	Zustand: Koennte eine Ueberarbeitung gebrauchen.
*/
#include "ownstd.h"

#include <stdarg.h>
#include <errno.h>
#if defined (HAVE_SYSLOG) && defined (HAVE_SYSLOG_H)
#define USE_SYSLOG
#include <syslog.h>
#endif

#include "mausconlib.h"

static FILE *msglogfile;

#ifdef USE_SYSLOG
static int syslog_open;
#else
static FILE *logfile;
#endif

static void
open_msglog(void)
{
	char fname[PATH_MAX];
	if (!strcasecmp(action,"route"))
		sprintf(fname,"%s/routelog",opts.spoolbase);
	else
		sprintf(fname,"%s/%s/msglog",opts.spoolbase,action);
	msglogfile=fopen(fname,"a");
	if (!msglogfile)
	{
		log(LOG_ERR,"kann %s nicht oeffnen (%s)",
			fname,strerror(errno));
		exit(1);
	}
}
static void 
close_msglog(void)
{
	if (msglogfile)
	{
		fclose(msglogfile);
		msglogfile=NULL;
	}
}

void
msglog(char *format,...)
{
	va_list ap;
	time_t t;
	struct tm *tm;
	char str[128];
	FILE *out;

	if (!msglogfile)
		out = stderr;
	else
		out = msglogfile;
	va_start (ap, format);
	t = time (NULL);
	tm = localtime (&t);
	strftime (str, sizeof (str) - 1, "%d.%m.%y %R", tm);
	fprintf (out, "%s: ", str);
	vfprintf (out, format, ap);
	fprintf (out, "\n");
	fflush (out);
	va_end (ap);
}

void
open_log (char *progname, BOOLEAN with_msglog)
{
#ifndef USE_SYSLOG
	if (opts.logmode != LOGMODE_SYSLOG)
	{
		char str[PATH_MAX];

		sprintf (str, "%s/%s/LOG", opts.spoolbase, action);
		logfile = fopen (str, "a");
		if (!logfile)
		{
			int e1,
			 e2;

			e1 = errno;
			sprintf (str, "%s/LOG", opts.spoolbase);
			logfile = fopen (str, "w");
			e2 = errno;
			/* egal ob es geklappt hat oder nicht, log() klappt immer, und sei
			   es auch auf stderr
			 */
			log (LOG_ERR, "cannot open %s/%s/LOG, %s",
				 opts.spoolbase, action, strerror (e1));
			if (!logfile)
			{
				log (LOG_CRIT, "cannot open %s/LOG, %s",
					 opts.spoolbase, strerror (e2));
			}
		}
	}
#else /* else if defined USE_SYSLOG */
	openlog (progname, 0, opts.sl_facility);
	syslog_open = 1;
#endif
	if (with_msglog)
		open_msglog();
}

void
close_log (void)
{
#ifdef USE_SYSLOG
	if (syslog_open)
		closelog ();
#else
	if (logfile)
	{
		fclose (logfile);
		logfile = NULL;
	}
#endif
	close_msglog();
}

void
log (int priority, char *format,...)
{
	va_list ap;
#ifndef USE_SYSLOG
	time_t t;
	struct tm *tm;
	char str[128];
	FILE *out;

	if (!logfile)
		out = stderr;
	else
		out = logfile;
	va_start (ap, format);
	t = time (NULL);
	tm = localtime (&t);
	strftime (str, sizeof (str) - 1, "%d.%m.%y %R", tm);
	fprintf (out, "%s: %d ", str, priority);
	vfprintf (out, format, ap);
	fprintf (out, "\n");
	fflush (out);
#else
	va_start (ap, format);
	vsyslog (priority, format, ap);
#endif
	/* Kopie der Nachricht nach stderr. Eventuell mal konfigurierbar machen */
	vfprintf (stderr, format, ap);
	fprintf (stderr, "\n");
	va_end (ap);
}
