/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1992 1993 1994 Michael Kraehe, Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*  
 * lib/maus.c: Allgemeine Hilfsfunktionen fuer den MausTausch.
 */

#include "ownstd.h"

#include <assert.h>
#include <ctype.h>
#include "mausconlib.h"
#include "metamsg.h"
#include "maus.h"
#include "xmalloc.h"

#include "groups.h"
#ifdef FEATURE_NNTP
#include "nntp.h"
#endif

#define LINEEND "\r\n"
#define MAUS_STRLEN	(256*2+2)

	/* Conditional Out Of Memory */
#define COOM(x); if (!x) { \
	log (LOG_CRIT, "out of memory working on %s",#x); \
	exit(1); \
	}

static void maus_mk_ref (metamsg_t * msg, char *mref, char *realref);
static size_t export_part (FILE * f, metamsg_t * meta, char *part);

#ifdef FEATURE_NNTP
static int maus_nntp_lookup_list (const char *id, char *kennung,
								  metalist_t ** what);

#endif

void
maus_convert_adr (char *mausadr, char *adress, char *realname)
{
	char domain[1024];
	char *p = mausadr;
	char *pud = realname;		/* Pointer User Domain */
	int in_user = 1;

	domain[0] = '\0';

	/* Erster Teil: In Local- und Domainpart zerlegen */
	while (*p)
	{
		switch (*p)
		{
		case '@':
			*pud = '\0';
			pud = domain;
			in_user = 0;
			break;
		case ' ':
			/* mehrere Leerzeichen zusammenziehen */
			if (p[1] == ' ')
				break;
			if (p[1] != '@' && p[-1] != '@')
				*pud++ = *p;
			break;
		default:
			*pud++ = *p;
			break;
		}						/* switch *p */
		p++;
	}							/* while *p */
	if (pud)
		pud[0] = '\0';			/* String terminieren */
	/* 
	 * So, jetzt haben wir zwei Strings darin. Der erste ist der Localpart,
	 * mit Leerzeichen darin. Der zweite der Domainpart, mglicherweise 
	 * ebenfalls mit Leerzeichen darin, was auf Fido-hnliche Adressierung
	 * hindeutet.
	 * In jedem Fall wird der Domainpart entumlautet.
	 */
	convert_umlaute (domain);
	if (strstr (domain, "Fido "))
	{
		/* Tja, "Fido ..." */
		char *p;
		char help[1024];

		/* Fido 2:245/604.15 -> p15.f604.n245.z2.fidonet.org */
		strcpy (help, domain);
		*domain = 0;
		p = strchr (help, '.');
		if (p)
		{
			sprintf (domain, "p%s.", p + 1);
			*p = 0;
		}
		p = strchr (help, '/');
		if (p)
		{
			sprintf (domain + strlen (domain), "f%s.", p + 1);
			*p = 0;
		}
		/* Fido 2:245/604.15 oder Fido 245/604.15 */
		p = strchr (help, ':');
		if (p)
		{
			sprintf (domain + strlen (domain), "n%s.", p + 1);
			*p = 0;
		}
		else
		{
			/* Keine Zone angegeben ... */
			/* Fido 245/604.15 */
			p = strchr (help, ' ');
			if (p)
			{
				sprintf (domain + strlen (domain), "n%s.z2.", p + 1);
				*p = 0;
			}
		}
		p = strchr (help, ' ');
		if (p)
		{
			sprintf (domain + strlen (domain), "z%s.", p + 1);
			*p = 0;
		}
		sprintf (domain + strlen (domain), "fidonet.org");
	}							/* if "Fido" in domain */
	/* 
	 * So, jetzt haben wir eine gltige Domain.
	 * Jetzt die Adresse zusammensetzen ...
	 */
	sprintf (adress, "%s@%s", realname, domain);
	/* und Leerzeichen durch Spaces ... */
	p = adress;
	while (*p)
	{
		if (*p == ' ')
			*p = '_';
		p++;
	}
	/* und Umlaute durch Langform ersetzen. */
	convert_umlaute (adress);

	/* Eventuell fehlende Domain ersetzen */
	if (!strchr (domain, '.'))
		strcpy (adress + strlen (adress), opts.mausdomain);

	/* Ein Username ohne Spaces knnte angegeben worden sein, in dem
	 * Fall gibt es keinen Realnamen 
	 */
	if (!strchr (realname, ' '))
		realname[0] = '\0';
}

char
maus_get_tauschflag (char *s, char flag)
{
	char *p = s;

	while (p)
	{
		if (*p == flag)
			return p[1];
		if (!p[1])
			return '\0';
		p += 2;
	}
	return '\0';
}

time_t
maus_convert_tauschdate (char *s)
{
	char zwi[5];
	struct tm tm;
	size_t l;

	memset (&tm, 0, sizeof (struct tm));

	l = strlen (s);

	if (l >= 4)
	{
		memcpy (zwi, s, 4);
		zwi[4] = '\0';
		tm.tm_year = strtol (zwi, NULL, 10) - 1900;
	}

	memcpy (zwi, s + 4, 2);

	if (l >= 6)
	{
		zwi[2] = '\0';
		tm.tm_mon = strtol (zwi, NULL, 10) - 1;
	}

	if (l >= 8)
	{
		memcpy (zwi, s + 6, 2);
		tm.tm_mday = strtol (zwi, NULL, 10);
	}

	if (l >= 10)
	{
		memcpy (zwi, s + 8, 2);
		tm.tm_hour = strtol (zwi, NULL, 10);
	}

	if (l >= 12)
	{
		memcpy (zwi, s + 10, 2);
		tm.tm_min = strtol (zwi, NULL, 10);
	}

	if (l >= 14)
	{
		memcpy (zwi, s + 12, 2);
		tm.tm_sec = strtol (zwi, NULL, 10);
	}
	return mktime (&tm);
}

static char *
last_reference (metalist_t * r)
{
	if (!r)
		return NULL;
	while (r->next)
		r = r->next;
	return r->text;
}


static void
write_message_refs (FILE * f, metamsg_t * meta, int ww, metalist_t * ref_gruppe)
{
	char mref[MAUS_STRLEN];
	char rref[MAUS_STRLEN];
	char *ref;

	ref = last_reference (meta->references);
	if (!ref || !strchr (ref, '@'))
		return;
	maus_mk_ref (meta, mref, rref);

	if (ww)
	{
		grp_data_t *grp;

		if (ref_gruppe == NULL)
		{
			/* 
			 * Kommentar auf persnliche Nachricht. 
			 * Falls in der History eine Maus-ID ist (sollte zwar meist
			 * der Fall sein, mu aber nicht notwendigerweise so sein),
			 * wird diese verwendet, sonst "ref")
			 */
			fprintf (f, ":Kommentar zu %s" LINEEND, mref);
			return;
		}
		/* Kommentar auf ffentliche Nachricht */
		grp = NULL;
		while (!grp && ref_gruppe)
		{
			grp = grp_suche_rfcname (ref_gruppe->text,opts.pollsite);
			ref_gruppe = ref_gruppe->next;
		}
		if (grp)
		{
			fprintf (f, ":Kommentar zu %s in der Gruppe %s" LINEEND,
					 mref, grp->mausname);
		}
		else
			fprintf (f, ":Kommentar zu %s" LINEEND, mref);
		return;					/* Fertig mit Wildwestverkettung */
	}

	if (opts.boxtyp==BOXTYP_QUARK_II)
		fprintf (f, "-%s" LINEEND, ref);
	else /* weniger fhige Boxtypen */
	{
		if (*mref)
		{
			/* Eine mausinterne References konnte ermittelt werden */
			fprintf (f, "-%s" LINEEND, mref);
		}
		if (*rref)
		{
			/* Und eine echte ebenso */
			fprintf (f, "R%s" LINEEND, rref);
		}						/* Usermodus? */
	}
}

static void
write_message_ids (FILE * f, metamsg_t * meta, char *part)
{
	if (opts.workmode == USER)
	{
		/* 
		 * Als Benutzer sind keine echten Message-IDs erlaubt, man mu
		 * temporre IDs vergeben. Die Mausprogrammierer halten das 
		 * auch noch fr sinnvoll, angeblich wre Chaos die Folge,
		 * drften Benutzer IDs selbst vergeben. Aber welchen Sinn 
		 * hat es, eine *eindeutige* Message-ID (und die erzeugen 
		 * Mail- und Newssysteme nunmal) wegzuwerfen, um eine
		 * neue, weniger eindeutige, zu erzeugen?
		 * IMO eine vllig berzogene Rcksichtnahme auf veraltete
		 * Systeme, zumindest fr die sicheren Systeme sollte da
		 * mehr erlaubt sein.
		 */
		/* Message-ID */
		fprintf (f, "#%s%s" LINEEND, meta->transfer_id, part);

	}							/* Usermodus? */
	else
	{
		char buf[512];
		/* Das funktioniert so nur an der Quark: */
		strcpy(buf,meta->message_id);
		if (part && *part)
		{
			char *p;
			p=strrchr(buf,'@');
			if (p)
				*p='%';
			strcat(buf,"@splitted-for-mausnet-part-");
			strcat(buf,part);
		}
		if (opts.boxtyp==BOXTYP_QUARK_II)
			fprintf (f, "#%s" LINEEND, buf);
		else
		{
			fprintf (f, "#%s%s" LINEEND, meta->transfer_id, part);
			fprintf (f, "I%s" LINEEND, buf);
		}
	}							/* User- oder Gatemodus */
}

#ifdef FEATURE_NNTP
static int
maus_nntp_lookup_list (const char *id, char *kennung,
					   metalist_t ** what)
{
	char *p;
	char *s;
	char c;

	s = nntp_get_headerentry (id, kennung);
	if (!s)
		return 1;

	while ((p = strpbrk (s, " \t,")) != NULL)
	{
		c = *p;
		*p = '\0';
		if (*s)
		{
			if (!meta_add_list (what, s, NULL, 0))
			{
				log (LOG_CRIT, "out of memory working on %s",kennung);
				exit(1);
			}
		}
		*p++ = c;

		while (*p == ',' || isspace (*p))
			p++;
		s = p;
	}
	if (*s)
	{
		if (!meta_add_list (what, s, NULL, 0))
		{
			log (LOG_CRIT, "out of memory working on %s",kennung);
			exit(1);
		}
	}
	return 0;
}
#endif

static size_t
export_part (FILE * f, metamsg_t * meta, char *part)
{
	unsigned char zwi[1024];
	struct tm *tm;

	int is_crossposting = 0;
	int is_mail = 0;
	size_t textsize = 0;
	int used_cludges = 0;
	char *ref;
	metalist_t *groups = NULL;
	int need_ww = 0;

	if (meta->references)
		ref = last_reference (meta->references);
	else
		ref = NULL;

	/* Einen Header plus eventuell Pseudoheader in die Datei schreiben */

	/*
	 * Die Message-ID schreiben.
	 */
	write_message_ids (f, meta, part);

	if (ref)
	{
#ifdef FEATURE_NNTP
		if (!maus_nntp_lookup_list (ref, "newsgroups", &groups))
		{
			metalist_t *akt = groups;
			metalist_t *this;;

			need_ww = 1;
			if (groups && !meta->is_mail)
			{
				while (akt && need_ww)
				{
					this = meta->newsgroups;
					while (this && need_ww)
					{
						if (!strcasecmp (akt->text, this->text))
							need_ww = 0;
						this = this->next;
					}
					akt = akt->next;
				}
			}
			else if (!groups)
			{
				/* PM auf PM? */
				need_ww = 0;
			}
			/* else persnlicher Kommentar auf ffentliche Nachricht */
		}
#else
		/* Haha - ohne Message-ID-DB ist das ganz schn schwierig */
#endif
	}

	if (meta->env_to)
	{
		/* Dann ist es eine Mail */
		is_mail = 1;

		/* Empfngerangabe schreiben.
		 * Grrr - Quark 1.x kann z.Z. keinen Unterstrich 
		 */
		if ((opts.boxtyp == BOXTYP_QUARK_I) && meta->head_to)
		{
			char *at;

			at = strchr (meta->env_to, '@');
			if (!at)
				fprintf (f, "A%s" LINEEND, meta->env_to);
			else
			{
				if (meta->head_to->ext)
					fprintf (f, "A%s@%s" LINEEND, meta->head_to->ext, at + 1);
				else
				{
					char *ul = strchr (meta->env_to, '_');

					if (ul && ul < at)
					{
						*ul = '\0';
						fprintf (f, "A%s %s" LINEEND, meta->env_to, ul + 1);
						*ul = '_';
					}
					else
						fprintf (f, "A%s" LINEEND, meta->env_to);
				}
			}
		}
		else
			fprintf (f, "A%s" LINEEND, meta->env_to);

		if (!need_ww)
			write_message_refs (f, meta, need_ww, groups);
	}
	else if (!meta->newsgroups)
	{
		/* ### LOG-Eintrag fehlt */
		return (size_t) - 1;
	}
	else
	{
		/* ffentliche Nachricht */
		metalist_t *ng;
		grp_data_t *grp;
		BOOLEAN grp_found=FALSE;

		ng = meta->newsgroups;
		/* Eine Gruppe kann auf jeden Fall gewandelt werden, das ist
		 * weiter oben schon geprft worden.
		 */
		while (ng)
		{
			grp = grp_suche_rfcname (ng->text,opts.pollsite);
			if (grp)
			{
				fprintf (f, "G%s" LINEEND, grp->mausname);
				grp_found=TRUE;
				if (!opts.send_crosspostings)
					break;
			}
			ng = ng->next;
		}
		if (ng && ng->next)
			/* spter Textcludge erzeugen */
			is_crossposting = 1;
		if (!grp_found)
		{
			/* Das kann zwar in rfc2maus *hier* nicht passieren, aber 
			 * mglicherweise in irgendwelchen Filtern, die die
			 * Gruppenliste nicht laden.
			 */
			fprintf (f, "G%s" LINEEND, meta->newsgroups->text);
		}

		if (!need_ww)
			write_message_refs (f, meta, need_ww, groups);

		if (meta->followup_to && opts.send_followup_to)
		{
			metalist_t *ng;
			BOOLEAN found=FALSE;

			ng = meta->followup_to;
			while (ng)
			{
				if (!strcasecmp (ng->text, "poster"))
				{
					fprintf (f, "Fposter"LINEEND);
					if (!opts.send_crosspostings)
						break;
				}
				else
				{
					grp_data_t *grp;
					grp=grp_suche_rfcname(ng->text,opts.pollsite);
					if (grp)
					{
						fprintf (f, "F%s" LINEEND, grp->mausname);
						found=TRUE;
					}
					else if (opts.send_crosspostings)
					{
						fprintf (f, "F%s" LINEEND, ng->text);
						found=TRUE;
					}
				}
				ng = ng->next;
			}
			/* Wenn berhaupt nichts bekannt war, dann eben so ... */
			if (!found)
				fprintf (f, "F%s" LINEEND, meta->followup_to->text);
		}
	}

	/* User drfen keine Absenderangabe verschicken */
	if (opts.workmode != USER)
	{
		/* Absender angegeben? */
		/* Just for the case */
		if (meta->env_from && meta->is_mail)
		{
			fprintf (f, "V%s" LINEEND, meta->env_from);
			if (meta->head_from && meta->head_from->text 
				&& !strcasecmp(meta->head_from->text,meta->env_from))
			{
				if (meta->head_from->ext)
					fprintf (f, "N%s" LINEEND, meta->head_from->ext);
			}
		}
		else if (meta->head_from)
		{
			fprintf (f, "V%s" LINEEND, meta->head_from->text);
			if (meta->head_from->ext)
				fprintf (f, "N%s" LINEEND, meta->head_from->ext);
		}
		else if (meta->sender)
		{
			fprintf (f, "V%s" LINEEND, meta->sender->text);
			if (meta->sender->ext)
				fprintf (f, "N%s" LINEEND, meta->sender->ext);
		}
	}
	/* Datumsangabe erzeugen */
	tm = localtime (&meta->date);
	strftime (zwi, sizeof (zwi) - 1, "%Y%m%d%H%M", tm);
	fprintf (f, "E%s" LINEEND, zwi);

	/*
	 * Subject
	 *
	 */
	{
		size_t subjmax;			/* maximale Laenge des Subjects */
		char *p;

		if (meta->subject)
			strcpy (zwi, meta->subject);
		else
			strcpy (zwi, "Kein Betreff angegeben");
		/* 
		 * Nach Umlautwandlung darf der Betreff in der MAUS hchstens 30
		 * Zeichen lang sein, sonst wird die Nachricht abgelehnt.
		 * Hier knnte man zwischen Quarklokalen und MausNet-Gruppen
		 * unterscheiden, aber dazu ist mir das nicht wichtig genug.
		 * Bei Quarks ist keine Umlautwandlung und keine Krzung 
		 * notwendig, Quark kann das selbst. Allerdings sollte man
		 * freundlicherweise re's killen, speziell bei Quark 1 mit
		 * dem Maximum von 28 Zeichen!
		 */
		if (opts.boxtyp != BOXTYP_MAUS)
		{
			if (opts.boxtyp == BOXTYP_QUARK_I)
				subjmax = 28;
			else
				/* Quark II hat keine Probleme mit langen Betreffs */
				subjmax = strlen (zwi);
		}
		else
		{
			/* MAUS-Standardverhalten simulieren. */
			/* Effektive erlaubte Laenge holen */
			subjmax = 30;
			if (opts.subjectlen > subjmax)
				subjmax = opts.subjectlen;
			else
				convert_umlaute (zwi);
		}
		/* Re: eventuell killen */
		if (opts.rekillmode == REKILL_NONE)
			/* nicht killen */
			p = zwi;
		else if (opts.rekillmode == REKILL_FULL)
		{
			/* immer killen */
			if (!strncasecmp (zwi, "re: ", 4))
				p = zwi + 4;
			else
				p = zwi;
		}
		else
			/* REKILL_SMART */
		{
			/* nur killen, wenn das Maximum berschritten wrde */
			if (!strncasecmp (zwi, "re: ", 4))
			{
				if (strlen (zwi) > subjmax)
					p = zwi + 4;
				else
					p = zwi;
			}
			else
				p = zwi;
		}
		/* Maus kann noch nicht mal selbst den Betreff krzen :-) */
		if (strlen (p) > subjmax && opts.boxtyp == BOXTYP_MAUS)
			p[subjmax] = '\0';
		fprintf (f, "W%s" LINEEND, p);
	}

	/* Distribution in D-Zeile senden? */
	if (opts.send_dzeile && meta->distribution)
	{
		/* World bzw. (net) ist Default */
		if (!strcasecmp (meta->distribution, opts.usenet_dist_mausnet)
			|| !strcasecmp (meta->distribution, "maus"))
			fprintf (f, "%s" LINEEND,opts.mausnet_dist_mausnet);
		else if (!strcasecmp (meta->distribution, opts.pollsite)
			|| !strcasecmp(meta->distribution,opts.usenet_dist_local))
			fprintf (f, "%s" LINEEND,opts.mausnet_dist_local);
	}

	if (opts.workmode!=USER && meta->organization)
		fprintf (f, "O%s" LINEEND, meta->organization);

	if (opts.send_path && meta->path)
	{
		if (strlen(meta->path)>250)
		{
			char *p;
			p=meta->path+250;
			while (p>meta->path && *p!='!')
				p--;
			if (p>meta->path)
			{
				*p='\0';
				fprintf(f,"p%s"LINEEND,meta->path);
				*p='!';
			}
		}
		else
			fprintf(f,"p%s"LINEEND,meta->path);
	}
	if (meta->reply_to && opts.send_reply_to)
	{
		/* nur *ein* Reply-To, mehr ist wohl nicht sinnvoll, oder? */
		fprintf(f,"T%s"LINEEND,meta->reply_to->text);
	}
	if (meta->sender && opts.send_sender)
	{
		fprintf(f,"S%s"LINEEND,meta->sender->text);
	}

	/* 
	 * Nun Textcludges erzeugen.
	 */
	/* als erster *muss* die Distribution kommen */
	if (!opts.send_dzeile && meta->distribution)
	{
		/* World bzw. (net) ist Default */
		if (!strcasecmp (meta->distribution, "mausnet")
			|| !strcasecmp (meta->distribution, "maus"))
			textsize += fprintf (f, ":(MausNet)" LINEEND);
		else if (!strcasecmp (meta->distribution, opts.pollsite))
			textsize += fprintf (f, ":(lokal)" LINEEND);
		/* btw: used_cludges wird nicht erhoeht */
	}
	/* Nun Wildwestverkettung */
	if (need_ww)
		write_message_refs (f, meta, need_ww, groups);


	if (meta->reply_to && !opts.send_reply_to)
	{
		/* nur *ein* Reply-To, mehr ist wohl nicht sinnvoll, oder? */
		textsize += fprintf (f, ":Reply-To: %s" LINEEND, meta->reply_to->text);
		used_cludges = 1;
	}

	if (meta->keywords)
	{
		textsize += fprintf (f, ":Keywords: %s" LINEEND, meta->keywords);
		used_cludges = 1;
	}

	if (meta->summary)
	{
		textsize += fprintf (f, ":Summary: %s" LINEEND, meta->summary);
		used_cludges = 1;
	}

	if (is_crossposting)
	{
		metalist_t *ng;

		textsize += fprintf (f, ":Newsgroups: ");
		ng = meta->newsgroups;
		while (ng)
		{
			grp_data_t *grp;
			grp=grp_suche_rfcname(ng->text,opts.pollsite);
			if (grp)
				textsize += fprintf (f, "%s", grp->mausname);
			else
				textsize += fprintf (f, "(%s)", ng->text);
			if (ng->next)
				textsize += fprintf (f, ",");
			else
				textsize += fprintf (f, LINEEND);
			ng = ng->next;
		}
		used_cludges++;
	}
	if (!is_mail && !opts.send_followup_to && meta->followup_to)
	{
		/* Bei News Followup-Cludge erzeugen */
		metalist_t *ng;

		textsize += fprintf (f, ":Followup-To: ");
		ng = meta->followup_to;
		while (ng)
		{
			if (!strcasecmp (ng->text, "poster"))
				textsize += fprintf (f, "poster");
			else
			{
				grp_data_t *grp;
				grp=grp_suche_rfcname(ng->text,opts.pollsite);
				if (grp)
					textsize += fprintf (f, "%s", grp->mausname);
				else
					textsize += fprintf (f, "(%s)", ng->text);
			}
			if (ng->next)
				textsize += fprintf (f, ",");
			else
				textsize += fprintf (f, LINEEND);
			ng = ng->next;
		}
		used_cludges++;
	}

	if (used_cludges)
	{
		/* Dann eine Leerzeile nach den Pseudoheadern einbauen */
		textsize += fprintf (f, ":%s", LINEEND);
	}
	return textsize;
}

int
maus_export (FILE * f, metamsg_t * meta)
{
	/* 
	 * Die zentrale Exportroutine fr Nachrichten im MausTausch-Format.
	 * Sie wird durch die Notwendigkeit, lngere Nachrichten zu splitten,
	 * nicht unbedingt eleganter. 
	 */
	char *start;
	char part[10];
	int partnr = 0;

	start = meta->body.start;

	while (start && *start)
	{
		size_t hlen;
		int llen;

		sprintf (part, "%d", partnr);
		hlen = export_part (f, meta, (partnr) ? part : "");
		/* Zuerst mal die relativ schne Variante ohne TAB-Wandelung. */
		if (!opts.tabsize)
		{
			while (start && *start)
			{
				char *p;
				p=strpbrk(start,"\r\n");
				if (p)
					llen=p-start;
				else
					llen=strlen(start);

				if (llen+2+hlen>opts.split_after)
					break;
				fprintf (f, ":%*.*s" LINEEND,
						 llen, llen, start);
				hlen+=llen+2;
				if (!p)
					start=NULL;
				else
				{
					if (*p=='\r')
						p++;
					if (*p=='\n')
						p++;
					start=p;
				}
			}
		}
		else /* Variante mit TAB-Wandelung */
		{
			while (start && *start)
			{
				char *p;
				char *q;
				int lc=0;


				p=strpbrk(start,"\r\n");
				for (q=start; (p) ? q<p : *q;q++)
				{
					lc++;
					if (*q=='\t')
					{
						lc+=lc%opts.tabsize;
					}
				}
				/* lc enthaelt nun die Laenge des Strings ohne \r\n */
#if 0
				fprintf(f,"%ld + %ld = %ld\n",
					(long) lc, (long) hlen, (long) lc+hlen+2);
#endif
				if (lc+hlen+2>opts.split_after)
					break;
				hlen+=lc+2;
				putc (':', f);

				lc=0;
				for (q=start; (p) ? q<p : *q;q++)
				{
					if (*q=='\t')
					{
						do
						{
							putc (' ', f);
							lc++;
						}
						while (lc % opts.tabsize);
					}
					else
					{
						putc (*q, f);
						lc++;
					}
				}
				fputs (LINEEND, f);
				if (!p)
					start=NULL;
				else
				{
					if (*p=='\r')
						p++;
					if (*p=='\n')
						p++;
					start=p;
				}
			}
		}
		/* Falls die Nachricht gesplittet werden mute, dann die
		 * Distribution setzen, um Dupes zu verhindern.
		 */
		if (partnr || (start && *start))
				fprintf (f, "D%s"LINEEND,opts.mausnet_dist_splitted);
		partnr++;
	}							/* while noch etwas brig */
	return 0;
}


static void
make_path (metamsg_t * m)
{
	/* Eine Path-Zeile zusammensetzen.
	 * Gegenueber frueheren MausConnectversionen stark vereinfacht. 
	 * Den Pfad einer Nachricht im MausNet zu rekonstruieren ist 
	 * ziemlich witzlos.
	 */

	char buf[16384];

	/* Den Namen unserer Kiste und den der Pollmaus in den Pfad setzen. */
	if (opts.path_entry && *opts.path_entry)
	{
		sprintf (buf, "%s!%s%s!not-for-mail", opts.path_entry,
			 opts.pollsite, opts.mausdomain);
	}
	else
	{
		sprintf (buf, "%s%s!not-for-mail", opts.pollsite, opts.mausdomain);
	}

	convert_umlaute (buf);
	m->path = XSTRDUP (buf);
}

static void
add_ref (metalist_t ** r, char *s)
{
	char *p;

	while (s && *s)
	{
		p = strchr (s, ' ');
		if (p)
			*p = '\0';
		meta_add_list (r, s, NULL, 0); /* Fehler ist nicht soo wild */
		if (p)
			*p = ' ';
		s = p;
		/* nur zur Sicherheit */
		while (s && *s == ' ')
			s++;
	}
}

int
maus_import (FILE * f, metamsg_t * meta, char *line, size_t linelen,
			 int *zurueckgestellt)
{
	int weiter;
	int ende_bei_doppelkreuz;
	int before_first_textline = 0;	/* In header or pseudoheader? */
	int raus = 0;
	int retcode = 0;
	/* eigentlich 256, aber wg. Umlautwandlung besser mehr ... */
	unsigned char zwi_fromrealname[MAUS_STRLEN] = "";
	unsigned char zwi_fromadress[MAUS_STRLEN] = "";
	unsigned char zwi_maus_id[MAUS_STRLEN] = "";	/* # */
	unsigned char zwi_real_id[MAUS_STRLEN] = "";	/* I */
	unsigned char zwi_maus_ref[MAUS_STRLEN] = "";	/* - */
	unsigned char zwi_real_ref[MAUS_STRLEN] = "";	/* R */
	unsigned char zwi_to[MAUS_STRLEN] = "";		/* A */
	unsigned char zwi_date[MAUS_STRLEN] = "";	/* E */
	unsigned char tmp[1024] = "";
	unsigned char tmp2[1024] = "";

	int b_found = 0;
	int g_found = 0;

	/* Erst mal die Nachricht auf einen definierten leeren Zustand setzen */
	meta_clean_message (meta);
	/* Zeilenweise lesen */
	if (*zurueckgestellt)
		weiter = 1;
	else
	{
		weiter = fgets (line, linelen - 1, f) != NULL;
		if (!weiter)
			retcode = 1;		/* Ende des Files */
	}

	*zurueckgestellt = 0;
	ende_bei_doppelkreuz = 0;

	while (weiter)
	{
		char *crlfp;

		crlfp = strpbrk (line, "\r\n");
		if (crlfp)
			*crlfp = '\0';

		switch (line[0])
		{
		case '#':
			if (line[1] == '\0')
			{
				/* Eine Zeile mit nur '#' beendet das Outfile */
				retcode = 1;
				raus = 1;
				break;
			}
			if (ende_bei_doppelkreuz)
			{
				*zurueckgestellt = 1;
				retcode = 0;	/* noch nicht zuende */
				raus = 1;
				break;
			}
			ende_bei_doppelkreuz = 1;

			before_first_textline = 1;

			if (strchr (line + 1, '@'))
			{
				/* Muss sein */
				convert_umlaute (line + 1);
				strcpy (zwi_maus_id, line + 1);
				meta->maus_id = strdup (line + 1);
				COOM (meta->maus_id);
			}
			else
			{
				/* Spezialnachricht */
				unsigned char timezwi[1024];
				minfo_t *found;

				struct tm *tm;
				time_t now;

				now = time (NULL);
				/* ID erzeugen */
				sprintf (zwi_real_id, "%s%ld@%s", line + 1, (long) now,
						 opts.mysitedomain);
				/* Special_ID sichern */
				meta->maus_special_id = XSTRDUP (line + 1);

				tm = localtime (&now);

				strftime (timezwi, sizeof (timezwi), "%c", tm);

				/* Sender erzeugen */
				sprintf (tmp, "MausConnect@%s", opts.mysitedomain);
					/* Fehler ist nicht so wild */
				meta_add_list (&meta->sender, tmp, NULL, 0);

				/* "From" erzeugen */
				sprintf (zwi_fromadress, "maus@%s%s", opts.pollsite, opts.mausdomain);
				sprintf (zwi_fromrealname, "%s %s",
						 opts.boxtyp == BOXTYP_MAUS ? "MAUS" : "Quark",
						 opts.pollsite);

				found = info_find (line + 1);
				if (found)
				{
					/* Ein Infofile */

					/* "Subject" */
					if (!found->kurztext || !*(found->kurztext))
					{
						char zwi2[1024];

						if (line[1] == 'I')
							sprintf (zwi2, "MAUSINFO %s (%s)", line + 1,
									 timezwi);
						else
							sprintf (zwi2, "MAUSDATEN %s (%s)", line + 1,
									 timezwi);
						meta->subject = strdup (zwi2);
					}
					else
					{
						char zwi2[1024];

						sprintf (zwi2, "%s (%s)", found->kurztext, timezwi);
						meta->subject = strdup (zwi2);
					}
					COOM (meta->subject);
				}
				else if (!strcmp (line + 1, "LOG"))
				{
					/* "Subject" */
					char zwi[1024];

					sprintf (zwi, "LOG %s", timezwi);
					meta->subject = strdup (zwi);
					COOM (meta->subject);
				}
				else if (!strcmp (line, "#HEAD"))
				{
					/* Eigentlich berflssig, wird weiter oben in der
					 * Hierarchie sowieso verworfen. 
					 */
					char zwi[1024];

					sprintf (zwi, "HEAD %s", timezwi);
					meta->subject = strdup (zwi);
					COOM (meta->subject);
				}
				/* unbekannte Bloecke */
				else
				{
					unsigned char zwi2[256];

					/* "Subject" */
					sprintf (zwi2, "MAUSDATEN %s (%s)", line + 1, timezwi);
					meta->subject = strdup (zwi2);
					COOM (meta->subject);
				}
			}
			break;
		case 'E':
			meta->date = maus_convert_tauschdate (line + 1);
			/* Fr Erzeugung der Message-ID sichern */
			if (!strchr (line, ' ') && strlen (line + 1) >= 12)
				strcpy (zwi_date, line + 1);
			break;
		case 'I':
			convert_umlaute (line + 1);
			strcpy (zwi_real_id, line + 1);
			break;
		case 'R':
			convert_umlaute (line + 1);
			strcpy (zwi_real_ref, line + 1);
			break;
		case 'V':				/* Von */
			if (zwi_fromrealname[0])
				maus_convert_adr (line + 1, zwi_fromadress, tmp2);
			else
				maus_convert_adr (line + 1, zwi_fromadress, zwi_fromrealname);
			break;
		case 'A':				/* an */
			strcpy (zwi_to, line + 1);
			break;
		case 'G':				/* gruppe */
			{
				grp_data_t *grp;

				grp=grp_suche_mausname(line+1,opts.pollsite);
				if (!grp)
				{
					/* neue Gruppe, noch nicht in der Liste */
					grp_create_rfcname(line+1,tmp);
					grp=XMALLOC(sizeof(grp_data_t));
					*grp=leer_grp;
					grp->mausname=XSTRDUP(line+1);
					grp->rfcname=XSTRDUP(tmp);
					grp->boxen=XSTRDUP(opts.pollsite);
					log(LOG_INFO,"Neue Gruppe %s -> %s",	
						grp->mausname,grp->rfcname);
					grp_append_new(grp);
				}
				/* hinten an die Liste der Gruppen anhaengen */
				if (!meta_add_list (&meta->newsgroups, grp->rfcname, NULL, 0))
				{
					log(LOG_CRIT,"out of memory");
					exit(1);
				}
				g_found = 1;
			}
			break;
		case 'N':
			strcpy (zwi_fromrealname, line + 1);
			break;
		case 'W':
			meta->subject = strdup (line + 1);
			COOM (meta->subject);
			break;
		case 'O':
			meta->organization = strdup (line + 1);
			COOM (meta->organization);
			break;
		case '-':
			convert_umlaute (line + 1);
			strcpy (zwi_maus_ref, line + 1);
			break;
		case 'D':
			if (line[1]=='L')
				meta->distribution=XSTRDUP(opts.usenet_dist_local);
			else if (line[1]=='M')
				meta->distribution=XSTRDUP(opts.usenet_dist_mausnet);
			else if (line[1]!='N')
				meta->distribution=XSTRDUP(line+1); /* unbekannt, was nun? */
			break;
		case 'B':
			b_found = 1;
			break;
		case 'Y':
			meta->x_gateway = XSTRDUP (line + 1);
			break;
		case ':':
			/* Textzeilen */
			if (before_first_textline)
			{
				/* Textcludges bearbeiten */
				char *atsign;
				int done = 0;


				/* 
				 * Wildwestverkettung. Widerlich, aber notwendig.
				 */
				atsign = strchr (line, '@');
				if (!meta->references && atsign)
				{
					/* dann mal Wildwestverkettung auswhlen */
					if (line[1] == '-')		/* -A1234@DU */
					{
						strtrim (line + 2);
						strcpy (zwi_maus_ref, line + 2);
						convert_umlaute (zwi_maus_ref);
						done = 1;
					}
					else if (strhead (line + 1, "Kommentar zu "))
					{
						char *p;

						done = 1;
						p = line + 1 + strlen ("Kommentar zu ");
						*tmp = '\0';
						if (*p == '\"' && strchr (p + 1, '\"'))
						{
							p++;
							strcpy (tmp, p + 1);
							*strchr (tmp, '\"') = '\0';
						}
						else if (*p != '\"')
						{
							strcpy (tmp, p);
							p = strchr (tmp, ' ');
							if (p)
							{
								*p = '\0';
							}
						}
						p = tmp + strlen (tmp) - 1;
						while (p > (char *) tmp && strchr (" .,:';/", *p))
							*p-- = '\0';
						if (p > (char *) tmp)
						{
							strcpy (zwi_maus_ref, tmp);
							convert_umlaute (zwi_maus_ref);
						}
					}
				}
				else
				{
					char *magic_followup = ":Followup-To:";
					char *magic2 = ":Reply-To:";
					char *magic_key = ":Keywords:";
					char *magic_summary = ":Summary:";
					char *p;

					if (strhead (line, magic_followup))
					{
						char *p2;

						p = line + strlen (magic_followup);
						while (isspace (*p))
							p++;
						p2 = p;
						while (*p2 && *p2 != ' ')
							p2++;
						if (*p2 == ' ' && !strncmp (p, "PM ", 3))
							sprintf (tmp, "poster");
						else if (!*p2 && !strcmp (p, "PM"))
							sprintf (tmp, "poster");
						else if (!*p)
							sprintf (tmp, "group_not_given");
						else
						{
							grp_data_t *grp;
							grp=grp_suche_mausname(p,opts.pollsite);
							if (!grp)
							{
								/* neue Gruppe, noch nicht in der Liste */
								grp_data_t leer=LEER_GRP;

								grp_create_rfcname(p,tmp);
								grp=XMALLOC(sizeof(grp_data_t));
								*grp=leer;
								grp->mausname=XSTRDUP(p);
								grp->rfcname=XSTRDUP(tmp);
								grp->boxen=XSTRDUP(opts.pollsite);
								grp->dont_save=1;
								grp_append_new(grp);
							}
							else
								strcpy(tmp,grp->rfcname);
						}
						if (!meta_add_list (&meta->followup_to, tmp, NULL, 0))
						{
							log(LOG_CRIT,"out of memory");
							exit(1);
						}
						done = 1;
					}
					else if (strhead (line, magic2))
					{
						p = line + strlen (magic2);
						while (isspace (*p))
							p++;
						if (*p)
						{
							maus_convert_adr (p, tmp, tmp2);
							if (!meta_add_list (&meta->reply_to, tmp, tmp2, 0))
							{
								log(LOG_CRIT,"out of memory");
								exit(1);
							}
						}
						done = 1;
					}
					else if (strhead (line, magic_key))
					{
						p = line + strlen (magic_key);
						while (isspace (*p))
							p++;
						if (*p)
							meta->keywords = strdup (p);
						done = 1;
					}
					else if (strhead (line, magic_summary))
					{
						p = line + strlen (magic_summary);
						while (isspace (*p))
							p++;
						if (*p)
							meta->summary = strdup (p);
						done = 1;
					}
				}
				if (done)
				{
					/* Die nchste Zeile ist mglicherweise ein weiterer
					 * Ersatzheader
					 */
					before_first_textline = 1;
				}
				else
				{
					/* 
					 * Header und Pseudoheader sind jetzt komplett gelesen,
					 * die nchste Zeile ist eine normale Textzeile. Jetzt
					 * muessen die letzten Teile des Headers zusammengesetzt
					 * werden.
					 */
					before_first_textline = 0;

					/* Die Zeile ist noch nicht verarbeitet, also wird 
					 * sie an den Body angehaengt 
					 */
					meta_add_to_body (meta, line + 1);
					meta_add_to_body (meta, "\r\n");
				}
			}					/* wenn erste Textzeile */
			else
			{
				meta_add_to_body (meta, line + 1);
				meta_add_to_body (meta, "\r\n");
			}
			break;
		}						/* switch */
		if (raus)
			break;				/* JETZT - vor dem fgets! */
		weiter = fgets (line, linelen - 1, f) != NULL;
	}							/* while */
	/* Was muss jetzt noch geschehen? */

	/* Der Absender muss nun erzeugt werden */
	{
		if (!meta_add_list (&meta->head_from, zwi_fromadress, zwi_fromrealname, 
			0))
		{
			log(LOG_CRIT,"out of memory");
			exit(1);
		}
		meta->env_from = XSTRDUP (zwi_fromadress);
	}

	/* Empfaenger auswerten */
	if (*zwi_to)
	{
		if (meta->newsgroups)
		{
			convert_umlaute (zwi_to);
			meta->x_to = strdup (zwi_to);
			COOM (meta->x_to);
		}
		else
		{
			maus_convert_adr (zwi_to, tmp, tmp2);
			if (!meta_add_list (&meta->head_to, tmp, tmp2, 0))
			{
				log(LOG_CRIT,"out of memory");
				exit(1);
			}
		}
	}

	/* Bei PMs muss jetzt eine Envelope-Adresse erzeugt werden */
	if (meta->env_to == NULL
		&& meta->newsgroups == NULL
		&& meta->head_to)
	{
		if (opts.workmode == GATE)
			meta->env_to = strdup (meta->head_to->text);
		else
			meta->env_to = strdup (opts.mailuserdomain);
		COOM (meta->env_to);
	}

	/* - Eine ID mu erzeugt werden. Die MAUS-internen IDs sind ja 
	 *   bekanntlich nicht eindeutig.
	 */
	if (*zwi_real_id)			/* I */
	{
		convert_umlaute (zwi_real_id);
		meta->message_id = strdup (zwi_real_id);
		COOM (meta->message_id);
	}
	else if (strchr (zwi_maus_id, '.')
		|| !(isupper(*zwi_maus_id) && isdigit(zwi_maus_id[1])))		/* # */
	{
		/* Auf irgendeinem Weg ist eine vollstndige ID in die # -Zeile
		 * gekommen. vielleicht ein use2maus oder Quark 2.x.
		 */
		convert_umlaute (zwi_maus_id);
		meta->message_id = strdup (zwi_maus_id);
		COOM (meta->message_id);
	}
	else if (*zwi_date)
	{
		char buf[MAUS_STRLEN];
		char *p;

		sprintf (buf, "%s.%s%s", zwi_date, zwi_maus_id, opts.mausdomain);
		convert_umlaute (buf);
		meta->message_id = strdup (buf);
		COOM (meta->message_id);

		/* Selbst zusammengesetzte IDs nach lowercase wandeln */
		p = meta->message_id;
		while (*p)
		{
			*p = tolower (*p);
			p++;
		}
	}
	else
	{
		/* Schrott, Infofile */
		char buf[MAUS_STRLEN];

		sprintf (buf, "%s%s", zwi_maus_id, opts.mausdomain);
		convert_umlaute (buf);
		meta->message_id = strdup (buf);
		COOM (meta->message_id);
	}

	/* - Eine References mu erzeugt werden */
	if (*zwi_real_ref || (*zwi_maus_ref && strchr (zwi_maus_ref, '.')))
	{
		char *work;

		/* eine "echte" References ist gefunden */
		if (*zwi_real_ref)
			work = zwi_real_ref;
		else
			work = zwi_maus_ref;
		convert_umlaute (work);
		meta_add_list (&meta->references, work, NULL, 0);
	}

	/* Pfad zusammensetzen */
	make_path (meta);
	/* X-Format-Zeile */
	meta->x_format = strdup ("MausTausch");		/* Error doesn't matter */

	meta->is_mail = 0;
	if (b_found)
		meta->is_mail = 1;
	if (g_found)
		meta->is_mail = 0;

	if (!meta->is_mail)
	{
		/* 
		 * Versuchen, noch ein paar Daten zusammenzuklauben, den
		 * Gruppennamen zu wandeln.
		 */
		metalist_t *ng;
		BOOLEAN had_followup=FALSE;
		BOOLEAN explicit_mime_header_true=FALSE;
		BOOLEAN explicit_mime_header_false=FALSE;
		BOOLEAN explicit_mime_body_true=FALSE;
		BOOLEAN explicit_mime_body_false=FALSE;

		if (meta->followup_to)
			had_followup=TRUE;

		for(ng=meta->newsgroups;ng;ng=ng->next)
		{
			grp_data_t *grp;
			grp = grp_suche_rfcname (ng->text,opts.pollsite);
			if (grp)
			{
				if (grp->default_followup && !had_followup)
				{
					meta_add_list(&meta->followup_to,
						grp->default_followup,NULL,0);
				}
				if (grp->mime_headers==TRI_TRUE)
					explicit_mime_header_true=TRUE;
				else if (grp->mime_headers==TRI_FALSE)
					explicit_mime_header_false=TRUE;
				if (grp->mime_body==TRI_TRUE)
					explicit_mime_body_true=TRUE;
				else if (grp->mime_body==TRI_FALSE)
					explicit_mime_body_false=TRUE;
				if (grp->approve && !meta->approved)
					meta->approved=XSTRDUP(grp->approve);
			}
		}
		if (explicit_mime_header_true)
			meta->mime_code_headers=TRUE;
		else if (explicit_mime_header_false)
			meta->mime_code_headers=FALSE;
		else
			meta->mime_code_headers=opts.mime_coded_headers;
		if (explicit_mime_body_true)
			meta->mime_code_body=TRUE;
		else if (explicit_mime_body_false)
			meta->mime_code_body=FALSE;
		else
			meta->mime_code_body=opts.mime_coded_body;
	}
	else
	{
		meta->mime_code_headers=opts.mime_coded_headers;
		meta->mime_code_body=opts.mime_coded_body;
	}

	if (retcode == 1 || feof (f))
		*zurueckgestellt = 0;
	return retcode;
}


static void
maus_mk_ref (metamsg_t * msg, char *mref, char *rref)
{
	char *mdomp;
	char *dot;
	char c;
	metalist_t *l;

	*rref = '\0';
	*mref = '\0';

	if (!msg->references)
		return;
	l = msg->references;
	while (l->next)
		l = l->next;

	if (!strchr (l->text, '@'))
		return;					/* das ist Schrott, soetwas interessiert nicht ... */

	mdomp = strstr (l->text, opts.mausdomain);
	if (!mdomp)
	{
		/* Ahem - keine Mausdomain angegeben */
		strcpy (rref, l->text);
		return;
	}
	*mdomp = '\0';

	/* Falls Punkt im Rest des Domainparts, dann keine Maus-ID */
	if (strchr (strchr (l->text, '@'), '.'))
	{
		/* keine legale ID fr das MausNet */
		*mdomp = *opts.mausdomain;
		strcpy (rref, l->text);
		return;
	}
	dot = strchr (l->text, '.');
	if (dot)
	{
		*dot = '\0';
		c = toupper (dot[1]);
		if (c == 'A' || c == 'P' || opts.boxtyp != BOXTYP_MAUS)
		{
			/* Schn! */
			char *p;

			strcpy (mref, dot + 1);

			p = mref;
			while (*p)
				*p++ = toupper (*p);

			*dot = '.';
			*mdomp = *opts.mausdomain;
			/* Keine Realref angeben? Oder doch? */
			strcpy (rref, l->text);
			return;
		}
		else
		{
			/* Keine legale Mausinterne ID */
			*dot = '.';
			*mdomp = *opts.mausdomain;
			strcpy (rref, l->text);
			return;
		}
	}
	else
	{
		/* Kein Punkt angegeben */
		c = toupper (*l->text);
		if (c == 'A' || c == 'P' || opts.boxtyp != BOXTYP_MAUS)
		{
			strcpy (mref, l->text);
			*mdomp = *opts.mausdomain;
			/* Keine Realref angeben! */
			return;
		}
		else
		{
			/* Keine legale Mausinterne ID */
			*mdomp = *opts.mausdomain;
			strcpy (rref, l->text);
			return;
		}
	}
}

