#if !defined(LIB_H) || defined(INIT)
#ifndef INIT
#define LIB_H
extern char *gate_version;

#include "config.h"

typedef struct
{
	char *start;
	char *next;
	size_t size; /* Nicht Lnge (next-start), sondern Puffergre */
} buf_t;

	/*
	 * Zeilenende fr MausTausch, SMTP, NNTP
	 */
#define LINEEND "\r\n"
	/*
	 * Erlaubte Zeilenendezeichen. *Nach* diesen Zeichen kommen nur
	 * noch andere Zeilenendezeichen.
	 */
typedef struct mc_list_struct
{
	char *data;
	char *ext;
	struct mc_list_struct *next;
} mc_list_t;

typedef enum 
{ 
	MAUS2RFC=1, 
	RFC2MAUS=2, 
	MROUTE=4,
	GATECONFTEST=8,
	CALL=16,
	GETMAUSVAL=32,
	MCOM=64,
	MHIST=128
} PROGRAM;


/* Defaultbezeichner */
#define DEFAULT_CNFNAME "mausgate.cnf"

/* allgemeine Libraryfunktionen */
int strhead(const char *s1, const char *s2);
char *strtrim(char *s);
char *instring(const char *s1,const char *s2);
int strhalfcasecmp(const char *uplow, const char *low);
int strnhalfcasecmp(const char *uplow, const char *low, size_t n);
int parse_bool(char *s);
FILE *xfopen(const char *fname, const char *mode);

void convert_umlaute(unsigned char *s);
BOOLEAN entumlauten(const char *from, char *to);
int do_conf(char *progname,char *libdir,char *conffilename, char *aktion);
char *get_conf_value(char *which);
void dump_conf(FILE *f);
void errormail(FILE *input,char *subject, char *error, ...);
time_t get_date(char *p, time_t now, long zone);
void configure(PROGRAM prog,int argc,char **argv,int *remainder);
void bounce(char *to, char *reason, char *mbuf);
int growprintf (unsigned char **buffer, unsigned char **aktpos, 
	size_t *bufsize, size_t grow, unsigned char *format,...);

#ifdef HAVE_TCP
/* Internet services */
int open_service(const char *servicename, const char *host);
FILE * fopen_service(const char *servicename, const char *host);
#endif

int is_allowed(const char *);

/* rund um Lockfiles */
int lock_create(const char *filename, int *fatal);
int lock_delete(const char *filename);
char *lock_dev_filename(const char *devicename);

typedef struct cancel_struct
{
	char *id;
	struct cancel_struct *next;
	int done;
} cancel_t;

/* canceln von Nachrichten */
int cancel_init(int keep_locked);
cancel_t *cancel_find_in_list(char *id);
cancel_t *cancel_anker(void);
void cancel_end(int delete,int rewrite);
void cancel_add(char *id);

/* Libraryfunktionen fr Logfile */
void open_log(char *name, BOOLEAN with_msglog);
void log(int priority, char *format, ...);
void close_log(void);
void msglog(char *format, ...);

#ifdef HAVE_SYSLOG_H
#include <syslog.h>
#endif
#ifndef LOG_WARNING
#define LOG_EMERG   0   /* system is unusable */
#define LOG_ALERT   1   /* action must be taken immediately */
#define LOG_CRIT    2   /* critical conditions */
#define LOG_ERR     3   /* error conditions */
#define LOG_WARNING 4   /* warning conditions */
#define LOG_NOTICE  5   /* normal but significant condition */
#define LOG_INFO    6   /* informational */
#define LOG_DEBUG   7   /* debug-level messages */
#endif

/* Kommandozeilenauswertung */
extern char *action;
extern char *conffilename;
extern char *libdirname;
extern char *transferidfilename;
extern char *outputfilename;
extern char *inputfilename;
extern char *mail_to;
extern int argument_hist_reorg;
extern int argument_hist_check;
extern int argument_test;
extern int argument_route_create;
extern int argument_route_show;
extern int argument_route_test;
extern int argument_hexpire_really;
extern int argument_hexpire_mdays;
extern int argument_hexpire_ndays;

	/* fr Programmaufruf mit Aktion "help" */
extern char *global_help_text; 

extern int verbose_flag;
extern int this_is_mail_flag;
extern int argument_is_rnews;
extern char *argument_from;
extern char *argument_value;
extern char *argument_id;
extern char *argument_id_delete;
extern char *argument_via;
enum work_only_enum { ONLY_ALL, ONLY_MAIL, ONLY_NEWS, ONLY_SPECIAL };
extern int work_only;
extern int dont_use_nntp;
extern int dont_use_smtp;

typedef enum { USER, GATE } WORKMODE;
typedef enum { LOGMODE_BOTH, LOGMODE_SYSLOG, LOGMODE_FILE } LOGMODE;
typedef enum 
{ 
	BOXTYP_NONE,
	BOXTYP_QUARK_I,
	BOXTYP_QUARK_II, 
	BOXTYP_MAUS 
} BOXTYP;

typedef enum 
{ 
	REKILL_FULL,  /* kill all re: */
	REKILL_SMART, /* kill re: if line too long */
	REKILL_NONE   /* don't kill re: */
} REKILLMODE;

typedef enum 
{ 
#ifdef FEATURE_SMTP
	MAILMODE_SMTP, /* direct call to smtp */
#endif
	MAILMODE_BSMTP  /* write mailbatch in incoming/ */
} MAILMODE;

/***************************************************************************
 * Variablen der Konfigurationsdatei
 * genutzt fr Deklaration und Initialisierung, via INIT-Makro 
 ***************************************************************************/
#define INIT(x,y)	x;
#define IS_DECLARE
extern struct overalloptions {
#else
struct overalloptions opts= {
#endif
	INIT(mc_list_t *route_actions, NULL)
	INIT(char *pollsite,       NULL)
	INIT(char *mailuser,       NULL)
	INIT(char *mailuserdomain, NULL)
	INIT(WORKMODE workmode,    USER)
	INIT(MAILMODE mailmode,    MAILMODE_BSMTP)
	INIT(int direct_smtp,      0)
	INIT(char *mailhost,       "localhost")
	INIT(char *newshost,       "localhost")
	INIT(char *usenet_dist_local, "local")
	INIT(char *usenet_dist_mausnet, "MausNet")
	INIT(char *mausnet_dist_mausnet, "M")
	INIT(char *mausnet_dist_local, "L")
	INIT(char *mausnet_dist_splitted, "L")
	INIT(char *compress,       "gzip")
#ifdef HAVE_SYSLOG_H
	INIT(int sl_facility,      LOG_NEWS)
	INIT(int sl_priority,      LOG_NOTICE)
#endif
	INIT(BOXTYP boxtyp,        BOXTYP_NONE)
	INIT(char *reroute_host,     NULL)
	INIT(mc_list_t *allowed, NULL)
	INIT(mc_list_t *reach_sites, NULL)

	INIT(char *gatemaster,     "postmaster")
	INIT(char *mysite,        NULL)
	INIT(char *mysitedomain,  NULL)
	INIT(char *organization,  "MausNet")
	INIT(char *path_entry,    "MausNet")
	INIT(char *gprefix,       "maus")

	INIT(char *mausgatedomain, NULL)
	INIT(char *mausdomain,     ".maus.de")


	INIT(char *spoolbase,      NULL)
	INIT(char *tempdir,        "/var/tmp")

	INIT(short lock_tries,     10)
	INIT(short lock_sleep,     5)
		
	INIT(int munge_mails,      0)
	INIT(int testmode,         0)
	INIT(int send_ag,          1)
	INIT(int send_dzeile,      1)
	INIT(int send_path,        0)
	INIT(int send_long_subject,1)
	INIT(int send_crosspostings,0)
	INIT(int send_sender,0)
	INIT(int send_followup_to,0)
	INIT(int send_reply_to,0)
	INIT(int warn_news_routing,0)
	INIT(short limit_references,3)
	INIT(short subjectlen,        30)
	INIT(long split_after,      ((15*1024)+512))
	INIT(int cancel_echo,      0)
	INIT(int kill_crosspostings,0)
	INIT(int unlink_unallowed,1)
	INIT(int mime_coded_headers,0)
	INIT(int mime_coded_body,0)
	INIT(int autoinfosubscribe, 1)
	INIT(int rekillmode,       REKILL_FULL)
	INIT(short tabsize,           4)
	INIT(char *diffcmd,        "diff -u --report-identical-files")

	/* fr mcall */
	INIT(int log_session,               0)
	INIT(short dial_retry_time,         60)
	INIT(short dial_timeout,            60)
	INIT(char *uucplockdir,             NULL)
	INIT(short uucplock_sleep,          30)

	INIT(char *modem_line,              NULL)
	INIT(char *modem_at,                "AT")
	INIT(unsigned long modem_speed,		0)
	INIT(char *modem_reset,             "+++ATZ")
	INIT(char *modem_init,              NULL)
	INIT(short modem_timeout,           10)
	INIT(char *modem_dial,              "ATD")

	INIT(short fp_max,                  100)
	INIT(short fp_busy,                 2)
	INIT(short fp_no_carrier,           20)
	INIT(short fp_no_dial,              20)
	INIT(short fp_error,                10)
	INIT(short fp_reset,                10)
	INIT(short fp_init,                 10)
	INIT(short fp_lock,                 1)
	INIT(short fp_dial_timeout,         40)
	INIT(short fp_modem_timeout,        10)
	INIT(short fp_dev_init,             2)
#ifdef IS_DECLARE
#undef INIT
#undef IS_DECLARE
	char *dummy;
} opts;
#else
	NULL
};
#endif

#endif
