/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1994 Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "ownstd.h"
#include "xmalloc.h"
#include "mausconlib.h"

/* sichere malloc/strdup-Funktionen */

static void 
memerror (const char *what, size_t bytes, const char *fname, int line);

/* int (*xmemfailure)(char *,size_t,char *,int) = memerror; */
void (*xmemfailure)(const char *,size_t,const char *,int) = memerror;

/*
 * Idee von INN geklaut ...
 */
static void 
memerror (const char *what, size_t bytes, const char *fname, int line)
{
	fprintf(stderr,"cannot %s %ld bytes (file %s line %d)\n",
		what,(long) bytes, fname,line);
	fflush(stderr);
	log(LOG_CRIT,"cannot %s %ld bytes (file %s line %d)",
		what,(long) bytes, fname,line);
	exit(1);
}

void *
xmalloc(size_t size, const char *fname,int line)
{
	void *p;
	p=malloc(size);
	while (p==NULL)
	{
		xmemfailure("malloc",size,fname,line);
		p=malloc(size);
	}
#ifdef MEMDEBUG
	fprintf(stderr,"malloc called from %s, line %d\n"
					"  for %ld bytes, result %lx\n",
					fname,line,(long)size,(long)p);
#endif
	return p;
}

char *
xstrdup(const char *copyme,const char *fname,int line)
{
	char *p;
	p=strdup(copyme);
	while (p==NULL)
	{
		size_t l=strlen(copyme);
		xmemfailure("strdup",l,fname,line);
		p=strdup(copyme);
	}
#ifdef MEMDEBUG
	fprintf(stderr,"strdup called from %s, line %d\n"
					"  for %lx, result %lx (%40.40s)\n",
					fname,line,(long)copyme,(long)p,copyme);
#endif
	return p;
}

void *
xrealloc(void *area,size_t size,const char *fname,int line)
{
	void *p;
	p=realloc(area,size);
	while (p==NULL)
	{
		xmemfailure("realloc",size,fname,line);
		p=realloc(area,size);
	}
#ifdef MEMDEBUG
	fprintf(stderr,"realloc called from %s, line %d\n"
					"  for %lx, to %ld size, result %lx\n",
					fname,line,(long)area,(long)size,(long)p);
#endif
	return p;
}

void *
xcalloc(size_t count, size_t esize, const char *fname, int line)
{
	void *p;
	p=calloc(count,esize);
	while (p==NULL)
	{
		xmemfailure("calloc",esize,fname,line);
		p=calloc(count,esize);
	}
#ifdef MEMDEBUG
	fprintf(stderr,"calloc called from %s, line %d\n"
					"  for %lx elements of %ld size, result %lx\n",
					fname,line,(long)count,(long)esize,(long)p);
#endif
	return p;
}

#ifdef MEMDEBUG
void
xfree(void *area,const char *fname,int line)
{
	fprintf(stderr,"free called from %s, line %d, for %lx\n",
					fname,line,(long)area);
	if ((long)area<0x28500)
		fprintf(stderr,"DANGER: %s\n",(char *)area);
	fflush(stderr);
	free(area);
}
#endif
