/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1994 Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* 
	lib/metamsg.c - handle metamsg 
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "ownstd.h"
#include "mausconlib.h"
#include "metamsg.h"
#include "xmalloc.h"

#undef LISTDEBUG
#ifdef LISTDEBUG
#define LOWLIM ((void *)0x25000)
#define HIGHLIM ((void *)0x100000)
#define OOB(x,s) if (x && (x<LOWLIM || x>HIGHLIM)) LICP(x,s)
static void
LICP(long x, char *s)
{
	fprintf(stderr,"DANGER: List out of bounds (%s, %lx)!\n",s,x);
}
#endif

void
meta_free_list (metalist_t **para)
{
	metalist_t *next;
	metalist_t *l;

	if (!para || !*para)
		return;

	l=*para;

#ifdef LISTDEBUG
	OOB(l,"element");
	OOB(l->text,"text");
	OOB(l->ext,"extension");
	OOB(l->next,"next");
#endif
	while (l)
	{
		if (l->text)
			XFREE (l->text);
		if (l->ext)
			XFREE (l->ext);
		next = l->next;
		XFREE (l);
		l = next;
	}
	*para=NULL;
}

void
meta_clean_message (metamsg_t * msg)
{
#define FREELI(x) if (msg->x) { meta_free_list(&msg->x);}
#define FREESTR(x) if (msg->x) { XFREE(msg->x); msg->x=NULL; }
#if 0
	if (msg->references 
		&& (msg->references->ext
			|| (msg->references->next && msg->references->next->ext)))
			fprintf(stderr,"DANGER\n");
#endif
	FREESTR (env_from);
	FREESTR (env_to);

	FREELI (head_to);
	FREELI (head_from);
	FREELI (reply_to);
	FREELI (sender);

	FREELI (newsgroups);
	FREELI (followup_to);

	FREESTR (message_id);
	FREESTR (maus_id);
	FREESTR (maus_special_id);
	FREESTR (transfer_id);
	FREELI (references);

	msg->date = (time_t) 0;
	FREESTR (distribution);
	FREESTR (subject);


	FREESTR (also_control);
	FREESTR (approved);
	FREESTR (content_type);
	FREESTR (content_transfer_encoding);
	FREESTR (control);
	FREESTR (mime_version);
	FREESTR (received);
	msg->expires = (time_t) 0;
	FREESTR (keywords);
	FREESTR (organization);
	FREESTR (path);
	FREESTR (summary);
	FREESTR (supersedes);
	FREESTR (comments);
	FREESTR (x_gateway);
	FREESTR (x_format);
	FREESTR (x_mailer);
	FREESTR (x_error);
	FREESTR (x_to);
	FREELI (unknown_headers);
	FREELI (return_receipt_to);
	FREELI (header_garbage);
	if (msg->body.start)
	{
		msg->body.start[0]='\0';
		msg->body.next=msg->body.start;
	}
	msg->bodystat.bytes=0;
	msg->bodystat.lines=0;
	msg->bodystat.achtbit=0;
	msg->is_mail=0;

	msg->mime_code_headers=FALSE;
	msg->mime_code_body=FALSE;

#undef FREELI
#undef FREESTR
}

metalist_t *
meta_create_element (const char *text, const char *ext)
{
	metalist_t *e;

	if (!text && !ext)
		return NULL;
	e = malloc (sizeof (metalist_t));
	if (!e)
		return NULL;
	if (text)
	{
		e->text = strdup (text);
		if (!e->text)
		{
			free (e);
			return NULL;
		}
	}
	if (ext)
	{
		e->ext = strdup (ext);
		if (!e->ext)
		{
			if (e->text)
				free (e->text);
			free (e);
			return NULL;
		}
	}
	return e;
}

BOOLEAN 
meta_add_list (metalist_t ** list, char *text, char *ext, int vorne)
{
	metalist_t *new;
	metalist_t *l;
	char *p1 = NULL;
	char *p2 = NULL;

	new = malloc (sizeof (metalist_t));
	if (!new)
		return FALSE;
	if (text)
	{
		p1 = strdup (text);
		if (!p1)
		{
			free (new);
			return FALSE;
		}
	}
	if (ext && *ext)
	{
		p2 = strdup (ext);
		if (!p2)
		{
			free (new);
			if (text)
				free (p1);
			return FALSE;
		}
	}
	new->text = p1;
	new->ext = p2;
	if (vorne)
	{
		new->next = *list;
		*list=new;
		return TRUE;
	}
	/* Hinten anhaengen */
	new->next = NULL;
	l = *list;
	while (l)
	{
		if (l->next == NULL)
		{
			l->next = new;
			return TRUE;
		}
		l = l->next;
	}
	/* neue Liste */
	*list=new;
	return TRUE;
}

#define GROW (64*1024)
void 
meta_add_to_body(metamsg_t *msg, char *s)
{
	size_t used;
	size_t l;
	char *p;

	p=s;
	while (*p)
	{
		if (*p & 0x80)
			msg->bodystat.achtbit++;
		if (*p == '\n')
			msg->bodystat.lines++;
		p++;
	}
	l=p-s;
	msg->bodystat.bytes+=l;

	used=msg->body.next-msg->body.start;
	if (l+1+used>msg->body.size)
	{
		msg->body.start = XREALLOC (msg->body.start, msg->body.size + GROW);
		msg->body.next = msg->body.start + used;
		msg->body.size +=GROW;
	}
	strcpy(msg->body.next,s);
	msg->body.next+=l;
}
#undef GROW
