#ifndef METAMSG_H
#define METAMSG_H

#include <time.h>

typedef struct metaliststruct
{
	char *text; /* data */
	char *ext;  /* extension */
	struct metaliststruct *next;
} metalist_t;

typedef struct metamsg
{
	/* Ist es Mail oder News? */
	int is_mail;
	/* Envelope der Nachricht, nur bei MAIL */
	char *     env_from;       /* physikalischer Absender */
	char *     env_to;         /* Zieladresse dieser Nachricht */
	/* Headerangaben */
	metalist_t *head_to;         /* Empfaenger */
	metalist_t *head_from;       /* Absenderadresse + -name */
	metalist_t *reply_to;        /* Empfnger einer Antwort (adresse+name) */
	metalist_t *sender;          /* tatschlicher Absender (adresse+name) */

	metalist_t *newsgroups;      /* Liste der Newsgroups */
	metalist_t *followup_to;     /* Liste der Followup-Groups */

	/* Message-ID */
	char *message_id;
	char *maus_id;               /* ID im MausNet */
	char *maus_special_id;       /* ITG, LOG, HEAD, ... */
	char *transfer_id;           /* Uebergangs-ID zwischen User und MAUS */
	metalist_t *references;      /* Usenetreihenfolge! */

	/* Sonstige wichtige Headerzeilen */
	time_t date;
	char *distribution;
	char *subject;

	char *also_control;
	char *approved;
	char *content_type;
	char *content_transfer_encoding;
	char *control;
	char *mime_version;
	char *received;
	time_t expires;
	char *keywords;
	char *organization;
	char *path;
	char *summary;
	char *supersedes;
	char *comments;
	char *x_gateway; /* MausConnect blah fasel */
	char *x_format;  /* Usprungsformat der Nachricht */
	char *x_error;   /* Fehlermeldungen */
	char *x_mailer;  /* Angabe der Software des Ursprungssystems */
	metalist_t *return_receipt_to;
	char *x_to;      /* FIDO-Style To in News */

	/* Still unused */
	metalist_t *unknown_headers;
	metalist_t *header_garbage;

	/* Pointer auf Pufferbereich mit dem Messagetext darin */
	/* Der Speicherbereich wird von meta_clean_message nicht angerhrt */
	buf_t body;

	/* Zaehler, die von der Routine zum Anhaengen an den Body verwaltet
	 * werden.
	 */
	struct {
		unsigned long lines;
		unsigned long bytes;
		unsigned long achtbit; /* Zahl der 8bit-Zeichen */
	} bodystat;

	BOOLEAN mime_code_headers; /* Header mit q-p verschlsseln? */
	BOOLEAN mime_code_body; /* Body? */

} metamsg_t;

void 
meta_clean_message(metamsg_t *msg);

BOOLEAN 
meta_add_list(metalist_t **li, char *text, char *ext,int vorne);

	/* Ein metalist_t * als string schreiben. Buf sollte *gross* sein! */
int rfc_metalist2adrheader(BOOLEAN mime, char *buf,metalist_t *list);

	/* ein Element mit Kopien der beiden Strings anlegen */
metalist_t *meta_create_element(const char *text, const char *ext);

	/* Einen String an den Body anfgen. Die Zeilen muessen mit \r\n 
	 * terminiert sein.
	 */
void meta_add_to_body(metamsg_t *msg, char *s);

	/* Eine Liste entfernen */
void meta_free_list (metalist_t **l);


#endif
