#include <stdlib.h>
#include <string.h>

#ifdef CSLIB
#include "cslib.h"
#endif

#include "mimesup.h"
#include "xmalloc.h"

#include <ctype.h>

static char *b64table=NULL;
static void b64init(void)
{
	int i;
	b64table=XMALLOC(256);
	memset(b64table,255,256);
	for (i='A'; i<='Z'; i++)  b64table[i]=i-'A';
	for (i='a'; i<='z'; i++)  b64table[i]=i-'a'+26;
	for (i='0'; i<='9'; i++)  b64table[i]=i-'0'+52;
	b64table['+']=62;
	b64table['/']=63;
}

/* Is it an supported charset? */
#ifdef CSLIB
static cstl_t *
mime_charset_supported(char *charset)
{
	char cs[256];
	char *p;
	size_t len;
	if (!charset)	
		return 0;

	/* 
	 * strtrim(charset) -> handle common problems 
	 */
	strcpy(cs,charset);
	p=cs+strlen(cs)-1;
	while (isspace(*p) && p>=cs)
		p--;
	p[1]='\0';
	p=cs;
	while(isspace(*p))
		p++;
	return cslib_get_charset_from_name(p);
}
#else
static int 
mime_charset_supported(char *charset)
{
	char cs[256];
	char *p;
	size_t len;
	if (!charset)	
		return 0;
	p=cs;
	len=0;
	while (*charset)
	{
		*p=tolower(*charset);
		p++;
		charset++;
		len++;
	}
	*p='\0';
	p=strstr("us-ascii,iso-8859-1",cs);
	if (!p)
		return 0;
	/* charset has trailing whitespace or \0 */
	if (p[len]==' ' || p[len]=='\t' || !p[len])
		return 1;
	return 0;
}
#endif

void 
mime_decode_headerline(char *str)
{
	char tmp[256];
	char *start;
	start=strstr(str,"=?");
	/* while there may be another encoded word */
	while (start)
	{
		char *charset;
		char *encoding;
		char *text;
		char *end;
#ifdef CSLIB
		cstl_t *got_charset;
#else
		int got_charset;
#endif
		int got_encoding;
		char c;
		/* 
		 * =?charset?encoding?enc.text=?
		 * ^- start
		 *   ^- charset
		 *           ^- encoding
		 *                    ^- test
		 *                            ^- end
		 */
		end=strstr(start,"?=");
		if (!end)
			break; /* can't translate, don't change (further) */
		charset=start+2;
		encoding=strchr(charset+1,'?');
		if (!encoding)
			break;
		encoding++; /* now on Q or B */
		text=strchr(encoding+1,'?');
		if (!text)
			break;

		/* try to find the charset, but try to preserve the string
		 * as long as possible.
		 * "tmp" is long enough anyway.
		 */
		memcpy(tmp,charset,encoding-charset-1);
		tmp[encoding-charset-1]='\0';
		got_charset=mime_charset_supported(tmp);

		/* known encoding? */
		c=tolower(*encoding);
		got_encoding = ((c=='q'||c=='b') && encoding[1]=='?');
		
		/* if supported charset && supported encoding */
		if (got_charset && got_encoding && c=='q')
		{
			/* known charset, quoted printable ->translate */
			char *p;
			char *ptmp;
			size_t restlen;
			size_t translen;

			ptmp=tmp;
			p=text+1;
			while (*p && *p!='?')
			{
				switch(*p)
				{
				case '=':
					{
						char buf[3];
						buf[0]=p[1];
						buf[1]=p[2];
						buf[2]='\0';
#ifdef CSLIB
						ptmp+=cslib_char_to_native(got_charset,ptmp,
							strtol(buf,NULL,16));
#else
						*ptmp=strtol(buf,NULL,16);
						ptmp++;
#endif
						p+=3;
					}
					break;
				case '_':
					p++;
					*ptmp=' ';
					ptmp++;
					break;
				default:
					*ptmp=*p;
					p++;
					ptmp++;
					break;
				} /* case */
			} /* while */
			*ptmp='\0';
			restlen=strlen(end+2);
			translen=strlen(tmp);
			memcpy(start,tmp,translen);
			memmove(start+translen,end+2,restlen+1);
		}
		else if (got_charset && got_encoding && c=='b')
		{
			/* base64 */
			char *pr;
			char *pw;
			size_t len;
			long l=0; /* keep gcc happy */
			int r;
			int val;
			if (!b64table)
				b64init();
			pr=text+1;
			pw=start;
			r=0;
			while (*pr!='?' && *pr)
			{
				if (*pr=='=')
				{	
					pr++;
					continue;
				}
				val=b64table[*pr++];
				if (val==255)
					continue; /* illegal char, ignore */
				switch(++r)
				{
				case 1:
					l=val*262144;
					continue;
				case 2:
					l+=val*4096;
					continue;
				case 3:
					l+=val*64;
					continue;
				case 4:
					l+=val;
					r=0;
#ifdef CSLIB
					pw+=cslib_char_to_native(got_charset,pw,l/65536);
					l%=65536;
					pw+=cslib_char_to_native(got_charset,pw,l/256);
					pw+=cslib_char_to_native(got_charset,pw,l%256);
#else
					*pw++=l/65536;
					l%=65536;
					*pw++=l/256;
					*pw++=l%256;
#endif
					l=0;
				}
			}
			/* decode leftovers, but careful */
#ifdef CSLIB
			if (l/65536)
			{
				pw+=cslib_char_to_native(got_charset,pw,l/65536);
				l%=65536;
			}
			if (l>256)
				pw+=cslib_char_to_native(got_charset,pw,l/256);
			if (l%256)
				pw+=cslib_char_to_native(got_charset,pw,l%256);
#else
			if (l/65536)
			{
				*pw++=l/65536;
				l%=65536;
			}
			if (l>256)
				*pw++=l/256;
			if (l%256)
				*pw++=l%256;
#endif
			if (*pr=='?')
				pr++;
			if (*pr=='=')
				pr++;
			len=strlen(pr);
			memmove(pw,pr,len+1); /* +1 wg. \0 */
		}
		/* 
		 * else: cannot translate. possibly another program has any idea,
		 * so leave it as it was. 
		 */
		start=strstr(start+1,"=?");
	}
}

int
mime_content_text(char *s)
{
	/* Content-Type: type/subtype [;pararameter]* 
	 * we can deal with "text/plain", and "charset=..." 
	 */

	if (*s == '\n')
		return 0; /* well, assume text/plain and us-ascii */
	/* skip whitespace */
	while (isspace(*s))
		s++;
	/* s now points to "type/subtype" */
	if (!strncasecmp(s,"text/plain",10))
		s+=10;
	else if (!strncasecmp(s,"text",4)) /* common mistake */
		s+=4;
	else
		return 0; /* cannot handle it */
	while (isspace(*s))
		s++;
	if (!*s)
		return 1;
	if (*s++!=';') /* Hey? What are you? */
		return 0;
	return 1;
#if 0
	do
	{
		if (!strncasecmp(s,"charset=",8))
		{
			/* found charset */
			char *t;
			char c;
			t=s+8;
			while (!isspace(*t) && *t!=';')
				t++;
			c=*t;
			*t='\0';
			if (mime_charset_supported(t))
				return 1;
			*t=c;
		}
		s=index(s,';');
		if (!s)
			return 1; /* us-ascii */
		s++;
	}
	while (s && *s);
#endif
}
                                                               
enum mime_encoding
mime_get_encoding(const char *s)
{
	enum mime_encoding enc=unknown_encoding;
	while (isspace(*s))
		s++;
	if (strncasecmp(s, "7bit", 4) == 0)
	{
		enc=seven_bit;
		s += 4;
	}
	else if (strncasecmp(s, "8bit", 4) == 0)
	{
		enc=eight_bit;
		s += 4;
	}
	else if (strncasecmp(s, "binary", 6) == 0)
	{
		enc=binary;
		s += 6;
	}
	else if (strncasecmp(s, "quoted-printable", 16) == 0)
	{
		s += 16;
		enc=quoted_printable;
	}
	else if (strncasecmp(s, "base64", 6) == 0)
	{
		s += 6;
		enc=base_64;
	}
	if (*s == '\0' || isspace(*s) || *s == ';' || *s == '(')
		return enc;
	else
		return unknown_encoding;
}

#define DECIMAL(c) \
	( isdigit(c) ? (c-'0') : \
	(c>='A' && c<='F') ? (c-'A'+10) : \
	(c>='a' && c<='f') ? (c-'a'+10) : 0 )

/* decode one buffer of quoted printable */
void
mime_decode_qp(char *buf)
{
	char *rpos;
	char *wpos;
	unsigned int c;
	rpos=wpos=buf;
	while (1)
	{
		switch(*rpos)
		{
		case '=':
			/* soft linebreaks? */
			if (!strcmp(rpos+1,"\r\n"))
				rpos+=3;
			else if (rpos[1]=='\n')
				rpos+=2;
			else
			{
				c=(unsigned int) DECIMAL(rpos[1])*16+DECIMAL(rpos[2]);
				if (c==0)
					*wpos++=' ';
				else
#ifdef CSLIB
					*wpos+=cslib_char_to_native(cs,wpos,c);
#else
					*wpos+=c;
#endif
				rpos+=3;
			}
			break;
		case '\0':
			*wpos='\0';
			return;
		case '\n':
			/* Therefore, when decoding a Quoted-Printable
			 * body, any trailing white space on a line must be
			 * deleted, as it will necessarily have been added
			 * by intermediate transport agents.
			 */
			 {
			 	char *p=rpos-1;
			 	if (*p=='\r')
			 		p--;
			 	while (isspace(*p))
			 	{
			 		p--;
			 		wpos--;
			 	}
			 }
			*wpos++=*rpos++;
			break;
		default:
			*wpos++=*rpos++;
			break;
		}
	}
}

void 
mime_decode_base64(char *buf)
{
	/* base64 */
	char *pr;
	char *pw;
	long l=0; /* keep gcc happy */
	int r;
	int val;
	if (!b64table)
		b64init();
	pr=buf;
	pw=buf;
	r=0;
	while (*pr)
	{
		if (*pr=='=')
		{	
			break; 
			pr++;
			continue;
		}
		val=b64table[*pr++];
		if (val==255)
			continue; /* illegal char, ignore */
		switch(++r)
		{
		case 1:
			l=val*262144;
			continue;
		case 2:
			l+=val*4096;
			continue;
		case 3:
			l+=val*64;
			continue;
		case 4:
			l+=val;
			r=0;
#ifdef CSLIB
			pw+=cslib_char_to_native(got_charset,pw,l/65536);
			l%=65536;
			pw+=cslib_char_to_native(got_charset,pw,l/256);
			pw+=cslib_char_to_native(got_charset,pw,l%256);
#else
			*pw++=l/65536;
			l%=65536;
			*pw++=l/256;
			*pw++=l%256;
#endif
			l=0;
		}
	}
	/* decode leftovers, but careful */
#ifdef CSLIB
	if (l/65536)
	{
		pw+=cslib_char_to_native(got_charset,pw,l/65536);
		l%=65536;
	}
	if (l>256)
		pw+=cslib_char_to_native(got_charset,pw,l/256);
	if (l%256)
		pw+=cslib_char_to_native(got_charset,pw,l%256);
#else
	if (l/65536)
	{
		*pw++=l/65536;
		l%=65536;
	}
	if (l>256)
		*pw++=l/256;
	if (l%256)
		*pw++=l%256;
#endif
	*pw='\0';
}

void 
mime_decode_buffer(char *buffer,enum mime_encoding enc)
{
	if (enc==base_64)
		mime_decode_base64(buffer);
	else if (enc==quoted_printable)
		mime_decode_qp(buffer);
}
