/* lib/nntp.c: Routinen fr nntp */

/* Zum Teil abgeschaut aus (s)trn */

#include "ownstd.h"

#ifdef HAVE_TCP

#ifdef HAVE_SYS_STAT_H
#  include <sys/stat.h>
#endif
#include <errno.h>
#include <ctype.h>
#include <conflib.h>
#include <stdarg.h>

#include "mausconlib.h"
#include "metamsg.h"
#include "rfc.h"
#include "nntp.h"
#include "xmalloc.h"

#ifdef FEATURE_NNTP
static FILE *nntpf;

	/* nntp-Zeilenende */
#define CRLF "\r\n"

static char nntprepbuffer[1024];
static char *format_reply(void);

void nntp_close(void)
{
	fprintf (nntpf, "QUIT" CRLF);
	(void) nntp_get_reply(NULL);
	fclose(nntpf);
}

int
nntp_open(const char *host)
{
	int ret;
	if (nntpf)
		return 0;

	nntpf = fopen_service ("nntp", host);
	if (!nntpf)
	{
		/* Die Fehlermeldung wurde von open_service schon geschrieben */
		return 1;
	}

	setvbuf (nntpf, NULL, _IOLBF, 0);

	ret = nntp_get_reply (NULL);	/* Connection establishment reply */
	if (ret > 299)
	{
		/* Tja, keine Verbindung ... */
		log(LOG_WARNING,"Connection not established",nntprepbuffer);
		fclose (nntpf);
		return 1;
	}
	return 0;
}

int
nntp_get_reply (char **s)
{
	fd_set fds;
	struct timeval timeout;

	FD_ZERO (&fds);
	FD_SET (fileno (nntpf), &fds);

	timeout.tv_sec = 10 * 60;			/* seconds */
	timeout.tv_usec = 0;
	if (s)
		*s=nntprepbuffer;

	/* `select' returns 0 if timeout, 1 if input available, -1 if error. */
	if (select (FD_SETSIZE, NULL, &fds, NULL, &timeout)!=1)
	{
		*nntprepbuffer='\0';
		return 000;				/* Timeout signalisieren */
	}

	/* nntp kennt keine Fortsetzungen */
	fgets (nntprepbuffer, sizeof (nntprepbuffer), nntpf);
	return (int) strtol (nntprepbuffer, NULL, 10);
}

static 
char *format_reply(void)
{
	char *p=nntprepbuffer;
	char *p2;
	while (isspace(*p) || isdigit(*p))
		p++;
	p2=strlen(p)+p-1;
	while (*p=='\r' || *p=='\n')
		*p--='\0';
	return p;
}

/*
 * Eine Nachricht senden.
 * Returncodes:
 * 0 -> OK.
 * 1 -> send later again (Nachricht ist OK, aber ...)
 * 2 -> Dont send again.
 * 3 -> Verbindung tot.
 */
#define RETCODE(r) ((r==400) ? 2 : 1 )
int nntp_send(metamsg_t *meta)
{
	int ret;
	char zwi[NNTP_STRLEN];
	/*
	 * 
	 */
	/* SMTP-Envelope */
	sprintf (zwi, "IHAVE <%s>" CRLF, meta->message_id);
	fprintf (nntpf, "%s", zwi);
	ret = nntp_get_reply(NULL);	/* IHAVE reply */
	if (ret != NNTP_SENDIT_VAL)
	{
		char *p=format_reply();
		/* Aha? Warum das? */
		if (ret==NNTP_GOODBYE_VAL) /* Verbindung zusammengebrochen */
		{
			log(LOG_WARNING,"Verbindung zum NNTP Server unterbrochen mit %s",
				p);
			msglog("Verbindung zum NNTP Server unterbrochen mit %s\n",
				p);
			return 3;
		}
		if (ret==NNTP_HAVEIT_VAL) /* typischerweise Dupe */
		{
			msglog("Server hat schon: %s",meta->message_id);
			return 2; /* spaeter aendern */
		}
		if (ret==NNTP_RESENDIT_VAL)
		{
			msglog("Spter nochmal versuchen: %s",meta->message_id);
			return 1;
		}
		msglog("%s: %s",meta->message_id,p);
		return 2;	/* irgendein marginales oder greres Problem */
	}

	rfc_print_message(meta,nntpf,CRLF,1);
	fprintf (nntpf, "." CRLF);
	ret = nntp_get_reply(NULL);
	if (ret>299)
	{
		if (ret==NNTP_GOODBYE_VAL)
		{
			char *p=format_reply();
			log(LOG_WARNING,"Verbindung zum NNTP Server unterbrochen mit %s",
				p);
			msglog("Verbindung zum NNTP Server unterbrochen mit %s\n",
				p);
			return 3;
		}
		if (ret==NNTP_RESENDIT_VAL)
		{
			msglog("Spter nochmal versuchen: %s",meta->message_id);
			return 1;
		}
		msglog("%s: %s",meta->message_id,format_reply());
		return 2;	/* irgendein marginales oder greres Problem */
	}
	msglog("%s: gesendet",meta->message_id);
	return 0;
}
#undef RETCODE

static int
nntp_art_cmd(const char *cmd, const char *id,long artnum)
{
	char zwi[NNTP_STRLEN];
	if (id)
		sprintf (zwi, "%s <%s>" CRLF, cmd, id);
	else
		sprintf (zwi, "%s %ld" CRLF, cmd, artnum);
	fprintf (nntpf, "%s", zwi);
	return  nntp_get_reply(NULL);
}

int
nntp_stat(const char *id,long artnum)
{
	return nntp_art_cmd("STAT",id,artnum); 
}

int
nntp_head(const char *id,long artnum)
{
	return nntp_art_cmd("HEAD",id,artnum); 
}

int
nntp_get_list(char **buf, size_t *bufsize)
{
	char lbuf[16384]; /* eine Zeile :-) */
	char *wp;
	char *cp;
	int ende=0;
	size_t l;
	wp=*buf;

	while (!ende)
	{
		if (!fgets(lbuf,sizeof(lbuf),nntpf))
			ende=1;
		else if (*lbuf=='.')
		{
			/* Ende-Zeile? */ 
			if ((lbuf[1]=='\r' && lbuf[2]=='\n' && lbuf[3]=='\0')
					|| (lbuf[1]=='\n' &&  lbuf[2]=='\0'))
				ende=1;
			else
				cp=lbuf+1;
		}
		else
			cp=lbuf;
		if (!ende)
		{
			l=strlen(cp);
			if (!*buf || !*bufsize)
			{
				*buf=XMALLOC(16384);
				*bufsize=16384;
				wp=*buf;
			}
			else if ((wp-*buf) + l > *bufsize - 1)
			{
				size_t offset;
				offset=wp-*buf;
				*bufsize+=16384;
				*buf=XREALLOC(*buf,*bufsize);
				wp=*buf+offset;
			}
			strcpy(wp,cp);
			wp+=l;
		}
	} /* while ! ende */
	return 0;
}

char *
nntp_get_headerentry(const char *id, const char *kennung)
{
    int ret;
	char *p,*q;
	static size_t hbufsize;
	static char *hbuf;
	static char last_id[512]="";

	if (!nntpf)
		return NULL;
	if (strcasecmp(id,last_id))
	{
		ret=nntp_head(id,0);
		if (ret>299)
			return NULL;

		/* lesen */
		if (nntp_get_list(&hbuf, &hbufsize))
		   return NULL;
		/* So - der ganze Header ist in hbuf ... */
	}

	/* Das sieht nach bertriebenem Aufwand aus, wo man doch auch mit
	* XHDR an einzelne Zeilen kommen kann. Leider jedoch ffnen der
	* Referenz-NNTP-Server und INN fr jeden Header erneut das File,
	* und das frit Zeit ...
	*/
	p=rfc_find_header(hbuf,kennung);
	if (!p)
		return NULL;
	while (1)
	{
		q=strstr(hbuf,"\r\n");
		if (!q)
			break;
		if (!isspace(q[2]))
		{
			*q='\0';
			break;
		}
		*q++=' ';
		*q=' ';
	}
	return p;
}

#endif /* FEATURE_NNTP */

#endif /* HAVE_TCP */
