/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1993 1994 Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* 
	lib/rfc.c: Libraryroutinen fr rfc-Behandlung 
*/


#include "ownstd.h"
#include <errno.h>
#include <ctype.h>
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif
#include <fcntl.h>

#include "mausconlib.h"
#include "metamsg.h"
#include "rfcaddr.h"
#include "rfc.h"
#include "xmalloc.h"
#ifdef CSLIB
#include "cslib.h"
#endif
#include "mimesup.h"

static int xread(int fd, char *pos, off_t bytes);
static int easy_parse(metalist_t ** ml,char *s);
static int unpack_one(const char *basename, int fd, buf_t *mbuf, 
	metamsg_t *meta, FILE *f, BOOLEAN (*call_fn)(FILE *, metamsg_t *meta));
static int start_child (int fd, char *path, char *argv[]);
static int read_line(char *p, size_t size, int fd);

static char UNPACK[] = "news-unpack";
#define RNEWS_MAGIC1               '#'
#define RNEWS_MAGIC2               '!'
#define MAX_FORKS                  10

size_t
rfc_head27bit (BOOLEAN mime, char *to, char *from)
{
	if (mime)
		return mime_code_headstr (to, from);
	else
	{
		size_t n;

		sprintf (to, "%s", from);
		convert_umlaute (to);
		n = strlen (to);
		return n;
	}
}

size_t
rfc_make_date (char *buf, size_t buflen, time_t t)
{
	struct tm *tm;
	struct tm *gmt;
	size_t l;
	time_t dif;

	if (t == (time_t) 0)
		t = time (NULL);

	tm = localtime (&t);

	/* 4 Mar 1994 18:09:00 GMT */

	/* 
	 * ARG! Wieso in aller Welt wird aus %Z im Sommer "GMT DST"? 
	 */
	/* Tue, 15 Mar 1994 16:15:00 GMT */
	/* %a   %d %b  %Y   %H:%M:%S %Z*/

	gmt=gmtime(&t);
	dif=(t - mktime(gmt))/60;
	
	l=strftime (buf, buflen, "%a, %d %b %Y %H:%M:%S ", tm);
	sprintf(buf+l,"%c%4.4d",
		(dif>0) ? '+' : '-',
		(int) ((dif/60)*100 + (dif % 60)));
	return strlen(buf+l)+l;
}


size_t
mime_code_headline (char *out, const unsigned char *in)
{
	const unsigned char *p;
	size_t l;
	size_t len;
	char buf[1024];
	char *p2;

	p = in;
	while (*p && *p != ' ' && *p != ':')
		p++;
	if (!*p)
	{
		strcpy (out, in);
		return strlen (out);
	}
	l = (p - in) + 1;
	strncpy (out, in, l);
	if (p[1] == ' ')
	{
		strcat (out, " ");
		p++;
		l++;
	}
	len = strlen (p);
	strcpy (buf, p);
	p2=strpbrk(buf,"\r\n");
	if (p2)
		*p2='\0';
	len = l + mime_code_headstr (out + l, buf);
	strcat (out + len, LINEEND);
	return strlen (out);
}

size_t
mime_code_headstr (char *out, const unsigned char *in)
{
	/* zur Zeit nur quotet printable */
	const unsigned char *pi;
	int nonasciis;
	int len,
	 l,
	 nextlen;
	char *po;
	char buf[128];

	pi = in;
	nonasciis = 0;
	while (*pi)
	{
		if (!isascii (*pi))
			nonasciis++;
		pi++;
	}
	if (!nonasciis)
	{
		strcpy (out, in);
		return strlen (out);
	}
	/* nicht-ascii-Zeichen im String */
	pi = in;
	po = out;

	len = sprintf (po, "=?ISO-8859-1?Q?");
	po = po + len;
	nextlen = len;

	while (*pi)
	{
		if (*pi == '=' || *pi == '?' || *pi == '_' || *pi & 128 || *pi < 32)
		{
			sprintf (buf, "=%02X", *pi);
			l = 3;
		}
		else if (*pi == ' ')
		{
			buf[0] = '_';
			buf[1] = '\0';
			l = 1;
		}
		else
		{
			buf[0] = *pi;
			buf[1] = '\0';
			l = 1;
		}
		nextlen += l;
		if (nextlen <= 72)
		{
			sprintf (po, "%s", buf);
			po += l;
		}
		else
		{
			po += sprintf (po, "?=%s =?ISO-8859-1?Q?",LINEEND);
			sprintf (po, "%s", buf);
			len = strlen ("=?ISO-8859-1?Q?") + l;
			nextlen = len;
			po += l;
		}
		pi++;
	}
	sprintf (po, "?=");
	return strlen (out);
}

int
rfc_metalist2adrheader (BOOLEAN mime, char *buf, metalist_t * list)
{
	char zwi[1024];
	char *pb;
	size_t len;
	metalist_t *pl;

	pl = list;
	pb = buf;
	len = 0;

	while (pl)
	{
		if (pl->text)
		{
			strcpy (buf, pl->text);
			convert_umlaute (pb);
			len += strlen (pb);
			pb = buf + len;
			if (pl->ext)
			{
				rfc_head27bit(mime, zwi, pl->ext);
				len += sprintf (pb, " (%s)", zwi);
				pb = buf + len;
			}
			if (pl->next)
			{
				len += sprintf (pb, ",%s\t",LINEEND);
				pb = buf + len;
			}
		}
		pl = pl->next;
	}
	*pb = '\0';
	return len;
}

/*
 * Eine Headerzeile in einer Nachricht suchen. Vergleichsweise 
 * ineffizient, deshalb nicht fr den Dauergebrauch empfohlen.
 */
char *rfc_find_header(const char *message, const char *headername)
{
	size_t l;
	l=strlen(headername);
	while (*message!='\0' && *message!='\n' && *message!='\r')
	{
		if (message[l]==':'
			&& (message[l+1]==' ' || message[l+1]=='\t')
			&& !strncasecmp(message,headername,l))
		{
			/* Dann ist es der Headername */
			char *p,*p2;
			/* Auf das naechste Zeichen nach "header: " gehen */
			message+=l+2;
			/* Leerzeichen ueberspringen */
			while (*message==' ' || *message=='\t')
				message++;
			p=strpbrk(message,"\r\n");
			if (!p)
				return XSTRDUP(message);
			p2=XMALLOC(p-message+1);
			memcpy(p2,message,p-message);
			p2[p-message]='\0';
			return p2;
		}
		message=strpbrk(message,"\r\n");
		if (!message)
			return NULL;
		if (*message=='\r')
			message+=2;
		else
			message+=1;
		if (*message=='\n' || *message=='\r')
			return NULL;
	}
	return NULL;
}

/*
 * Eine Nachricht aus einer Datei lesen, von der wir wissen, da sie n
 * Byte lang ist oder die ganze Nachricht umfat (-1).
 */
int 
rfc_read_message_bytes(int fd, size_t bytes, buf_t *buf)
{
	char *from;
	char *to;

	int err;
	if (bytes!=(size_t) -1)
	{
		if (buf->size<bytes)
		{
			buf->start=XREALLOC(buf->start,bytes+1);
			buf->size=bytes+1;
		}
		
		if (xread(fd,buf->start,bytes)!=0)
		{
			err=errno;
			log(LOG_ERR,"rfc_read_message_bytes: cannot read (%m)\n");
			errno=err;
			return -1;
		}
		/* terminate string */
		buf->next=buf->start+bytes;
		*buf->next='\0';
	}
	else
	{
		/* stat auf Pipes ist nicht so frchterlich gut ... */
#if INT_MAX > 65536
#	define BLOCK 65536
#else
#	define BLOCK 16384
#endif
		int count;
		
		buf->next=buf->start;
		do
		{
			if (BLOCK+buf->next-buf->start+1 > buf->size)
			{
				off_t offset;
				offset=buf->next-buf->start;
				buf->start=XREALLOC(buf->start,buf->size+BLOCK+1);
				buf->next=buf->start+offset;
				buf->size+=BLOCK+1;
			}
			count=read(fd,buf->next,BLOCK);
			if (count>0)
				buf->next+=count;
		} while (count>0);
		*buf->next='\0';
#undef BLOCK
	}

	/*
	 * Im Folgenden werden die Fortsetzungszeilen zusammengefasst. Das 
	 * ist notwendig, knnte aber auch in rfc_to_meta geschehen. Dort
	 * wre es wahrscheinlich etwas effizienter - aber auch 
	 * erheblicher komplizierter.
	 */
	from=buf->start;
	to=buf->start;
	while (*from)
	{
		/* Achtung: Das funktioniert mit \n und \r\n, und das ist auch
		 * sehr gut so! NNTP/SMTP verwenden \r\n.
		 */
		if (*from=='\n')
		{
			if (from[1]==' ' || from[1]=='\t')
			{
				/* Fortsetzungszeile, \n und folgende Spaces bergehen */
				from++;
				while (from[1]==' ' || from[1]=='\t')
					from++;
				*to=' ';
			}
			else if (from[1]=='\n')
			{
				/* Ende des Headers, nun kopieren */
				size_t l;
				/* Ja, 2 mal */
				*to++=*from++;
				*to++=*from++;
				/* In der Hoffnung auf _optimierte_ strlen/memmove-Routinen */
				l=strlen(from);
				memmove(to,from,l+1);
				return 0;
			}
			else if (from[1]=='\r' && from[2]=='\n')
			{
				/* Ende des Headers, nun kopieren */
				size_t l;
				/* Ja, 3 mal */
				*to++=*from++;
				*to++=*from++;
				*to++=*from++;
				/* In der Hoffnung auf _optimierte_ strlen/memmove-Routinen */
				l=strlen(from);
				memmove(to,from,l+1);
				return 0;
			}
			else
				*to=*from;
		}
		else if (to!=from)
			*to=*from;
		from++;
		to++;
	}
	return 0;

}
/*
 * Eine Datei mit einer rfc-Nachricht darin lesen.
 * *body zeigt hinterher auf den Beginn des Inhalts.
 */
int 
rfc_read_message_fd(int fd,buf_t *buf)
{
	struct stat sb;
	int err;

	if (fstat(fd,&sb)==-1)
	{
		/* Wieso das? */
		err=errno;
		log(LOG_WARNING,"rfc_read_message: cannot stat open file (%m)\n");
		errno=err;
		return -1;
	}
	if (S_ISFIFO(sb.st_mode) || sb.st_size == 0)
	{
		if (rfc_read_message_bytes(fd,-1,buf))
			return -1;
	}
	else
	{
		if (rfc_read_message_bytes(fd,sb.st_size,buf))
			return -1;
	}
	return 0;
}

/*
 * Eine Datei mit einer rfc-Nachricht darin lesen.
 */
int 
rfc_read_message(char *fname, buf_t *buf)
{
	int fd;
	int err;
	int ret;

	fd=open(fname,O_RDONLY);
	if (fd==-1)
	{
		err=errno;
		log(LOG_WARNING,"rfc_read_message: cannot open %s (%m)\n",
			fname);
		errno=err;
		return -1;
	}
	ret=rfc_read_message_fd(fd,buf);
	close(fd);
	return ret;
}

static int
xread(int fd, char *pos, off_t bytes)
{
	int    count;

	while (bytes)
	{
		count=read(fd,pos,(size_t) bytes);
		if (count<=0)
			return -1;
		pos+=count;
		bytes-=count;
	}
	return 0;
}

static int
rfc_references_to_metalist(metalist_t **ml, char *s)
{
	metalist_t *last;
	metalist_t *new;
	char *p;

	if (!s)
		return 0;
	while (isspace(*s))
		s++;
	if (!*s)
		return 0;

	last=*ml;
	while (last && last->next)
		last=last->next;

	while (*s)
	{
		while (*s && *s!='<')
		{
			while (*s && *s!=' ')
				s++;
			if (!*s)
				return 0; /* silently ignored */
			s++; /* auf das nchste Zeichen */
		}
		if (!*s)
			return 0;
		p=strchr(s,'>');
		if (!p)
			return 0; /* Was ist denn das? */
		new=XMALLOC(sizeof(metalist_t));
		new->text=XMALLOC(p-s);
		memcpy(new->text,s+1,p-s-1);
		new->text[p-s-1]='\0';
		new->ext=NULL;
		new->next=NULL;
		if (last)
		{
			last->next=new;
			last=new;
		}
		else
		{
			*ml=new;
			last=new;
		}
		s=p+1;
	}
	return 0;
}

static int
rfc_groups_to_metalist(metalist_t **ml,char *s)
{
	metalist_t *last;
	metalist_t *new;
	char *p;

	if (!s)
		return 0;
	while (isspace(*s))
		s++;
	if (!*s)
		return 0;

	last=*ml;
	while (last && last->next)
		last=last->next;

	p=strchr(s,',');
	while (p)
	{
		*p='\0';
		new=XMALLOC(sizeof(metalist_t));
		new->text=XSTRDUP(s);
		new->ext=NULL; 
		new->next=NULL;
		if (last)
			last->next=new;
		else
			*ml=new;
		last=new;
		s=p+1;
		/* Leerzeichen, leere Gruppennamen uebergehen */
		while (isspace(*s) || *s==',')
			s++;
		p=strchr(s,',');
	}
	if (!*s)
		return 0;
	new=XMALLOC(sizeof(metalist_t));
	new->text=XSTRDUP(s);
	new->ext=NULL; 
	new->next=NULL;
	if (last)
		last->next=new;
	else
		*ml=new;
	return 0;
	
}

static void
clean_domaindata(rp_domain_t *dp)
{
	rp_domain_t *next;
	int j;
	while (dp)
	{
		for (j=0;j<NSUBDOM;j++)
		{
			if (dp->sub[j])
			{
				free(dp->sub[j]);
				dp->sub[j]=NULL;
			}
		}
		next=dp->next;
		free(dp);
		dp=next;
	}
}

static void
clean_parserdata(void)
{
	rp_addr_t *ap;
	rp_addr_t *next;
	int i;
	for (i=0;i<2;i++)
	{
		ap=i ? adrlist : errlist;
#define FREEIT(x) if (x) { free(x); x=NULL; }
		while (ap)
		{
			FREEIT(ap->localp);
			FREEIT(ap->name);
			FREEIT(ap->group);
			FREEIT(ap->comment);
			FREEIT(ap->error);
#undef FREEIT
			clean_domaindata(ap->route);
			/*
			 * Niemals! Die beiden Listen knnen gemeinsame 
			 * Elemente enthalten!
				clean_domaindata(ap->destdom);
			 */
			ap->route=NULL;
			ap->destdom=NULL;
			next=ap->next;
			free(ap);
			ap=next;
		}
	}
}

#define EASY_FIRST
int 
rfc_adress_to_metalist(int meckern,metalist_t **ml,char *s)
{
	metalist_t *last;
	metalist_t *new=NULL;
#if 0
	metalist_t *akt;
#endif
	rp_addr_t *ap;
	int fails;
	size_t dl;
	char **dom;

	int ep_done=0; /* Easy-Parse tatschlich aufgerufen? */

	last=*ml;
	while (last && last->next)
		last=last->next;

#ifdef EASY_FIRST
	/* Nach Mglichkeit easy_parse benutzen, da diese Funktion wesentlich
	 * schneller ist.
	 */
	if (!strpbrk(s,",:"))
	{
		if (!easy_parse(ml,s))
			return 0;
		ep_done=1;
	}
#endif

	rfc_parse_address(s);

	fails=0;

	ap=adrlist;
	while (ap)
	{
		if (ap->localp && ap->destdom)
		{
			new=XMALLOC(sizeof(metalist_t));
			if (ap->name)
			{
				new->ext=XSTRDUP(ap->name);
			}
			else if (ap->comment)
			{
				new->ext=XSTRDUP(ap->comment);
			}
			else
				new->ext=NULL;
			dl=strlen(ap->localp)+1;
			dom=ap->destdom->sub;
			for (dom = ap->destdom->sub; dom != ap->destdom->top; dom++)
				dl+=strlen(*dom)+1;
			dl+=strlen(*dom)+1; /* +1 wegen \0 */
			new->text=XMALLOC(dl);
			sprintf(new->text,"%s@",ap->localp);
			for (dom = ap->destdom->sub; dom != ap->destdom->top; dom++)
			{
				strcat(new->text,*dom);
				strcat(new->text,".");
			}
			strcat(new->text,*dom);

			new->next=NULL;
			if (last)
			{
				last->next=new;
				last=new;
			}
			else
			{
				*ml=new;
				last=new;
			}
		}
		else
			fails++;
		ap=ap->next;
	}
	if (errlist)
		fails++;

	clean_parserdata();

	if (fails && !new)
	{
		if (ep_done || easy_parse(ml,s))
		{
			if (meckern)
			{
				log(LOG_ERR,"could not parse %s",s);
			}
			return 1;
		}
	}
	return 0;
}

static int 
easy_parse(metalist_t ** ml,char *s)
{
	/* der zweite Versuch, mit einer Adresse klarzukommen */
	char *p=strchr(s,'<');
	metalist_t *new=NULL;
	metalist_t *last;
	if (p)
	{
		/* "Irgendein Text" <adresse> 
		 * oder
		 *  Mein Name <adresse> 
		 */
		char *p1; /* Zeiger auf > */
		char *p2; /* Zeiger auf Beginn des Realnames */
		char *p3; /* Zeiger auf Ende des Realnames */
		p1=strchr(p,'>');
		if (!p1)
			return 1;
		new=XMALLOC(sizeof(metalist_t));
		/* 
		 * Vor nach <alpha@beta>
		 *          p          p1
		 */
		new->text=XMALLOC(p1-p);
		memcpy(new->text,p+1,p1-p-1);
		new->text[p1-p-1]='\0';
		p2=s;
		while (isspace(*p2))
			p2++;
		if (p2!=p)
		{
			p3=p-1;
			while (isspace(*p3) && p3>p2)
				p3--;
			/* Vor nach <alpha@beta> 
		     * p2     p3
		     * 0      7  Platzbedarf fr "Vor Nach" ist 9.
		     */
			new->ext=XMALLOC(p3-p2+2);
			memcpy(new->ext,p2,p3-p2+1);
			new->ext[p3-p2+1]='\0';
		}
		else
			new->ext=NULL;
		new->next=NULL;
	}
	else
	{
		/* Dann ist es hoffentlich die andere verbreitete Form,
		 * adresse (name) , mit (name) adresse kommen wir aber auch
		 * klar.
		 */
		char *p1;       /* Zeiger auf ( */
		char *p2;       /* Zeiger auf ) */
		p=s;
		while (isspace(*p))
			p++;
		p1=p;
		while (*p1 && *p1!='(')
			p1++;
		if (*p1=='(')
		{
			p2=p1;
			while (*p2 && *p2!=')')
				p2++;
			if (!*p2)
				return 1; /* nichts zu machen */
			if (p1!=p)
			{
				/* adresse (name) */
				char *p4; /* Zeiger auf Ende der Adresse */
				int namel;
				p4=p1-1;
				while (isspace(*p4))
					p4--;
				new=XMALLOC(sizeof(metalist_t));

				/*
				 * Adresse  , Laenge 7, Platzbedarf 8
				 * p1    p4
				 * 0     6
				 */
				new->text=XMALLOC(p4-p+2);
				memcpy(new->text, p, p4-p+1);
				new->text[p4-p+1]='\0';

				/*
				 * (name) , Laenge 6, Laenge name 4, Platzbedarf 5
				 * p1   p2
				 * 0    5
				 */
				namel=p2-p1-1;
				if (namel>0) /* () ist gar nicht so selten */
				{
					new->ext=XMALLOC(namel+1);
					memcpy(new->ext, p1+1, namel);
					new->ext[namel]='\0';
				}
				else 
					new->ext=NULL;

				new->next=NULL;
			}
			else
			{
				/* (name) adresse */
				char *p3; /* Zeiger auf Beginn der Adresse */
				int namel;
				p3=p2+1;
				while (isspace(*p3))
					p3++;
				if (!*p3)
					return 1; /* Keine Adresse angegeben */

				new=XMALLOC(sizeof(metalist_t));

				new->text=XSTRDUP(p3);

				namel=p2-p1-1;
				if (namel>0) /* () ist gar nicht so selten */
				{
					new->ext=XMALLOC(namel+1);
					memcpy(new->ext, p1+1, namel);
					new->ext[namel]='\0';
				}
				else 
					new->ext=NULL;

				new->next=NULL;
			}
		}
		else
		{
			/* Dann kann es nur eine einfache Adresse sein */
			new=XMALLOC(sizeof(metalist_t));
			new->text=XSTRDUP(p);
			p=new->text+strlen(new->text)-1;
			while (isspace(*p))
				*p--='\0';
			new->ext=NULL;
			new->next=NULL;
		}
	}
#if 1
	if (*new->text=='\"')
	{
		char *p;
		p=strchr(new->text+1,'\"');
		if (p)
		{
			char tmp[1024];
			strcpy(tmp,new->text+1);
			strcpy(new->text, tmp);
			strcpy(p-1, tmp + (p - new->text));
		}
	}
#endif

	last=*ml;
	while (last && last->next)
		last=last->next;
	if (last)
		last->next=new;
	else
		*ml=new;
	return 0;
}

	/* Kompatibilittsmodus zu V <0.3.4 */
#define BODYTO
int 
rfc_to_meta(buf_t *buf,metamsg_t *meta)
{
	char *p;
	char *linep;          /* Beginn der Zeile */
	char *namep;
	size_t namel;
	char *contentp;       /* Zeiger auf Inhalt einer Headerzeile */
	int errors_detected=0;
	int in_header=1;
#ifdef CSLIB
	cstl_t *cs=NULL;
#endif
	
	int is_mime=0;
	enum mime_encoding enc=seven_bit;
	int is_text=1;

	metalist_t *inrepto=NULL;
#ifdef BODYTO
	int got_env_to=0;
	char bodyto[1024];
	*bodyto='\0';
#endif

	/* Nachrichtenverarbeitung */
	linep=buf->start;

	/* 
	 * Header verarbeiten: Dies kann auf zwei Arten geschehen. 
	 * 1. man durchluft alle Headerzeilen und sucht den Namen in 
	 *    der Liste der bekannten Namen.
	 * 2. man sucht im Header nach allen interessanten (bekannten / zu
	 *    verarbeitenden) Strings.
	 * Die meisten Nachrichten haben nur wenige Headerzeilen, von daher
	 * ist 1. wohl schneller.
	 *
	 * Gesplittete Headerzeilen sind vorher schon zusammengesetzt worden
	 * (rfc_read_message_bytes).
	 */
	while (linep && *linep!='\0' && in_header)
	{
		namep=linep;
		/* 
		 * Zeile mit \0 statt \n terminieren und linep auf den Beginn der
		 * naechsten Zeile setzen.
		 */
		p=strpbrk(namep,"\r\n");
		if (p)
		{
			/* linep ist von nun an der Zeiger auf die nchste Zeile */
			if (*p=='\r')
				linep=p+2;
			else
				linep=p+1;
			*p='\0';
		}
		else
			linep=NULL; /* Keine weitere Zeile */
		if (*namep=='\0')
		{
			/* Leerzeile beendet Header */
			size_t l;
			
			in_header=0;
			if (is_mime)
			{
#ifdef CSLIB
				mime_decode_buffer(cs,linep,enc);
#else
				mime_decode_buffer(linep,enc);
#endif
			}

			/* Nun aus dem Lesepuffer in die metamsg_t-Struktur kopieren */
			l=strlen(linep);
			if (l+1>meta->body.size)
			{
				size_t lneu;

				lneu=l+64*1024; /* nicht kleckern, das rentiert sich nicht */

				if (meta->body.start)
					free(meta->body.start);
				meta->body.start=XMALLOC(lneu);
				meta->body.size=lneu;
			}
			meta->body.next=meta->body.start+l;
			memcpy(meta->body.start,linep,l+1);
			*meta->body.next=0;
			continue;
		}
		contentp=strchr(namep,':');
		if (!contentp)
		{	
			/* Was? Wie bitte? Zeile ohne Doppelpunkt? Im Header? */
			continue;
		}
		namel=contentp-namep-1;
		*contentp++='\0';
		if (*contentp==' ' || *contentp=='\t')
			contentp++;

		/* hier mte man MIME-Header dekodieren */
		mime_decode_headerline(contentp);

		/* simple Form von Hashing, aber effizient */
		switch(tolower(*(namep++)))
		{
		case 'a':
			/* Also-Control */
			if (!strnhalfcasecmp(namep,"lso-control",namel))
			{
				meta->also_control=XSTRDUP(contentp);
				continue;
			}
			break;
		case 'c':
			/* Control */
			if (!strnhalfcasecmp(namep,"ontrol",namel))
			{
				meta->control=XSTRDUP(contentp);
				continue;
			}
			else if (!strnhalfcasecmp(namep,"ontent-type",namel))
			{
				char *p;
				is_text=mime_content_text(contentp);
#ifdef CSLIB
				if (is_text)
				{
					p=contentp;
					while (*p && strcasecmp(p,"charset"))
						p++;
					if (*p)
					{
						p+=strlen("charset");
						while (*p && *p!='=')
							p++;
						if (*p=='=')
						{
							char buf[128];
							char *q=strchr(p,';');
							if (!q)
								strcpy(buf,p);
							else
								strncpy(buf,p,q-p);
							buf[q-p]='\0';
							strtrim(buf);
							cs=cslib_get_charset_from_name(buf);
						}
					}
				}
#endif
				continue;
			}
			else if (!strnhalfcasecmp(namep,"ontent-transfer_encoding",namel))
			{
				enc=mime_get_encoding(contentp);
				continue;
			}
			break;
		case 'd':
			/* Distribution */
			if (!strnhalfcasecmp(namep,"istribution",namel))
			{
				meta->distribution=XSTRDUP(contentp);
				continue;
			}
			/* Date */
			if (!strnhalfcasecmp(namep,"ate",namel))
			{
				/* 
				 * getdate funktioniert auch dann, wenn Date leer ist, und 
				 * setzt dann das aktuelle Datum, aber Uhrzeit 0:0 ein.
				 * Bei Verwendung einer anderen Routine muss das Verhalten
				 * geprueft werden.
				 */
				time_t now;
				now = time (NULL);
				meta->date = get_date (contentp, now, timezone);
				if (meta->date==0 || meta->date==(time_t) -1)
				{
					/* MET DST? - getdate kommt mit DST nicht klari,
					 * die Variante ist nicht erlaubt, wird aber 
					 * gelegentlich verwendet. Dummerweise ist MET alleine
					 * auch nicht erlaubt, sondern nur UT oder GMT.
					 * Auerdem kommen gelegentlich Zeitzonen wie
					 * "UNDEFINED" ... also hack.
					 */
					p=strrchr(contentp,' ');
					if (p)
					{
						/* Zweiter Versuch ... */
						*p='\0';
						meta->date = get_date (contentp, now, timezone);
						if (meta->date==0 || meta->date==(time_t) -1)
						{
							char *q=strrchr(p,' ');
							if (q)
							{
								*q='\0';
								meta->date = get_date (contentp, now, 
									timezone);
								*q=' ';
							}
						}
						*p=' ';
					}
				}
				/* Weiter oben wird der Rest abgefangen */
				continue;
			}
			break;
		case 'f':
			/* From: */
			if (!strnhalfcasecmp(namep,"rom",namel))
			{
				/* Fehler merken, aber dennoch weitermachen */
				(void) rfc_adress_to_metalist(1,&meta->head_from,contentp);
				continue;
			}
			/* Followup-To: */
			else if (!strnhalfcasecmp(namep,"ollowup-to",namel))
			{
				if (rfc_groups_to_metalist(&meta->followup_to,contentp))
					errors_detected++;
				continue;
			}
			break;
		case 'i':
			/* in-reply-to */
			if (!strnhalfcasecmp(namep,"n-reply-to",namel))
			{
				char *kl;
				char *at;
				char *gr;
				char *sp;
				if (!meta->is_mail)
					continue; /* in-reply-to ist fr News sinnlos */
				kl=contentp-1;
				while ((kl=strchr(kl+1,'<'))!=NULL)
				{
					/* kl zeigt auf < */
					gr=strchr(kl,'>');
					if (!gr)
						continue;
					at=strchr(kl,'@');
					if (!at)
						continue;
					if (at<gr)
					{
						sp=strpbrk(kl," \t");
						if (!sp || sp>gr)
						{
							*gr='\0';
							inrepto=XMALLOC(sizeof(metalist_t));
							inrepto->text=XSTRDUP(kl+1);
							inrepto->next=NULL;
							inrepto->ext=NULL;
							*gr='>';
							continue;
						}
						break;
					}
				}
				continue;
			}
		case 'm':
			/* Message-ID: */
			if (!strnhalfcasecmp(namep,"essage-id",namel))
			{
				while (isspace(*--p))
					*p='\0';
				meta->message_id=XMALLOC(p-contentp);
				memcpy(meta->message_id,contentp+1,p-contentp-1);
				meta->message_id[p-contentp-1]='\0';
				continue;
			}
			else if (!strnhalfcasecmp(namep,"ime-version",namel))
			{
				is_mime=1;
				continue;
			}
			break;
		case 'k':
			/* Keywords */
			if (!strnhalfcasecmp(namep,"eywords",namel))
			{
				meta->keywords=XSTRDUP(contentp);
				continue;
			}
			break;
		case 'n':
			/* Newsgroups? */
			if (!strnhalfcasecmp(namep,"ewsgroups",namel))
			{
				if (rfc_groups_to_metalist(&meta->newsgroups,contentp))
					errors_detected++;
				continue;
			}
			break;
		case 'o':
			/* Organization */
			if (!strnhalfcasecmp(namep,"rganization",namel))
			{
				meta->organization=XSTRDUP	(contentp);
				continue;
			}
			break;
		case 'p':
			/* Path */
			if (!strnhalfcasecmp(namep,"ath",namel))
			{
				meta->path=XSTRDUP	(contentp);
				continue;
			}
			break;
		case 's':
			if (*namep=='u')
			{
				/* Subject? */
				namep++;
				if (!strnhalfcasecmp(namep,"bject",namel))
				{
					meta->subject=XSTRDUP(contentp);
					continue;
				}
				if (!strnhalfcasecmp(namep,"mmary",namel))
				{
					meta->summary=XSTRDUP(contentp);
					continue;
				}
			}
			else if (*p=='e' && !strnhalfcasecmp(namep+1,"nder",namel))
			{
				/* Es gibt zuviele kaputte Sender-Zeilen in der
				 * Welt, *seuufz*. Wenn ich mal sehr viel Zeit habe,
				 * schreibe ich einen Autoflamewerfer gegen die 
				 * Leute, die "@anywhere", den Realname oder soetwas
				 * da eintragen.
				 * Also nicht "meckern".
				 */
				(void) rfc_adress_to_metalist(0,&meta->sender,contentp);
				continue;
			}
			break;
		case 'r':
			/* references */
			if (!strnhalfcasecmp(namep,"eferences",namel))
			{
				if (rfc_references_to_metalist(&meta->references,contentp))
					errors_detected++;
				continue;
			}
			/* Reply-To: */
			if (!strnhalfcasecmp(namep,"eply-to",namel))
			{
				rfc_adress_to_metalist(1,&meta->reply_to,contentp);
				continue;
			}
			break;
		case 't':
			/* To: ? */
			if (tolower(*namep)=='o' && namep[1]=='\0')
			{
				if (meta->env_from)
				{
					/* Wenn Mail, dann Fehler im Header ignorieren,
					 * wir haben ja den Envelope. Adressproblem 
					 * aber trotzdem ins Logfile eintragen.
					 */
					(void) rfc_adress_to_metalist(1,&meta->head_to,contentp);
					continue;
				}
				/* To im Innern der Nachricht? Oeffentlich? 
				 * Hat laut 1036-Nachfolger und meiner ureigenen
				 * berzeugung wenig Sinn, wird also ignoriert, zumal
				 * da meist sowieso nur Schrott wie "All" oder so
				 * drin steht.
				 */
				continue;
			}
			break;
		case 'x':
			if (!strnhalfcasecmp(namep,"-mausconnect-env-from",namel))
			{
				metalist_t *ml=NULL;
				(void) rfc_adress_to_metalist(1,&ml,contentp);
				if (ml && ml->text)
					meta->env_from=XSTRDUP(ml->text);
				meta_free_list(&ml);
				continue;
			}
			else if (!strnhalfcasecmp(namep,"-mausconnect-env-to",namel))
			{
				metalist_t *ml=NULL;
				(void) rfc_adress_to_metalist(1,&ml,contentp);
				if (ml && ml->text)
					meta->env_to=XSTRDUP(ml->text);
				meta_free_list(&ml);
#ifdef BODYTO
				got_env_to=1;
#endif
				continue;
			}
#ifdef BODYTO
			else if (!strnhalfcasecmp(namep,"-bodyto",namel))
			{
				/* Kompatibilitaetsmodus zu V < 0.3.4 */
				metalist_t *ml=NULL;
				(void) rfc_adress_to_metalist(1,&ml,contentp);
				if (ml && ml->text)
					strcpy(bodyto,ml->text);
				meta_free_list(&ml);
				continue;
			}
#endif
			break;
		} /* switch */
	} /* while */
#ifdef BODYTO
	if (!got_env_to)
	{
		/* Kompatitibilittsmodus */
		if (*bodyto)
		{
			/* bodyto enthlt head_to, 
			 * head_to->text den Envelope-Empfaenger.
			 * Nicht unbedingt genial, das Design :-)
			 */
			if (meta->head_to && meta->head_to->text)
			{
				meta->env_to=meta->head_to->text;
				meta->head_to->text=XSTRDUP(bodyto);
			}
			else 
			{
				meta->head_to=XMALLOC(sizeof(metalist_t));
				meta->head_to->text=XSTRDUP(bodyto);
				meta->head_to->ext=NULL;
				meta->head_to->next=NULL;
				meta->env_to=XSTRDUP(bodyto);
			}
		}
	}
	if (!meta->env_from && meta->head_from && meta->head_from->text)
	{
		meta->env_from=XSTRDUP(meta->head_from->text);
	}
#endif
	if (meta->env_from && !strchr(meta->env_from,'@'))
	{
		/* Smail will sich bei lokalen oder vermutliche lokalen
		 * Nachrichten nicht zum Erzeugen eines Domainparts 
		 * berreden lassen.
		 */
		char *p=meta->env_from;
		meta->env_from=XMALLOC(strlen(p)+2+strlen(opts.mysitedomain));
		sprintf(meta->env_from,"%s@%s",p,opts.mysitedomain);
		free(p);
	}
	if (meta->is_mail && inrepto)
	{
		if (!meta->references)
			meta->references=inrepto;
		else
		{
			metalist_t *akt;
			akt=meta->references;
			while (akt->next)
				akt=akt->next;
			akt->next=inrepto;
		}
	}
	return errors_detected;
}

void 
rfc_print_message(metamsg_t *meta,FILE *f, char *crlf,int dotescape)
{
	time_t now;
	char zwi[16384];
	int strip_cr;

	if (!meta->is_mail)
	{
		if (meta->env_to)
			meta->is_mail=1;
		else
			meta->is_mail=0;
	}

	/* 
	 * Der Nachfolger von RFC 1036 sagt, Pflichtheader seien:
	 * Date, From, Message-ID, Subject, Newsgroups, Path.
	 * Alle die drfen nur einmal auftreten. In einigen Situationen
	 * sind auch References, Sender, Approved Pflicht.
	 * Sie sollten als erste in der Nachricht kommen, vor den 
	 * optionalen Headern und den nicht standardisierten (aus
	 * Effizienzgruenden).
	 */
	if (!meta->message_id)
	{
		/* Haeh? */
		return;
	}
	strcpy(zwi,meta->message_id);
	convert_umlaute(zwi); /* paranoid, aber was soll's */
	fprintf(f, "Message-ID: <%s>%s",zwi,crlf);

	now = time (NULL);
	rfc_make_date(zwi,sizeof(zwi),meta->date ? meta->date : now);
	fprintf(f, "Date: %s%s",zwi,crlf);

	if (meta->head_from)
	{
		rfc_metalist2adrheader(meta->mime_code_headers,zwi,meta->head_from);
		fprintf(f, "From: %s%s",zwi,crlf);
	}
	else if (meta->env_from)
		fprintf(f, "From: %s%s",meta->env_from,crlf);

	if (meta->subject)
	{
		int has_re=0;
		rfc_head27bit(meta->mime_code_headers,zwi,meta->subject);
		if (!strnhalfcasecmp(zwi,"re: ",4))
			has_re=1;
		if (meta->references && !has_re)
			fprintf(f, "Subject: Re: %s%s",zwi,crlf);
		else if (!meta->references && has_re)
			/* re: weg bei Nachrichten ohne References */
			fprintf(f, "Subject: %s%s",zwi+4,crlf);
		else
			fprintf(f, "Subject: %s%s",zwi,crlf);
	}
	else
		fprintf(f, "Subject: (empty)%s",crlf);

	/* Newsgroups nur bei News erzeugen */
	if (meta->newsgroups && !meta->is_mail)
	{
		metalist_t *p;
		p=meta->newsgroups;	
		fprintf(f, "Newsgroups:");
		while (p)
		{
			fprintf(f, " %s", p->text);
			p=p->next;
		}
		fprintf(f, "%s",crlf);
	}

	/* Path ist in MAIL nicht sinnvoll, Received-Zeilen knnen wir
	 * nicht.
	 */
	if (meta->path && !meta->is_mail)
		fprintf(f, "Path: %s%s",meta->path,crlf);

	if (meta->references && meta->is_mail)
	{
		metalist_t *p;

		p=meta->references;
		while (p->next)
			p=p->next;
		strcpy(zwi,p->text);
		convert_umlaute(zwi);
		fprintf(f, "In-Reply-To: <%s>%s",zwi,crlf);
	}
	else if (meta->references)
	{
		metalist_t *p;

		fprintf(f,"References:");

		p=meta->references;
		while (p)
		{
			if (p->text)
			{
				strcpy(zwi,p->text);
				convert_umlaute(zwi);
				fprintf(f, " <%s>",zwi);
			}
			p=p->next;
		}
		fprintf(f, "%s",crlf);
	}

	if (meta->sender)
	{
		rfc_metalist2adrheader(meta->mime_code_headers,zwi,meta->sender);
		fprintf(f, "Sender: %s%s",zwi,crlf);
	}


	/* Soweit die Pflichtheader. Nun die anderen Standardheader */

	if (meta->reply_to)
	{
		rfc_metalist2adrheader(meta->mime_code_headers,zwi,meta->reply_to);
		fprintf(f, "Reply-To: %s%s",zwi,crlf);
	}

	if (meta->followup_to)
	{
		metalist_t *p;
		p=meta->followup_to;	
		fprintf(f, "Followup-To:");
		while (p)
		{
			fprintf(f, " %s",p->text);
			p=p->next;
		}
		fprintf(f, "%s",crlf);
	}

	if (meta->keywords)
	{
		rfc_head27bit(meta->mime_code_headers,zwi,meta->keywords);
		fprintf(f,"Keywords: %s%s",zwi,crlf);
	}

	if (meta->summary)
	{
		rfc_head27bit(meta->mime_code_headers,zwi,meta->summary);
		fprintf(f,"Summary: %s%s",zwi,crlf);
	}

	if (meta->organization)
	{
		if (meta->organization)
			rfc_head27bit(meta->mime_code_headers,zwi,meta->organization);
		fprintf(f,"Organization: %s%s",zwi,crlf);
	}

	if (!meta->is_mail && meta->approved)
	{
		rfc_head27bit(meta->mime_code_headers,zwi,meta->approved);
		fprintf(f,"Approved: %s%s",zwi,crlf);
	}

	if (!meta->is_mail && meta->distribution
		&& strcasecmp(meta->distribution,"world"))
	{
		rfc_head27bit(meta->mime_code_headers,zwi,meta->distribution);
		fprintf(f,"Distribution: %s%s",zwi,crlf);
	}

	/* Falls noetig, einen MIME-Header erzeugen. Der Test ist etwas
	 * primitiv, tut es aber fuer den Anfang 
	 */
	if (meta->bodystat.achtbit)
	{
		fprintf(f,	"%s%s"
				"%s%s"
				"%s%s",
			"MIME-Version: 1.0",crlf,
			"Content-Type: text/plain; charset=ISO-8859-1",crlf,
			"Content-Transfer-Encoding: 8bit",crlf);
	}

    if (meta->head_to && !meta->x_to)
	{
		/* Der 1036-Nachfolger sagt, Point-to-Point-Header wie
		 * To und Cc gehren nicht in News. 
		 */
		rfc_metalist2adrheader(meta->mime_code_headers,zwi,meta->head_to);
		if (meta->is_mail)
			fprintf(f, "To: %s%s",zwi,crlf);
		else
			fprintf(f, "X-To: %s%s",zwi,crlf);
	}

	if (meta->x_to)
	{
		rfc_head27bit(meta->mime_code_headers,zwi,meta->x_to);
		fprintf(f, "X-To: %s%s",zwi,crlf);
	}

	if (opts.workmode!=USER)
	{
		if (meta->x_format)
		{
			fprintf(f, "X-Gateway: MausConnect/%s/%s, %s%s",
				gate_version, opts.mysitedomain,meta->x_format,crlf);
		}
		else
		{
			fprintf(f, "X-Gateway: MausConnect/%s/%s%s",
				gate_version, opts.mysitedomain,crlf);
		}
		if (strchr(opts.gatemaster,'@'))
			fprintf(f, "X-Gateway-Admin: %s%s",opts.gatemaster,crlf);
		else
			fprintf(f, "X-Gateway-Admin: %s@%s%s",opts.gatemaster,
				opts.mysitedomain,crlf);
	}
	if (meta->control)
	{
		fprintf(f, "Control: %s%s",meta->control,crlf);
	}


	/* Leerzeile zum Abschlu des Headers schreiben */
	fprintf(f, "%s",crlf);

	/* Jetzt wird es interessanter */
	strip_cr=strcmp(crlf,"\r\n");
	
	{
		char *start;
		char *p;
		char saved='\0'; /* keep gcc happy */
		size_t l;
		start=meta->body.start;
		while (start && *start)
		{
			p=strpbrk(start,"\r\n");
			if (!p)
			{
				l=strlen(start);
			}
			else
			{
				l=p-start;
				saved=*p;
				*p='\0';
			}
			/* So, jetzt wissen wir den auszugebenden String!
			 * Hier wre anzusetzen, um Mime-Codierungen auszugeben.
			 */
			if (*start=='.' && dotescape)
				fprintf(f,".%*.*s%s",
					(int) l, (int)l,start,crlf);
			else
				fprintf(f,"%*.*s%s",
					(int) l, (int) l,start,crlf);

			if (p)
			{
				*p=saved;
				start=p+2;
			}
			else
				start=NULL;
		}
	}
}


/* stolen from INN */
static int 
read_line(char *p, size_t size, int fd)
{
	char    *save;

	/* Fill the buffer, a byte at a time. */
	for (save = p; size > 0; p++, size--) 
	{
		if (read(fd, p, 1) != 1) 
		{
			*p = '\0';
			syslog(LOG_ERR, "cant read first line got %s %m", save);
			return -1;
		}
		if (*p == '\n') 
		{
			*p = '\0';
			return 0;
		}
	}
	*p = '\0';
	syslog(LOG_ERR, "bad_line too long %s", save);
	return -1;
}

/* stolen from INN */
static int
start_child (int fd, char *path, char *argv[])
{
	int     pan[2];
	int     i;
	pid_t   pid;

	/* Create a pipe. */
	if (pipe(pan) < 0) 
	{
		log(LOG_ERR, "cant pipe for %s %s", path,strerror(errno));
		exit(1);
	}

	/* Get a child. */
	for (i = 0; (pid = fork()) < 0; i++) 
	{
		if (i == MAX_FORKS) 
		{
			log(LOG_ERR, "cant fork %s %s -- leaving", path,
				strerror(errno));
			return -1;
		}
		log(LOG_NOTICE, "cant fork %s -- waiting", path);
		(void)sleep(60);
	}

	/* Run the child, with redirection. */
	if (pid == 0) 
	{
		(void)close(pan[0]);

		/* Stdin comes from our old input. */
		if (fd != 0) 
		{
			if ((i = dup2(fd, 0)) != 0) 
			{
				log(LOG_ERR, "cant dup2 %d to 0 got %d %s", fd, i,
					strerror(errno));
				_exit(1);
			}
			(void)close(fd);
		}

		/* Stdout goes down the pipe. */
		if (pan[1] != 1) 
		{
			if ((i = dup2(pan[1], 1)) != 1) 
			{
				log(LOG_ERR, "cant dup2 %d to 1 got %d %s",
					pan[1], i,strerror(errno));
				_exit(1);
			}
			(void)close(pan[1]);
		}
		(void)execv(path, argv);
		log(LOG_ERR, "cant execv %s %s", path,strerror(errno));
		_exit(1);
	}

	(void)close(pan[1]);
	(void)close(fd);
	return pan[0];
}

BOOLEAN 
rnews_batch(const char *filename, metamsg_t *meta,
	FILE *f, BOOLEAN (*call_fn)(FILE *, metamsg_t *meta))
{
	int fd;
	const char *basename;
	buf_t buf = { NULL,NULL,0};

	if (filename)
		fd=open(filename,O_RDONLY);
	else
		fd=0;
	if (fd<0)
	{
		syslog(LOG_ERR, "cannot open %s: %m",filename);
		return FALSE;
	}
	basename=strrchr(filename,'/');
	if (!basename)
		basename=filename;
	if (unpack_one(basename,fd,&buf,meta,f,call_fn))
		log(LOG_NOTICE,"trouble working on %s",filename);
	if (filename)
		close(fd);
	if (buf.start)
		free(buf.start);
	return TRUE;
}

/* stolen from INN */
static int
unpack_one(const char *basename, int fd, buf_t *mbuf, metamsg_t *meta, 
	FILE *f, BOOLEAN (*call_fn)(FILE *, metamsg_t *meta))
{
	char    buff[512];
	char *  cargv[4];
	long    artsize;
	int     i;
	int     HadCount;
	int     SawCunbatch;
	long trouble=0;

	for (SawCunbatch = 0, HadCount = 0; ; ) 
	{
		/* Get the first character. */
		if ((i = read(fd, &buff[0], 1)) < 0) 
		{
			syslog(LOG_ERR, "cant read first character %m");
			return -1;
		}
		if (i == 0)
			break;

		if (buff[0] != RNEWS_MAGIC1)
		{
			/* Not a batch file.  If we already got one count, the batch
			 * is corrupted, else read rest of input as an article. 
			 */
			if (HadCount)
				return -1;
			/* den Rest knnte man jetzt als normale News lesen und
			 * verarbeiten. die ursprngliche INN-Routine macht das
			 * auch so, aber fr MausConnect ist das wohl eher 
			 * sinnlos.
			 */
			return -1;
		}

		/* Get the second character. */
		if ((i = read(fd, &buff[1], 1)) < 0) 
		{
			log(LOG_WARNING, "cant read second character %s",strerror(errno));
			return -1;
		}
		if (i == 0)
			/* A one-byte batch? */
			return -1;

		/* Check second magic character. */
		if (buff[1] != RNEWS_MAGIC2)
		{
			/* wir haben schon ein '#' - und damit beginnt garantiert
			 * kein Newsartikel. Ein Batch ist es wegen des Fehlenden
			 * ! auch nicht -> discard.
			 */
			log(LOG_ERR, "second character != %c", RNEWS_MAGIC2);
			return -1;
		}
		/* Some kind of batch -- get the command. */
		if (read_line(&buff[2], (int)(sizeof buff - 3), fd)!=0)
			return -1;

		if (strncmp(buff, "#! rnews ", 9) == 0) 
		{
			artsize = atol(&buff[9]);
			if (artsize <= 0) 
			{
				log(LOG_ERR, "bad_line bad count %s", buff);
				return -1;
			}
			HadCount = 1;

			if (0==rfc_read_message_bytes(fd, artsize, mbuf))
			{
				meta_clean_message(meta);
				if (rfc_to_meta(mbuf,meta))
				{
					/* Fehler beim Umwandeln */
					trouble++;
					continue;
				}
				if (!call_fn(f,meta))
					trouble++;
				continue;
			}
			return -1;
		}
		if (HadCount)
			/* Already saw a bytecount -- probably corrupted. */
			return -1;

		if (strcmp(buff, "#! cunbatch") == 0) 
		{
			if (SawCunbatch) 
			{
				log(LOG_ERR, "nested_cunbatch");
				return -1;
			}
			cargv[0] = UNPACK;
			cargv[1] = "-d";
			cargv[2] = NULL;
			fd = start_child(fd, 
				(opts.compress) ? opts.compress : "gzip", cargv);
			if (fd < 0)
				return -1;
			SawCunbatch = 1;
			continue;
		}
		log(LOG_ERR, "bad_format unknown command %s", buff);
		return -1;
	}
	if (trouble)
		log(LOG_NOTICE,"trouble converting %ld msgs of %s",trouble,basename);
	return 0;
}

