/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1994 Michael Kraehe, Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
	lib/route.c: Routingroutinen
*/

#include "ownstd.h"

#include <assert.h>
#include <ctype.h>
#include <fnmatch.h>
#include "xmalloc.h"

#include "mausconlib.h"
#include "route.h"

typedef struct route_struct
{
	char *action;
	char *from;
	char *dest;
	struct route_struct *next;
} route_t;

route_t *anker;

int route_show(const char *which)
{
	route_t *a;
	if (!anker)
		return 1;
	a=anker;
	while (a)
	{
		if (!which || !strcasecmp(which,a->action) 
			|| !strcasecmp(which,"route"))
		{
			printf("%s: %s / %s\n",	
				a->action,
				a->from,
				a->dest);
		}
		a=a->next;
	}
	return 0;
}

int route_open(void)
{	
	FILE *f;
	route_t *last=NULL;
	route_t *new;
	char buf[10240];
	char *pa,*pf,*pd,*p;
	char fname[PATH_MAX];
	sprintf(fname,"%s/routes",libdirname);
	f=fopen(fname,"r");
	if (!f)
		return 1;
	while (fgets(buf,sizeof(buf),f))
	{
		pa=buf;
		while(*pa && isspace(*pa))
			pa++;
		if (!*pa || *pa=='#')
			continue;
		pf=strpbrk(pa,"\t ");
		if (!pf)
			continue;
		*pf++='\0';
		pd=strpbrk(pf,"\t ");
		if (!pd)
			continue;
		*pd++='\0';
		p=pd+strlen(pd)-1;
		while (isspace(*p) && p>=pd)
			p--;
		if (p<pd)
			continue;
		p[1]='\0';
		new=XMALLOC(sizeof(route_t));
		new->action=XSTRDUP(pa);
		new->from=XSTRDUP(pf);
		new->dest=XSTRDUP(pd);
		if (!last)
			anker=new;
		else
			last->next=new;
		last=new;
	}
	fclose(f);
	return 0;
}

void route_close(void)
{
	route_t *akt,*next;
	akt=anker;
	while (akt)
	{
		XFREE(akt->dest);
		XFREE(akt->from);
		XFREE(akt->action);
		next=akt->next;
		XFREE(akt);
		akt=next;
	}
	anker=NULL;
}

static int
inlist (char *what, char *list)
{
	char *p;
	int ret;
	do
	{
		p=strchr(list,':');
		if (!p)
			return !fnmatch(list,what,FNM_CASEFOLD);
		*p='\0';
		ret=!fnmatch(list,what,FNM_CASEFOLD);
		*p=':';
		if (ret)
			return ret;
		list=p+1;
	} while (*list);
	return 0;
}

char *
route_it(char *from, char *dest)
{
	route_t *akt;
	char *p;

	p=strchr(dest,'@');
	if (p)
		dest=p+1;
	akt=anker;

	while (akt)
	{
		if (inlist(from,akt->from))
		{
			if (!dest)
				/* Oeffentliche Nachricht */
				return akt->action;
			else if (inlist(dest,akt->dest))
				return akt->action;
		}
		akt=akt->next;
	}
	return NULL;
}

int 
route_make(char *progname, char *libdir, char *conffilename)
{
	int ret=0;
	char fname[PATH_MAX];
	mc_list_t *actions;
	route_t *last=NULL;
	route_t *new;
	FILE *f;
	char abuf[1024];
	char dbuf[1024];
	actions=opts.route_actions;

	while (actions)
	{
		mc_list_t *li;
		char *akt;
		akt=actions->data;
		ret= do_conf (progname,libdir, conffilename, akt);
		if (ret)
			break;

		sprintf(abuf,"%s:%s",opts.mailuser,opts.mailuserdomain);
		li=opts.allowed;
		while (li)
		{
			strcat(abuf,":");
			strcat(abuf,li->data);
			li=li->next;
		}

		sprintf(dbuf,"%s%s",opts.pollsite,opts.mausdomain);
		li=opts.reach_sites;
		while (li)
		{
			strcat(dbuf,":");
			strcat(dbuf,li->data);
			li=li->next;
		}
			
		new=XMALLOC(sizeof(route_t));
		new->action=XSTRDUP(akt);
		new->from=XSTRDUP(abuf);
		new->dest=XSTRDUP(dbuf);
		new->next=NULL;
		if (last)
			last->next=new;
		else
			anker=new;
		last=new;
		actions=actions->next;
	}
	if (ret)
	{	
		fprintf(stderr,"cannot create routing table\n");
		return 1;
	}
	sprintf(fname,"%s/routes",libdir);
	f=fopen(fname,"w");
	if (!f)
	{
		perror(fname);
		return 1;
	}
	new=anker;
	while (new)
	{
		fprintf(f,"%s	%s	%s\n",new->action,new->from,new->dest);
		new=new->next;
	}
	fclose(f);
	return 0;

}
