/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1994 Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* 
	lib/service.c: internet service handling 
	Hauptsaechlich entstanden, um andere Module sauberer zu halten.
	Kommt nur mit Streamsockets klar.
*/

#include "ownstd.h"
#ifdef HAVE_TCP

#ifdef HAVE_SYS_STAT_H
#  include <sys/stat.h>
#endif
#include <errno.h>
#include <ctype.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <netdb.h>

#include "mausconlib.h"

FILE *
fopen_service (const char *servicename, const char *host)
{
	int fd;
	FILE *f;

	fd = open_service (servicename, host);
	if (fd < 0)
		return NULL;
	f = fdopen (fd, "r+");
	if (!f)
	{
		int err;

		err = errno;
		log (LOG_ERR, "Cannot fdopen socket for %s on %s: %s",
			 servicename, host, strerror (err));
		close (fd);
	}
	return f;
}

int
open_service (const char *servicename, const char *host)
{
	int socketfd;
	struct protoent *protocol;
	struct servent *service;
	struct sockaddr_in name;
	struct hostent *hostinfo;

	hostinfo = gethostbyname (host);
	if (!hostinfo)
	{
		log (LOG_ERR, "Cannot get host %s: %s", host,strerror(errno));
		return -1;
	}

	service = getservbyname (servicename, "tcp");
	if (!service)
	{
		log (LOG_ERR, "Cannot get service %s: %s", servicename,strerror(errno));
		return -1;
	}

	protocol = getprotobyname (service->s_proto);
	if (!protocol)
	{
		log (LOG_ERR, "Cannot get protocol %s: %s",
			 service->s_proto,strerror(errno));
		return -1;
	}

	socketfd = socket (PF_INET, SOCK_STREAM, protocol->p_proto);
	if (socketfd < 0)
	{
		log (LOG_ERR, "Cannot open socket: %s",strerror(errno));
		return -1;
	}
	/* Give the socket a name. */
	name.sin_family = AF_INET;
	name.sin_port = service->s_port;	/* s_port is in network byte order */
	name.sin_addr.s_addr = htonl (INADDR_ANY);
	name.sin_addr = *(struct in_addr *) hostinfo->h_addr;
	if (connect (socketfd, (struct sockaddr *) &name, sizeof (name)) < 0)
	{
		log (LOG_ERR, "Cannot connect socket: %m",strerror(errno));
		return -1;
	}
	return socketfd;
}
#endif /* HAVE_TCP */
