/* info2msg.c - infofileverarbeitung fr maus2rfc */

#include "ownstd.h"
#ifdef HAVE_SYS_STAT_H
	/* cheat fr die kaputt installierte Sun im Labor ... */
#  include <sys/stat.h>
#endif

#include "mausconlib.h"
#include "maus.h"
#include "info2msg.h"
#include "xmalloc.h"

/*
 * returncodes:
 * 0: nothing changed
 * 1: new infofile or old data changed
 */
static int 
work_iti_infofile(char *kuerzel, char *kurztext, long needs_crc)
{
	minfo_t *found;
	found=info_find(kuerzel);
	if (!found)
	{	
		minfo_t *new;
		new=XCALLOC(1,sizeof(minfo_t));

		new->kuerzel=XSTRDUP(kuerzel);
		new->kurztext=XSTRDUP(kurztext);
		new->needs_crc=needs_crc;
		if (needs_crc)
			new->last_crc=-1;
		if (opts.autoinfosubscribe)
			new->subscribed=1;
		info_add_new(new);
		return 1;
	}
	else
	{
		/* haben sich eventuell Daten gendert? */
		int changed=0;
		if (found->needs_crc!=needs_crc)
		{
			found->needs_crc=needs_crc;
			if (needs_crc)
				found->last_crc=-1; /* beim naechsten Mal bestellen */
			changed=1;
		}
		if (found->kurztext && kurztext)
		{
			if (strcmp(found->kurztext,kurztext))
			{
				free(found->kurztext);
				found->kurztext=XSTRDUP(kurztext);
			}
		}
		else if (!found->kurztext && kurztext)
		{
			found->kurztext=XSTRDUP(kurztext);
		}
		if (changed)
		{
			info_modified(found);
			return 1;
		}
		return 0;
	}
}

static int
work_new_iti(char *iti)
{
	char *p;
	int changes=0;
	char kuerzel[256];
	char kurztext[1024];
	int needs_crc;

	char *thisline;
	char *nextline;

	*kuerzel='\0';
	*kurztext='\0';
	needs_crc=0;

	thisline=iti;
	while (*thisline)
	{
		p=strpbrk(thisline,"\r\n");
		if (p)
		{
			*p++='\0';
			while (*p=='\r' || *p=='\n')
				p++;
			nextline=p;
		}
		else
			nextline=thisline+strlen(thisline);
		switch(thisline[0])
		{
		case '#':	/* Name des Infofiles */
			if (*kuerzel)
			{
				switch(work_iti_infofile(kuerzel, kurztext, needs_crc))
				{
				case 0:
					break;
				default:
					changes++;
					break;
				}
				*kuerzel='\0';
				*kurztext='\0';
				needs_crc=0;
			}
			strcpy(kuerzel,thisline+1);
			break;
		case ':':	/* Beschreibung des Infofiles */
			strcpy(kurztext,thisline+1);
			break;
		case 'F':
			if (maus_get_tauschflag(thisline+1,'C')=='+')
				needs_crc=1;
			break;
		}
		thisline=nextline;
	}
	if (*kuerzel)
	{
		switch (work_iti_infofile(kuerzel, kurztext, needs_crc))
		{
		case 0:
			break;
		default:
			changes++;
			break;
		}
	}
	return changes;
}


void
info_work_new_infofile(metamsg_t *meta, minfo_t *info)
{
	char fname[PATH_MAX];
	/* Wir haben ein Infofile bekommen. Der Inhalt steht in 
	 * meta->body, alle notwendigen Daten, es als Mail zu verschicken,
	 * stehen in meta.
	 */

	sprintf(fname,"%s/%s/infofiles/%s",opts.spoolbase,action,
	        info->kuerzel);

	if (!strcasecmp(info->kuerzel,"ITI"))
		work_new_iti(meta->body.start); /* Eventuelle neue ITI behandeln */

	/* Soll ein Diff gesendet werden? Das geht natrlich nur, wenn 
	 * schon ein solches Infofile da war.
	 */
	if (info->send_diff && !access(fname,R_OK)
		&& (opts.diffcmd && *opts.diffcmd))
	{
		/* diffs sind eine schoene Sache. Man bekommt sie auch sehr 
		 * einfach hin:
		 */
		FILE *wfd;
		char cmd[1024+PATH_MAX];
		sprintf(cmd,"%s %s - | mail -s 'MAUSINFO %s (diff)' '%s'",
			opts.diffcmd,fname,info->kuerzel,opts.mailuserdomain); 
		wfd=popen(cmd,"w");
		if (!wfd)
		{
			log(LOG_WARNING,"cannot diff infofile %s (%s)",
				info->kuerzel,strerror(errno));
		}
		else
		{
			/* Das interne Datenformat trennt die Zeilen mit \r\n, um
			 * nntp-Transfer zu beschleunigen und zu vereinfachen. Hier
			 * wird es dafr etwas langsamer.
			 * Tja, man kann nicht alles haben ...
			 */
			char *p;
			p=meta->body.start;
			while (*p)
			{
				if (*p!='\r')
					putc(*p,wfd);
				p++;
			}
			pclose(wfd);
			/* Und nun ist der diff unterwegs */
		}
	} /* Diff anfertigen? */
	if (info->send_mail)
	{
		/* Das Infofile als Mail an den User senden. Das wird
		 * recht einfach erreicht, nmlich durch Eintrag einer 
		 * Empfngeradresse, wodurch die Nachricht spter als
		 * normale Mail versendet wird.
		 */
		/* falls das nicht klappt, ist es auch nicht so frchterlich
		 * schlimm.
		 */
		meta->env_to=strdup(opts.mailuserdomain);
	}
	/* Sichern? */
	if (info->send_diff || info->needs_crc || !info->send_mail)
	{
		FILE *f;
		f=fopen(fname,"w");
		if (!f)
		{
			log(LOG_ERR,"cannot save infofile %s (%s)",
				info->kuerzel,strerror(errno));
		}
		else
		{
			char *p;
			p=meta->body.start;
			while (*p)
			{
				if (*p!='\r')
					putc(*p,f);
				p++;
			}
			fclose(f);
		}
	}
}

