/* ----------------------------------------------------------------------------
 *
 * Maus2RFC: Outfile in Mail/News-Batches und Infofiles konvertieren,
 * Logfile auswerten.
 *
 */

#include "ownstd.h"
#ifdef HAVE_SYS_STAT_H
#  include <sys/stat.h>
#endif
#include <errno.h>
#include <ctype.h>
#include <conflib.h>

#include "mausconlib.h"
#include "metamsg.h"
#include "rfc.h"
#include "maus.h"
#include "info2msg.h"
#include "srfcmail.h"
#include "srfcnews.h"
#include "groups.h"
#include "xmalloc.h"
#ifdef FEATURE_NNTP
#	include "nntp.h"
#endif

#define HEADLEN	1024
enum messagetyp { NO_MSG, MAIL_MSG, NEWS_MSG, IGNORE_MSG };
enum specialtyp { NO_SPECIAL, INFO_SPECIAL, LOG_SPECIAL };
enum messagetyp this_message;
enum specialtyp this_special;

char *global_help_text=
	"maus2rfc: MausTausch -> RFC Konverter\n"
	"Benutzung: maus2rfc [optionen] aktion <outfile.txt\n"
	;

/* ------------------------------------------------------------------------- */
metamsg_t meta;

unsigned char str[1024];

static void maus2rfc (FILE *f);

#define COOM(x); if (!x) { \
	fprintf(stderr,"maus2rfc: out of memory working on %s\n",#x); \
	log (LOG_CRIT, "out of memory working on %s",#x); \
	exit(1); \
	}


/* ------------------------------------------------------------------------- */

static void parse_cmdlog(char *start, char **next)
{
	/* Bis jetzt heisst "parsen" ueberspringen */
	char *rueckmeldungen="?!";
	do
	{
		start=strpbrk(start,"\r\n");
		if (!start)
		{
			*next=NULL;
			return;
		}
		start+=2; /* \r\n ueberspringen */
	} while (strchr(rueckmeldungen,*start));
	*next=start;
}

static void parse_msglog(char *start, char **next)
{
	/* Das Format des Logfiles:
	 *  :#12345
	 *  :?Haben sie eine Zeitmaschine (Datum/Uhrzeit in der Zukunft)
	 *  :=A123@...
	 * oder
	 *  :#12345
	 *  :?Das duerfen sie nicht.
	 * Ob eine Nachricht akzeptiert wurde, kann man also bei der ersten
	 * Gleichheitszeichenzeile sagen.
	 */
	char *this;
	char *rueckmeldungen="?!=";
	char *nullp;
	char nullc='\0'; /* keep gcc happy */
	char fname[PATH_MAX];
	struct stat st;
	int is_old=0;
	char mausidbuf[1024];
	char transfer_id[256];
	char errbuf[1024];
	char spruchbuf[1024];
	unsigned char str[PATH_MAX * 3 + 32];
	int ok;

	this=start;
	mausidbuf[0]='\0';
	spruchbuf[0]='\0';
	errbuf[0]='\0';

	ok=0; /* noch keine "="-Zeile bekommen. */

	do
	{
		nullp=strpbrk(this,"\r\n");
		if (!nullp)
		{
			*next=this+strlen(this);
		}
		else
		{
			char *p;
			p=nullp+1;
			while (*p=='\r' || *p=='\n')
				p++;
			*next=p;
		}

		/* start+1 zeigt jetzt auf die ID im Infile, 
		 * next auf die nchste Zeile, this auf diese Zeile 
		 */
		if (nullp)
		{
			nullc=*nullp;
			*nullp='\0';
		}
		switch(*this)
		{
		case '=':
			strcpy(mausidbuf,this+1);
			ok=1;
			break;
		case '!':
			/* irgendein Spruch. */
			if (!*spruchbuf)
				strcpy(spruchbuf,this+1);
			break;
		case '?':
			if (!*errbuf)	
				strcpy(errbuf,this+1);
			break;
		case '#':
			strcpy(transfer_id,this+1);
			break;
		}
		if (nullp)
			*nullp=nullc;
		this=*next;

	} while (*this && strchr(rueckmeldungen,*this));

	if (opts.boxtyp==BOXTYP_MAUS && !strncmp(transfer_id,"MAIL",4))
	{
		/* 
		 *	Die MAUS <7.94 wandelte die Benutzerids in Uppercase ...
		 */
		char *p=transfer_id;
		while (*p)
		{
			*p=tolower(*p);
			p++;
		}
	}

	/* Rckmeldungen Infofiles und CMD weg */
	if (!*mausidbuf && !*errbuf)
		return;

	sprintf (fname, "%s/%s/outgoing/%s", 
		opts.spoolbase, action, transfer_id);

	if (opts.workmode==GATE)
	{
		/* dirty hack fr Gatemodus: Da die Transfer-ID des Batchfiles
		 * als Message-ID mitgeschickt wurde, bekommen wir nun eine
		 * Fehlermeldung fr das Batchfile, und damit die Mglichkeit,
		 * es zu lschen 
		 */
		if (!strchr(transfer_id,'@') && *errbuf)
		{
			unlink(fname);
			return;
		}
	}


	if (stat (fname, &st) != 0)
	{
		/* Oops? Datei nicht vorhanden! */
		/* Versuchen wir noch die .old-Datei */
		strcat (fname, ".old");
		if (stat (fname, &st) != 0)
		{
			/* Geht uns nichts mehr an, die Datei ist weg. Bloss warum? */
			*(strrchr (fname, '.')) = '\0';
			if (opts.workmode==USER)
				log (LOG_NOTICE, "Datei %s verschwunden", fname);
			/* Oder sollten wir bei Mail eine Fehlermeldung schreiben? */
			return;
		}
		is_old = 1;
	}
	
	if (*errbuf && !is_old)
	{
		unsigned char str[PATH_MAX * 3 + 32];
		sprintf (str, "%s/%s/outgoing/%s.old", 
			opts.spoolbase, action, transfer_id);
		rename(fname,str);

		if (opts.workmode==USER)
			log (LOG_NOTICE, "Message %s (first error): %s",
				 fname, start + 1);
		msglog ("Message %s (first error): %s\n",
			 fname, start + 1);
		return;
	}
	else if (*errbuf)
	{
		char cmd[PATH_MAX*3];
		int was_mail=0;
		/* Struktur mit den Daten der gescheiterten Nachricht */
		metamsg_t done;
		/* Alles schoen als leer markieren */
		memset(&done,0,sizeof(metamsg_t));

		/* Einmal per Syslog */
		if (opts.workmode==USER)
			log (LOG_WARNING, "Message %s failed: %s", fname, start + 1);
		/* und ins Logfile */
		msglog( "Message %s failed: %s\n",fname,start+1);

		/* So, jetzt die Message bouncen. Aber vorsichtig: Im Gateway-
		 * betrieb darf das nur fr MAIL geschehen, im Userbetrieb
		 * sollte es fr News und MAIL passieren.
		 */
		was_mail=!strncmp(transfer_id,"mail",4);
		if (was_mail || opts.workmode==USER)
		{
			/* Struktur mit der Bouncemeldung. 
			 * Achtung: Diese Struktur enthaelt keine mit Malloc
			 * allozierten Daten!
			 */
			metamsg_t msg;
			char zwi[1024];
			char ozwi[2048];
			
			metalist_t ml_from;
			metalist_t ml_to;

			memset(&msg,0,sizeof(metamsg_t));

			sprintf(zwi,"MausConnect@%s",opts.mysitedomain);
			msg.env_from=zwi;
			msg.head_from=&ml_from;
			ml_from.next=NULL;
			ml_from.ext=NULL;
			ml_from.text=zwi;
			msg.head_to=&ml_to;
			ml_to.next=NULL;
			ml_to.ext=NULL;
			ml_to.text=msg.env_to;
			msg.subject="Message failed";
			msg.date=time(NULL);
			sprintf(ozwi,
				"Your message was not accepted by %s%s\r\n"
				"Reason: %s\r\n"
				"Failed message follows:\r\n"
				"\r\n",
				opts.pollsite,opts.mausdomain,errbuf);
			meta_add_to_body(&msg,ozwi);
			/* Jetzt *knnte* man noch den Message-Body anhngen, aber
			 * lohnt das?
			 */
			{
				buf_t bouncebuf={NULL,NULL,0};
				if (!rfc_read_message(fname,&bouncebuf) && bouncebuf.start)
				{
					meta_add_to_body(&msg,bouncebuf.start);
				}
				else
				{
					log(LOG_WARNING, "cannot read %s (in bounce)",fname);
					meta_add_to_body(&msg,"Sorry - cannot read message!");
				}
				if (bouncebuf.start)
					free(bouncebuf.start);
			}
			if (msg.env_to)
			{
				srfcmail(&msg);
				msg.subject="Message failed, gatemasters copy";
			}
			else
				msg.subject="Message failed, sender unknown, waking gatemaster";
			msg.env_to=opts.gatemaster;
			srfcmail(&msg);
			/* Bounce abgeschlossen */
		} /* soll gebounced werden? */
		sprintf (cmd, "%s/%s/failed/%s", opts.spoolbase, action, transfer_id);
		if (rename(fname,cmd))
		{
			sprintf(cmd,"mv %s %s/%s/failed/",
				fname,opts.spoolbase,action);
			system(cmd);
		}
		return;
	}
	/* Nachricht war OK.
	 * Im Usermodus nun die ursprngliche Nachricht canceln 
	 */
	if (opts.workmode==USER && isdigit(*transfer_id)
		&& (work_only==ONLY_ALL || work_only==ONLY_NEWS))
	{
		static buf_t buf={NULL,NULL,0};
		if (!rfc_read_message(fname,&buf))
		{
			char *h;
			char *id;
			char *ng;
			id=rfc_find_header(buf.start,"Message-ID");
			ng=rfc_find_header(buf.start,"Newsgroups");
			h=rfc_find_header(buf.start,"Sender");
			if (!h)
				h=rfc_find_header(buf.start,"From");
			if (ng && id && h && !rfc_find_header(buf.start,"X-MausConnect-Env-To"))
			{
				metamsg_t msg;
				
				memset(&msg,0,sizeof(metamsg_t));
				if (!rfc_to_meta(&buf,&msg))
				{
					char *zwi;

					/* Control-Header */

					zwi=XMALLOC(strlen(msg.message_id)
						+sizeof("cancel <>")+1);
					sprintf(zwi,"cancel <%s>",msg.message_id);
					if (msg.control)
						free(msg.message_id);
					msg.control=zwi;

					/* Subject */

					zwi=XMALLOC(strlen(msg.message_id)
						+sizeof("cmsg cancel <>")+1);
					sprintf(zwi,"cmsg cancel <%s>",msg.message_id);
					if (msg.subject)
						free(msg.subject);
					msg.subject=zwi;

					/* Neue Message-ID jetzt erst vergeben */

					zwi=XMALLOC(strlen(msg.message_id)
						+sizeof("cancel-")+1);
					sprintf(zwi,"cancel-%s",msg.message_id);
					free(msg.message_id);
					msg.message_id=zwi;

#define CANCELBODY "Original gecancelt von MausConnect"
					if (msg.body.start 
						&& strlen(msg.body.start)>strlen(CANCELBODY))
						strcpy(msg.body.start,CANCELBODY);

					if (msg.distribution)
						free(msg.distribution);
					if (opts.usenet_dist_local)
					{
						msg.distribution=XSTRDUP(opts.usenet_dist_local);
						srfcnews(&msg);
					}
					meta_clean_message(&msg);
				}
			}
		}
	}
	sprintf (str, "%s/%s/sent2maus/%s", opts.spoolbase, action, transfer_id);
	if (rename(fname,str))
	{
		sprintf (str, "mv %s %s/%s/sent2maus/%s", 
			fname, opts.spoolbase, action, transfer_id);
		system (str);
	}
	msglog ("Message %s done: %s\n", fname, transfer_id);
}

static void
work_log (unsigned char *body)
{
	char *this;
	char *next;
	char *p;
	char *nullp;
	char nullc='\0'; /* keep gcc happy */

	this=next=body;

	while (*this)
	{
		nullp=strpbrk(this,"\r\n");
		if (nullp)
		{
			p=nullp+1;
			while (*p=='\r' || *p=='\n')
				p++;
			next=p;
		}
		else
			next=strlen(this)+this;
		switch(*this)
		{
		case '!': /* Kommentarzeilen und so */
			break;
		case '\"':
			parse_cmdlog(this,&next);
			break;
		case '#':
			parse_msglog(this,&next);
			break;
		case '$': /* Infofilequittungen */
			p=strchr(this,'=');
			if (p)
			{
				*p='\0';
				if (nullp)
				{
					nullc=*nullp;
					*nullp=0;
				}
				info_new_crc(this+1,strtol(p+1,NULL,10));
				if (nullp)
				{
					*nullp=nullc;
				}
				*p='=';
			}
			break;
		default:
			break;
		} /* switch */
		this=next;
	}
}

void
maus2rfc (FILE *f)
{
	unsigned char line[32*1024];    /* Lesepuffer */
	int zurueckgestellt=0;

	int ende;

	do
	{
		ende=maus_import(f,&meta,line,sizeof(line),&zurueckgestellt);
#ifdef FEATURE_NNTP
		if ((meta.env_to || meta.newsgroups) && !meta.maus_special_id
			&& meta.message_id && meta.references)
		{
			char *ref;
			metalist_t *r;
			/* Versuchen, irgendwie die References zu verlngern */
			r=meta.references;
			while (r->next)
				r=r->next;
			ref=nntp_get_headerentry(r->text,"references");
			if (ref)
			{
				metalist_t *new,*last=NULL;
				char *r;
				char *p;
				r=ref;
				while (*r)
				{
					while (*r && *r!='<')
						r++;
					p=strchr(r,'>');
					if (!p)
						break;
					*p='\0';
			
					new=NULL;	
					meta_add_list(&new,r+1,NULL,1);
					if (new)
					{
						if (!last)
						{
							new->next=meta.references;
							meta.references=new;
						}
						else
						{
							new->next=last->next;
							last->next=new;
						}
						last=new;
					}
					*p='>';
					r=p+1;
					while (isspace(*r) || *r==',')
						r++;
				}
			}
		}
#endif
		if (meta.maus_special_id)
		{
			minfo_t *found;
			if (work_only==ONLY_ALL || work_only==ONLY_SPECIAL)
			{

				found=info_find(meta.maus_special_id);

				if (found)
					info_work_new_infofile(&meta,found);
				else if (!strcasecmp(meta.maus_special_id,"LOG"))
				{
					/* #LOG als Mail versenden */
					work_log(meta.body.start);
					meta.env_to=strdup(opts.mailuserdomain);
					COOM(meta.env_to);
				}
				else if (strcasecmp(meta.maus_special_id,"HEAD"))
				{
					/* Alles auer Head mailen */
					meta.env_to=strdup(opts.mailuserdomain);
					COOM(meta.env_to);
				}
			}
		}
		/* NICHT else if (meta.env_to), da work_infofile das 
		 * vielleicht setzt 
		 */
		if (meta.env_to)
		{
			/* mail */
			if (work_only==ONLY_ALL || work_only==ONLY_MAIL
				|| (work_only==ONLY_SPECIAL && meta.maus_special_id))
				srfcmail(&meta);
		}
		else if (meta.newsgroups)
		{
			if (work_only==ONLY_ALL || work_only==ONLY_NEWS)
				srfcnews(&meta);
		}
	} while (!ende);
}

int
main (int argc, char **argv)
{
	FILE *outfile;					/* Outfile.txt for Reading */

	outfile = stdin;

	configure (MAUS2RFC, argc, argv,NULL);
	open_log ("maus2rfc", TRUE);

	if (!grp_lock())
		exit (1);
	if (!grp_load_list())
		exit (1);

	if (info_read_list())
		exit(1);

#if defined(HAVE_TCP) && defined(FEATURE_NNTP)
	if (opts.newshost)
		nntp_open(opts.newshost);
#endif

	maus2rfc (outfile);

	if (!grp_save_list())
	{
		close_log ();
		grp_unlock();
		exit (1);
	}
	grp_unlock();

	srfcmail_end_all();
	srfcnews_end_all();

	info_write_list();
	close_log ();

	exit (0);
}

