
/* 
 * call.c Einfacher Unix-Dialer (c) 92 Kraehe
 * rehacked by uo, 1993
 */

#include "ownstd.h"

#include <sys/types.h>
#include <sys/ioctl.h>
#include <signal.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <ctype.h>
#include <errno.h>
#include <termios.h>
#include <setjmp.h>

#include "mausconlib.h"

#ifndef TANDEM
#define TANDEM 0
#endif

/* ------------------------------------------------------------------ */

char *line = 0;
int modem_i,
  modem_o;
FILE *logfp;

int fehlerpunkte = 0;
int call_done = 0;
time_t verbindung_seit;

struct termios old_linemode;

char lockuucp[PATH_MAX];
int ports_open = 0;
int hold_lock=0;

jmp_buf hup_jumpbuf;

char *global_help_text=
"mcall: Dialer fuer den MausTausch\n"
"Benutzung: mcall [optionen] aktion\n"
;


void prepare (void);
void write_error (char *s);
void add_fp (char *what, int fp);
void free_lock (void);
void get_lock (void);
void log_verbindung(void);

RETSIGTYPE term_handler (int signum);
RETSIGTYPE alarm_handler (int signum);
RETSIGTYPE hup_handler (int signum);
void call_script (void);
void dial_nr (char *to_dial);
void call_shell (char *sh);
int scan_modem (int timeout, char *s1, char *s2, char *s3, char *s4);
void write_modem (char *s);
unsigned char read_modem (void);
void dev_close (void);
int dev_init (void);
void log_init (void);
void eat_typeahead(void);

/* ------------------------------------------------------------------ */

void
write_error (char *s)
{
	write (2, s, strlen (s));
}

/* ------------------------------------------------------------------ */

void
add_fp (char *what, int fp)
{
	char str[512];
	time_t t;

	t = time (NULL);
	fehlerpunkte += fp;
	sprintf (str, "%24.24s: %s, %d Fehlerpunkt%s\n", ctime (&t), what, 
		fehlerpunkte, (fehlerpunkte!=1) ? "e" : "" );
	write_error (str);
	log (LOG_DEBUG, "%s: +%d -> %d Fp", what, fp, fehlerpunkte);
	if (fehlerpunkte >= opts.fp_max)
	{
		write_error ("das war zuviel!\n");
		log (LOG_WARNING, "zu viele (%ld) Fehlerpunkte",
			 (long) fehlerpunkte);
		if (ports_open)
		{
			close (modem_i);
			close (modem_o);
		}
		if (logfp)
			pclose (logfp);
		free_lock ();
		log_verbindung();
		close_log ();
		exit (1);
	}
}

void
prepare (void)
{
	char *tmp;

	tmp=lock_dev_filename(opts.modem_line);
	if (!tmp)
	{
		log (LOG_CRIT, "cannot lock, exiting");
		close_log ();
		exit (1);
	}
	sprintf(lockuucp,"%s/%s",opts.uucplockdir,tmp);
	free(tmp);
	
}

void eat_typeahead(void)
{
	char *iocount;
	char buf[1024];
	int n,z=0;
	ioctl(modem_i, FIONREAD, &iocount);
	n=(int)iocount;
	while(n>0 && z>=0)
	{
		z=read(modem_i,buf,1024>n ? n : 1024);
		if (z>=0)
			n-=z;
	}
}

void
free_lock (void)
{
	if (hold_lock)
		lock_delete(lockuucp);
	hold_lock=0;
}

void
get_lock (void)
{
	int errp=0;
	
	while (1)
	{
		if (!lock_create(lockuucp,&errp))
		{
			hold_lock=1;
			return;
		}
		if (errp)
		{
			/* fatal error */
			log (LOG_ERR, "error locking device (%s)", lockuucp);
			/* und Ende */
			add_fp("cannot lock device",opts.fp_max);
		}
		add_fp ("lockfile detected", opts.fp_lock);
		if (opts.uucplock_sleep>0)
			sleep (opts.uucplock_sleep);
	}
}

void
log_init (void)
{

	if (opts.log_session)
	{
		char str[128];

		sprintf (str, "mail -s call.session %s", opts.gatemaster);
		logfp = popen (str, "w");
		if (!logfp)
		{
			log (LOG_ERR, "%s: %s", str, strerror (errno));
		}
	}
	else
		logfp = 0;
}

int
dev_init (void)
{
	char str[PATH_MAX];
	struct termios term;
	long realspeed = B19200;

	sprintf (str, "/dev/%s", opts.modem_line);
	modem_o = open (str, O_WRONLY | O_NONBLOCK);
	if (modem_o <= 0)
	{
		log (LOG_ERR, "%s: %s", str, strerror (errno));
		return 0;
	}

	tcgetattr(modem_o,&term);
	old_linemode=term;
	term.c_cflag |= HUPCL;
	term.c_cflag |= CLOCAL;
	tcsetattr(modem_o,TCSADRAIN,&term);

#if 1
	modem_i = open (str, O_RDONLY);
#else
	modem_i = open ("/dev/ttyS1", O_RDONLY | O_NONBLOCK);
#endif
	if (modem_i <= 0)
	{
		log (LOG_ERR, "%s: %s", str, strerror (errno));
		close (modem_o);
		return (0);
	}

	ports_open = 1;

	tcgetattr(modem_i,&term);

#if 0
	term.c_lflag &= ~(ECHO);
	term.c_iflag &= ~(ISTRIP | IUCLC);
#endif
	term.c_iflag = 0;
	term.c_oflag = 0;
	term.c_cflag &= ~(CLOCAL);
	term.c_lflag |= ECHO;
#if 0
	linemode.sg_flags |= (RAW | TANDEM);
	linemode.sg_flags &= ~(ECHO | CRMOD);
#endif

	if (opts.modem_speed)
	{							/* User specified a speed? */

		switch (opts.modem_speed)
		{						/* Get internal system code */
		case 110:
			realspeed = B110;
			break;
		case 150:
			realspeed = B150;
			break;
		case 300:
			realspeed = B300;
			break;
		case 1200:
			realspeed = B1200;
			break;
		case 2400:
			realspeed = B2400;
			break;
		case 4800:
			realspeed = B4800;
			break;
		case 9600:
			realspeed = B9600;
			break;
		case 19200:
			realspeed = B19200;
			break;
		case 38400:
			realspeed = B38400;
			break;
		}
	}
	/* term.c_cflag |= (HUPCL | CRTSCTS | CREAD | CS8); */
	term.c_cflag &= ~CBAUD;
	term.c_cflag += realspeed;
	term.c_cflag |= (HUPCL + CRTSCTS + CREAD);
	term.c_cflag = 0xcbf;
	term.c_lflag = 0x8a00;
	tcsetattr(modem_i,TCSADRAIN,&term);


	return (1);
}

void
dev_close (void)
{
	if (ports_open)
	{
		tcsetattr(modem_i,TCSADRAIN,&old_linemode);
		close (modem_i);
		close (modem_o);
		ports_open = 0;
	}
}

/* ------------------------------------------------------------------ */

unsigned char
read_modem (void)
{
	unsigned char ch;
	int count;

	count= read (modem_i, &ch, 1);
	if (count > 0)
	{
		/* Lese ein Zeichen */
		ch = ch & 0177;
		if (logfp)
		{
			write (fileno (logfp), &ch, 1);
		}
		write (1, &ch, 1);		/* Schreibs auf den Schirm */
	}
	else if (count==0)
	{
		/* EOF? */
		if (!call_done)
			kill(getpid(),SIGHUP);
	}
	else
		ch = 0;
	return (ch);
}

void
write_modem (char *s)			/* String aufs Modem schreiben */
{
	char *p;

	p = s;
	while (*p)
	{
		write (modem_o, p, 1);
		p++;
	}
}

int scan_timeout;

RETSIGTYPE
alarm_handler (int signum)
{
	scan_timeout = signum ? signum : signum;
}

/* Warten bis einer der 4 Strings vom Modem gesendet wird*/
int
scan_modem (int timeout, char *s1, char *s2, char *s3, char *s4)
{
	unsigned char ch;
	int scan = 0;
	int i;

#define BSZ 64
	char buffer[BSZ];
	size_t sl1,
	  sl2,
	  sl3,
	  sl4;

	sl1 = sl2 = sl3 = sl4 = 0;	/* to keep gcc quiet */
	if (s1)
		sl1 = strlen (s1);
	if (s2)
		sl2 = strlen (s2);
	if (s3)
		sl3 = strlen (s3);
	if (s4)
		sl4 = strlen (s4);

	memset (buffer, ' ', BSZ);

	signal (SIGALRM, alarm_handler);
	scan_timeout=0;
	if (timeout > 0)
	{
		printf ("\n->scan for %s (timeout %d)\n", s1, timeout);
		alarm (timeout);
	}
	else
	{
		printf ("\n->scan for %s (no timeout)\n", s1);
		alarm (0);				/* endlos */
	}

	while (!scan)				/* Solange kein String gefunden */
	{
		ch = read_modem ();
		if (scan_timeout)
			break;

		for (i = 0; i < BSZ - 1; i++)
			buffer[i] = buffer[i + 1];
		buffer[BSZ - 1] = ch;	/* Drehe den Buffer nach Links */

		/* Schau nach obs stimmt !-) */
		if (s1 && !memcmp (s1, buffer + BSZ - sl1, sl1))
			scan = 1;
		if (s2 && !memcmp (s2, buffer + BSZ - sl2, sl2))
			scan = 2;
		if (s3 && !memcmp (s3, buffer + BSZ - sl3, sl3))
			scan = 3;
		if (s4 && !memcmp (s4, buffer + BSZ - sl4, sl4))
			scan = 4;
	}
	signal (SIGALRM, SIG_DFL);
	return (scan);
}

void
call_shell (char *sh)
{
	struct termios raw;
	struct termios old;

	tcgetattr(0,&raw);
	old=raw;
	raw.c_lflag &= ~(ICANON|ISIG);
	raw.c_lflag &= ~ECHO;
	raw.c_iflag &= ~ICRNL;
	raw.c_oflag &= ~ONLCR;

	tcsetattr(0,TCSADRAIN,&raw);

	printf ("\n\rCalling shell\t%s\n\r", sh);
	system (sh);
	printf ("\n\rBack from shell\n\r");

	tcsetattr(0,TCSADRAIN,&old);

}

/* ------------------------------------------------------------------ */

void
dial_nr (char *to_dial)
{
	int i;
	int scan;
	int done;
	struct termios term;
	time_t startzeit;

	done = 0;
	while (!done)
	{
		eat_typeahead();
		write_modem (opts.modem_reset);	/* Modem zuruecksetzen */
		write_modem ("\r");
		sleep (2);

		scan = scan_modem (opts.modem_timeout, "OK", "ERROR", NULL, NULL);
		switch (scan)
		{
		case 0:
			add_fp ("timeout in modem reset",opts.fp_modem_timeout);
			break;
		case 1:
			done = 1;
			break;
		default:
			add_fp ("modem reset failed", opts.fp_reset);
			break;
		}

	}							/* while !done */
	sleep (1);

	if (opts.modem_init)
	{
		done = 0;
		while (!done)
		{
			eat_typeahead();
			write_modem (opts.modem_init);	/* Modem zuruecksetzen */
			write_modem ("\r");

			scan = scan_modem (opts.modem_timeout, "OK", "ERROR", NULL, NULL);
			switch (scan)
			{
			case 0:
				add_fp ("timeout in modem init",opts.fp_modem_timeout);
				break;
			case 1:
				done = 1;
				break;
			default:
				add_fp ("modem init failed", opts.fp_init);
				break;
			}
		}
	}							/* if modem_init gegeben */

	sleep (1);

	do
	{
		eat_typeahead();
		write_modem (opts.modem_dial);	/* Nummer waehlen */
		write_modem (to_dial);
		write_modem ("\r");
		startzeit=time(NULL);

		scan = scan_modem (opts.dial_timeout, "CONNECT", "BUSY", "NO CARRIER", "NO DIAL");

		if (scan != 1)
		{
			/* besetzt oder Modemfehler */
			switch (scan)
			{
			case 2:
				add_fp ("modem busy", opts.fp_busy);
				break;
			case 3:
				add_fp ("no carrier", opts.fp_no_carrier);
				write_modem(opts.modem_at);
				write_modem("\r");
				break;
			case 4:
				add_fp ("no dialtone", opts.fp_no_dial);
				break;
			case 0:
				add_fp ("timeout", opts.fp_dial_timeout);
				write_modem(opts.modem_at);
				write_modem("\r");
				break;
			default:
				add_fp ("unknown error", opts.fp_error);
				write_modem(opts.modem_at);
				write_modem("\r");
				break;
			}
			printf ("%d Sekunden warten\n", opts.dial_retry_time);
			i = opts.dial_retry_time;
			while (--i > 0)
			{					/* Damit der User nicht so ungeduldig */
				sleep (1);		/* ist, bekommt er jede Sekunde einen */
				write (2, ".", 1);	/* Punkt zu sehen :-) */
			}
		}						/* scan != 1 */
	}
	while (scan != 1);			/* Endlich CONNECT !-) */
	verbindung_seit=startzeit;


	tcgetattr(modem_o,&term);
	old_linemode=term;
	term.c_cflag &= ~(CLOCAL);
	tcsetattr(modem_o,TCSADRAIN,&term);

	log (LOG_INFO, "Verbindung steht");

	write (2, "\007\007\007", 3);	/* Mittbewohner nerfen ;-) */

}

void
call_script (void)
{
	FILE *script;
	char cmd[PATH_MAX];
	char *p;

	sprintf (cmd, "%s/call.%s", libdirname, action);
	script = fopen (cmd, "r");
	if (!script)
	{
		add_fp ("cannot open script", opts.fp_max + 1);
		close_log ();
		exit (1);
	}

	while (fgets (cmd, 127, script))
	{
		/* \n weg */
		if (cmd[strlen (cmd) - 1] == '\n')
			cmd[strlen (cmd) - 1] = 0;
		p = cmd;
		while (strchr (" \t", *p))
			p++;

		if (strhead (p, "dial "))
			dial_nr (p + 5);
		if (strhead (p, "eat "))
			eat_typeahead();
		if (strhead (p, "log "))
			log (LOG_INFO, "%s", p + 4);
		if (strhead (p, "write "))
		{
			write_modem (p + 6);
		}
		if (strhead (p, "writeln "))
		{
			write_modem (p + 8);
			write_modem ("\r");
		}
		if (strhead (p, "scan "))
			scan_modem (0, p + 5, (char *) 0, (char *) 0, (char *) 0);
		if (strhead (p, "tscan "))
		{
			char *p1;
			char *ptime;

			ptime=p+6;

			p1 = ptime;
			while (*p1 != '\0' && *p1 != ' ' && *p1 != '\t')
				p1++;
			if (*p1 == '\0')
				log (LOG_WARNING, "illegal scriptcommand %s", p);
			else
			{
				int res;
				res=scan_modem (atoi (ptime), p1 + 1, (char *) 0, (char *) 0, (char *) 0);
			}
		}
		if (strhead (p, "shell "))
			call_shell (p + 6);
		if (strhead (p, "done"))
			call_done = 1;
	}
	fclose (script);
}

/* ------------------------------------------------------------------ */

RETSIGTYPE
term_handler (int signum)
{
	log (LOG_CRIT, "exiting because of signal %d", signum);
	dev_close ();
	free_lock ();
	log_verbindung();
	close_log ();
	exit (1);
}

RETSIGTYPE
hup_handler (int signum)
{
	log (LOG_NOTICE, "got hangup %d", signum);
	log_verbindung();
	add_fp ("SIGHUP", opts.fp_no_carrier);
	longjmp(hup_jumpbuf,1); /* verliert ein Filehandle! */
}

void 
log_verbindung(void)
{
	if (verbindung_seit)
	{
		log(LOG_INFO,"Verbindung dauerte %ld Sekunden",
			(long) (time(NULL)-verbindung_seit));
		verbindung_seit=0;
	}
}

int
main (int argc, char **argv)
{
	configure (CALL, argc, argv,NULL);

	prepare ();

	log_init ();
	open_log ("mcall", FALSE);
	log (LOG_INFO, "Anruf fr %s beginnt", action);

#if 0
	setsid ();
#endif

	signal (SIGTERM, term_handler);
	signal (SIGHUP, hup_handler);
	signal (SIGINT, term_handler);

	if (setjmp(hup_jumpbuf))
	{
		dev_close();
		free_lock();
		signal (SIGHUP, hup_handler);
	}

	while (!call_done)
	{
		get_lock ();			/* exit, wenn kein lock! */
		if (!dev_init ())
		{
			add_fp ("device init", opts.fp_dev_init);
			free_lock ();
		}
		else /* if dev_init ok */
		{
			call_script ();
			dev_close ();
			free_lock ();
		}						/* if dev_init nicht ok */
	}							/* while not call_done */
	log_verbindung();
	log (LOG_INFO, "Anruf fr %s beendet", action);

	close_log ();

	if (logfp)
		pclose (logfp);
	exit (0);
}
