/* 
 * mcom - Kommandos an MausConnect 
 */

#include "ownstd.h"

#include <signal.h>

#include "mausconlib.h"
#include "xmalloc.h"
#include "groups.h"

char *global_help_text=
"Benutzung: mcom aktion kommando [kommando-parameter]*\n"
"  Aktion: siehe mauscon.cnf\n"
"  Kommandos:\n"
"  newgroup usenetname MausNet flag [followup] -- Gruppe anlegen\n"
"  rmgroup usenetname                          -- Gruppe entfernen\n"
"  setgroup usenetname groupdata               -- Gruppenparameter aendern\n"
"    groupdata:\n"
"      mausnetname new-mausnetname   -- MausNetnamen aendern\n"
"      usenetname new-usenetname     -- Gruppennamen Usenet aendern\n"
"      box new-box                   -- Pollbox aendern\n"
"      info new-info                 -- Infotext aendern\n"
"      followup new-followup         -- Defaultfollowup setzen\n"
"      approve new-address           -- Approve-Adresse setzen\n"
"      mausnetdist new-distribution  -- MausNetdistribution setzen\n"
"      usenetdist new-distribution   -- Usenetdistribution setzen\n"
"      mime [header|none]            -- MIME-Kodierung setzen\n"
"      checkgroups [true|false]      -- Checkgroupsflag setzen\n"
"      itg [true|false]              -- ITG-Flag setzen\n"
;

volatile void parameterfehler(void);
BOOLEAN cmd_newgroup(char *rfc, char *maus, char *flag, char *followup);
BOOLEAN cmd_rmgroup(char *rfc);
BOOLEAN load_groups(void);
void save_groups(void);

volatile void 
parameterfehler(void)
{
	fprintf(stderr,"%s'mcom help' for more informations\n",global_help_text);
	exit(2);
}

int
main (int argc, char **argv)
{
	int pos;
	int paras;
	configure (MCOM, argc, argv,&pos);
	if (!argv[pos])
	{
		fprintf(stderr,"usage: %s action command parameter\n",	
			argv[0]);
		fprintf(stderr,
			"  use action or command 'help' to get a list of commands\n");
		exit(1);
	}

	paras=argc-pos-1;
	if (paras<1)
		parameterfehler();
	if (!strcasecmp(argv[pos],"newgroup"))
	{
		if (paras<3 || paras >4)
			parameterfehler();
		exit (!cmd_newgroup(argv[pos+1],argv[pos+2],argv[pos+3],argv[pos+4]));
	}
	else if (!strcasecmp(argv[pos],"rmgroup"))
	{
		if (paras>1)
			parameterfehler();
		exit (!cmd_rmgroup(argv[pos+1]));
	}
#if 0
"  setgroup usenetname groupdata               -- Gruppenparameter aendern\n"
"      mausnetname new-mausnetname   -- MausNetnamen aendern\n"
"      usenetname new-usenetname     -- Gruppennamen Usenet aendern\n"
"      box new-box                   -- Pollbox aendern\n"
"      info new-info                 -- Infotext aendern\n"
"      followup new-followup         -- Defaultfollowup setzen\n"
"      approve new-address           -- Approve-Adresse setzen\n"
"      mausnetdist new-distribution  -- MausNetdistribution setzen\n"
"      usenetdist new-distribution   -- Usenetdistribution setzen\n"
"      mime [header|none]            -- MIME-Kodierung setzen\n"
"      checkgroups [true|false]      -- Checkgroupsflag setzen\n"
"      itg [true|false]              -- ITG-Flag setzen\n"
	}]
#endif
	parameterfehler();
	exit(0); /* NOTREACHED */
}

BOOLEAN
load_groups(void)
{
	if (!grp_lock())
		return FALSE;
	if (!grp_load_list())
	{
		grp_unlock();
		return FALSE;
	}
	return TRUE;
}

void 
save_groups(void)
{
	grp_save_list();
	grp_unlock();
}

BOOLEAN 
cmd_newgroup(char *rfc, char *maus, char *flag, char *followup)
{
	grp_data_t *grp;

	if (!load_groups())
		return FALSE;

	grp=grp_suche_rfcname(rfc,opts.pollsite);
	if (!grp)
	{
		grp_data_t *new;
		new=XMALLOC(sizeof(grp_data_t));
		*new=leer_grp;
		new->rfcname=XSTRDUP(rfc);
		new->mausname=XSTRDUP(maus);
		if (followup)
			new->default_followup=XSTRDUP(default_followup);
		grp_append_new(new);
	}
	else
	{
		if (grp->default_followup)
		{
			free(grp->default_followup);
			grp->default_followup=XSTRDUP(default_followup);
		}
		if (grp->mausname)
		{
			free(grp->mausname);
			grp->mausname=XSTRDUP(mausname);
		}
	}
	grp_save_list();
	grp_unlock();
}

BOOLEAN 
cmd_rmgroup(char *rfc)
{
	char zwi[1024];
	grp_data_t *grp;

	if (!load_groups())
		return FALSE;

	grp=grp_suche_rfcname(rfc,opts.pollsite);
	if (!grp)
	{
		grp=grp_suche_mausname(rfc,opts.pollsite);
		if (grp)
			fprintf(stderr,"%s: Die Gruppe heit %s\n",grp->mausname);
		else
			fprintf(stderr,"%s: Gruppe nicht gefunden\n",rfc);
		grp_unlock();
		return FALSE;
	}

	printf("Control: rmgroup %s\n",grp->rfcname);
	printf("Newsgroups: %s\n",grp->rfcname);
	printf("From: %s\n",opts.mailuserdomain);
	printf("Path: not-for-mail\n");
	printf("Subject: cmsg rmgroup %s\n",grp->rfcname);
	printf("Message-ID: rmgroup-%s-%s@%s",
		grp->rfcname,opts.mailuser,opts.mysitedomain);
	rfc_make_date(zwi,sizeof(zwi),time(NULL));
	printf("Date: %s\n",zwi);
	printf("Approved: %s\n",opts.mailuser);
	printf("Distribution: %s\n",opts.usenet_dist_local);
	printf("\n");
	printf( "rmgroup created by mcom for %s (%s)\n",
		opts.mailuserdomain,
		getenv("LOGNAME")? getenv(LOGNAME) : "No Name");

	/* Und aus der eigenen Gruppenliste austragen! */
	grp_unlock();
	return TRUE;
}
