
/* spool4maus -> Spoolt News+Mail fr MausNet */

#include "ownstd.h"

#include <errno.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <ctype.h>

#include "mausconlib.h"
#include "metamsg.h"
#include "rfc.h"
#include "route.h"
#include "xmalloc.h"

static char error_to[1024];
static char gm_addr[256];
static int is_routing=0;
static int got_cancels=0;

static long get_transfer_id (void);
static int xwrite(int fd, char *p, size_t i);
static int handle_rmail(char *from, char *to);
static int handle_news(void);
static int handle_rnews(void);
static void	test_routing(void);

char *global_help_text=
"mroute: Router fuer MausConnect\n"
"Benutzung: mroute [optionen] aktion\n"
;

buf_t mbuf;


static int
work_cancel(char *s)
{
	char *p;
	while (isspace(*s))
		s++;
	while (s && *s)
	{
		p=strchr(s,' ');
		if (*s!='<')
			return 1; /* das ist keine ID */
		if (p)
		{
			if (p[-1]=='>')
				p[-1]='\0';
			else
				return 1;
		}
		else
		{
			char *p2;
			p2=s+strlen(s)-1;
			if (*p2!='>')
				return 1;
			*p2='\0';
		}
		if (got_cancels++==0)
		{
			cancel_init(1);
		}
		cancel_add(s+1);
		s=p;
		if (s)
		{
			while (isspace(*s))
				s++;
		}
	} /* while (s && *s) */
	return 0;
}  /* work cancel */

/*
**  Keep writing until everything has been written or we get an error.
*/
static int
xwrite(int fd, char *p, size_t i)
{
	int        c;

	for ( ; i; p += c, i -= c)
	{
		c = write(fd, p, (size_t) i);
		if (c <= 0)
			return -1;
	}
	return 0;
}

static long
get_transfer_id (void)
{
	int count;
	int err;
	long mid;
	char midname[2 * PATH_MAX];
	char lockname[2 * PATH_MAX];
	char str[255];
	FILE *f;

	sprintf (midname, "%s/%s/transfer.id", opts.spoolbase, action);
	sprintf (lockname, "%s.LCK", midname);
	count = 0;
	err = 0;
	while (1)
	{
		if (!lock_create(lockname,NULL))
			break;
		count++;
		if (opts.lock_tries && count>=opts.lock_tries)
		{
			errormail (NULL,
				error_to ?  "error spooling message"
				:	"error spooling message - sender unknown",
		   		"couldn't get a transfer id (spooling message failed)",
		   gm_addr, *error_to ? (error_to) : (NULL) , NULL);
			exit (1);
		}
		if (opts.lock_sleep > 0)
			sleep (opts.lock_sleep);
	}
	f = fopen (midname, "r+");
	if (!f)
	{
		/* einfach waere es, jetzt 0 zurueckliefern. Haken dabei:
		   Falls die Datei mal irgendwie "verschwindet", wuerden neue
		   Nachrichten unter den alten IDs erzeugt, und zumindest der
		   Dupecheck der Quark II findet das nicht so toll.
		   Also nehmen wir einen von der aktuellen Uhrzeit bzw. Datum
		   abhaengigen Wert. Problem: User-IDs in MAUS duerfen nur
		   bis zu 10 Zeichen lang sein. Da der Prefix "mail" vor
		   Mails aber schon vier Zeichen belegt, kann der 32-Bit-Wert
		   von time() nicht direkt verwendet werden (ausser man 
		   verkuerzt den Prefix, und das halte ich nicht fuer sehr
		   benutzerfreundlich/wartungsfreundlich). Nimmt man aber den
		   Rest einer Division des time()-Rueckgabewerts durch eine
		   Million, sollten Kollisionen hinreichend unwahrscheinlich
		   sein.
		*/
		mid=time(NULL) % 1000000;
		f=fopen(midname,"w");
		if (!f)
		{
			errormail (NULL,
				error_to ?
			   		"error spooling message"
				:	"error spooling message - sender unknown",
			   "couldn't write transfer id (spooling message failed)",
			   gm_addr, *error_to ? (error_to) : (NULL) , NULL);
			exit (1);
		}
	}
	else
	{
		fgets (str, sizeof (str) - 1, f);
		mid = atol (str) + 1;
		rewind (f);
	}
	fprintf (f, "%ld", mid);
	fclose (f);
	lock_delete(lockname);
	return mid;
}


static int 
handle_rmail(char *from, char *to)
{
	int fd;
	int i;
	long id;
	char fname[PATH_MAX];
	char env_buf[2048];
	char *route;
	/* alles wissenswerte haben wir schon aus der Kommandozeile
	 * entnommen.
	 */
	if (!mail_to)
	{
		log(LOG_ERR,"Empfnger von Mail nicht angegeben!\n");
		exit(1);
	}
	if (argument_from)
		strcpy(error_to,argument_from);
	else
	{
		log(LOG_ERR,"Absender von Mail nicht angegeben!\n");
		exit(1);
	}
	if (argument_test)
	{
		route=route_it(argument_from,mail_to);
		if (route)
			printf("route for %s to %s via %s\n",argument_from,
				mail_to,route);
		else
			printf("no route for %s to %s\n",argument_from,mail_to);
		return 0;
	}

	/* 
	 * Nun die Nachricht lesen 
	 */
	if (inputfilename)
	{
		/* Dann Nachricht aus der Datei lesen */
		if (rfc_read_message(inputfilename,&mbuf))
		{
			/* Hm? Eingabedatei nicht da? */
			log(LOG_ERR,"%s: %s\n", inputfilename, strerror (errno));
			errormail (NULL,
			   "weird: cannot read input file",
			   strerror (errno),
			   gm_addr, *error_to ? (error_to) : (NULL) , NULL);
			exit (1);
		}
	}
	else
	{
		/* Dann Nachricht von stdin lesen */
		if (rfc_read_message_fd(0,&mbuf))
		{
			/* Hm? Eingabedatei nicht da? */
			log(LOG_ERR,"stdin: %s\n", strerror (errno));
			errormail (NULL,
			   "weird: cannot read stdin",
			   strerror (errno),
			   gm_addr, *error_to ? (error_to) : (NULL) , NULL);
			exit (1);
		}
	}
	/* 
	 * Die Nachricht ist jetzt also im Puffer. Nun routen.
	 */
	if (is_routing)
	{
		route=route_it(argument_from,mail_to);
		if (!route && opts.reroute_host)
		{
			/* Mail via eine andere Kiste schicken */
			char adr[1024];
			char cmd[2048];
			char *p;
			FILE *wfd;
			strcpy(adr,mail_to);
			p=strchr(adr,'@');
			if (p)
				*p='%';
			strcat(adr,"@");
			strcat(adr,opts.reroute_host);
			sprintf(cmd,"rmail '%s'",adr);
			wfd=popen(cmd,"w");
			if (wfd)
			{
				fwrite(mbuf.start,strlen(mbuf.start),1,wfd);
				exit(0);
			}
		}
		if (!route)
		{
			/* Mail immer bouncen */
			bounce(argument_from,"You are not allowed to use the gateway",
				mbuf.start);
			exit(0);
		}
		action=route;
	}

	i=0;
	do
	{
		/* 
		 * eine Transfer-ID holen 
		 */
		id = get_transfer_id ();
		/* 
		 * In welches File spoolen? 
		 */
		sprintf (fname, "%s/%s/outgoing/mail%ld", 
			opts.spoolbase, action, (long) id);
		fd=open(fname,O_WRONLY|O_CREAT|O_EXCL,0640);
	} while (i<10 && fd<0);

	if (fd<0)
	{
		/* Mail immer bouncen */
		bounce(argument_from,"Cannot open spool file",
			mbuf.start);
		errormail (NULL,
		   "weird: cannot open spool file",
		   strerror (errno),
		   gm_addr, NULL);
		exit(0);
	}

	sprintf(env_buf,
		"X-MausConnect-Env-To: %s\n"
		"X-MausConnect-Env-From: %s\n",
		mail_to,
		argument_from);

	if (xwrite(fd,env_buf,strlen(env_buf))<0 
		|| xwrite(fd,mbuf.start,strlen(mbuf.start))<0)
	{
		close(fd);
		unlink(fname);
		/* Mail immer bouncen */
		bounce(argument_from,"Cannot spool message",
			mbuf.start);
		errormail (NULL,
		   "weird: cannot spool mail",
		   strerror (errno),
		   gm_addr, NULL);
		exit(0);
	}

	close(fd);
	exit(0);
}

static int 
handle_news(void)
{
	char *from;
	metalist_t *ml;
	char *route;
	long id;
	char fname[PATH_MAX];
	int ok;
	int fd;
	int i;

	/* Bei News mssen alle wesentlichen Daten aus der Nachricht entnommen
	 * werden.
	 */
	if (inputfilename == NULL)
	{
    	if (rfc_read_message_fd(0,&mbuf))
    	{
			/* Hm? Eingabedatei nicht da? */
			log(LOG_ERR,"stdin: %s\n", strerror (errno));
			errormail (NULL,
			   "weird: cannot read stdin",
			   strerror (errno),
			   gm_addr, NULL);
			return (1);
    	}
    }
    else
    {
		/* Dann Nachricht aus der Datei lesen */
    	if (rfc_read_message(inputfilename,&mbuf))
    	{
			/* Hm? Eingabedatei nicht da? */
			log(LOG_ERR,"%s: %s\n", inputfilename, strerror (errno));
			errormail (NULL,
			   "weird: cannot read input file",
			   strerror (errno),
			   gm_addr, NULL);
			return 1;
    	}
    }
    /* 
     * Die Nachricht ist also vorhanden 
     * Jetzt Absender und Gruppe feststellen.
     */
    from=rfc_find_header(mbuf.start,"From");
    if (!from)
    {
		errormail (NULL,
		   "weird: news without From?",
		   strerror (errno),
		   gm_addr, NULL);
		return 1;
    }
    ml=NULL;
    if (rfc_adress_to_metalist(1,&ml,from)||!ml || !ml->text)
    {
		errormail (NULL,
		   "weird: cannot parse From header",
		   strerror (errno),
		   gm_addr, NULL);
		return 1;
    }
    strcpy(from,ml->text);

	if (is_routing)
	{
		route=route_it(from,NULL);
		if (argument_test)
		{
			if (route)
				printf("route for news from %s via %s\n",from, route);
			else
				printf("no route for news from %s\n",from);
			return 0;
		}
	    if (!route)
	    {
	    	/* das bedeutet, da wir noch nicht einmal wissen, ob
	    	 * die Nachricht ber ein Gate oder lokal kam. Die 
	    	 * Nachricht darf auch nicht "auf Verdacht" gebounced
	    	 * werden (wg. Gatewaybetrieb).
	    	 * Der Fall, da keine Route gefunden wird, kommt sehr hufig
	    	 * vor, wenn man Nachrichten aus dem Usenet bekommt mit
	    	 * Crosspostings in Maus-Gruppen. Von daher kann man die
	    	 * Warnung ausschalten :-)
	    	 */
	    	if (opts.warn_news_routing)
	    	{
				errormail (NULL,
					"weird: cannot find a route for news from",
					from,
				   gm_addr, NULL);
			}
	    	return 1;
	    }
	    action=route;
	}

	/* Diverse Formen von Control-Headern bearbeiten */
	{	
		char *control;
		char *also_control=NULL;
		char *supersedes=NULL;
		char *vgl;
		control=rfc_find_header(mbuf.start,"control");
		if (!control)
			also_control=rfc_find_header(mbuf.start,"also-control");
		if (!control && !also_control)
			supersedes=rfc_find_header(mbuf.start,"supersedes");
		if (control || also_control || supersedes)
		{
			int exit_after=0;
			if (control)
				exit_after=1;
			if (supersedes)	
				work_cancel(supersedes);
			if (control)
			{
				vgl=control;
				if (!strncasecmp(vgl,"cancel ",7))
					work_cancel(vgl+7);
				exit_after=1;
			}
			if (also_control)
			{
				vgl=also_control;
				if (!strncasecmp(vgl,"cancel ",7))
					work_cancel(vgl+7);
			}
			if (exit_after)
				return 0;
		}
	}

	ok=0;
	fd=-1; /* keep gcc happy */
	i=0;
	do
	{
		/* 
		 * eine Transfer-ID holen 
		 */
		id = get_transfer_id ();
		/* 
		 * In welches File spoolen? 
		 */
		sprintf (fname, "%s/%s/outgoing/%ld", 
			opts.spoolbase, action, (long) id);
		if (!inputfilename)
		{
			fd=open(fname,O_WRONLY|O_CREAT|O_EXCL,0640);
			if (fd>=0)
				ok=1;
		}
		else
		{
			if (symlink(inputfilename,fname)==0)
				ok=1;
		}
	} while (i<10 && !ok);

	if (!ok)
	{
		if (inputfilename)
		{
			errormail (NULL,
				"weird: cannot spool news",
				strerror(errno),
			   gm_addr, NULL);
		}
		else
		{
			errormail (NULL,
				"weird: cannot create symlink",
				strerror(errno),
			   gm_addr, NULL);
		}
		return(1);
	}
	if (inputfilename)
	{
		return 0;
	}
	if (xwrite(fd,mbuf.start,strlen(mbuf.start))<0)
	{
		close(fd);
		unlink(fname);
		/* News smart bouncen */
		errormail (NULL,
		   "weird: cannot spool message",
		   strerror (errno),
		   gm_addr, NULL);
		return 1;
	}
	close(fd);
	return 0;
}

static int 
handle_rnews(void)
{
	char fname[PATH_MAX];
	int fdin;
	int fdout;
	int i;
	unsigned long id;
	char buf[64*1024];
	size_t got;

	/* einen Newsbatch nehmen und komplett spoolen */

	i=0;
	do
	{
		/* 
		 * eine Transfer-ID holen 
		 */
		id = get_transfer_id ();
		/* 
		 * In welches File spoolen? 
		 */
		sprintf (fname, "%s/%s/outgoing/nb%lu", 
			opts.spoolbase, action, id);
		fdout=open(fname,O_WRONLY|O_CREAT|O_EXCL,0640);
		i++;
	} while (i<10 && fdout<0);

	if (fdout<0)
	{
		errormail (NULL,
			"weird: cannot spool newsbatch",
			strerror(errno),
		   gm_addr, NULL);
		return(1);
	}

	if (inputfilename == NULL)
		fdin=STDIN_FILENO;
	else
	{
		fdin=open(inputfilename,O_RDONLY);
		if (fdin<0)
		{
			/* Hm? Eingabedatei nicht da? */
			log(LOG_ERR,"%s: %s\n", inputfilename, strerror (errno));
			errormail (NULL,
			   "weird: cannot read input file",
			   strerror (errno),
			   gm_addr, NULL);
			return 1;
    	}
    }

    while ((got=read(fdin,buf,sizeof(buf)))>0)
    {
		if (xwrite(fdout,buf,got)<0)
		{
			close(fdout);
			unlink(fname);

			errormail (NULL,
			   "weird: cannot spool newsbatch",
			   strerror (errno),
			   gm_addr, NULL);
			return 1;
		}
    }
	close(fdin);
	return 0;
}

static void
test_routing(void)
{
	char from[1024];
	char to[1024];
	char *p;
	char *route;
	while (1)
	{
		printf("from> ");
		if (!fgets(from,sizeof(from),stdin))
			break;
		p=strpbrk(from,"\r\n");
		if (p)
			*p='\0';
		if (!*from)
			break;
		printf("  to> ");
		if (!fgets(to,sizeof(to),stdin))
			break;
		p=strpbrk(to,"\r\n");
		if (p)
			*p='\0';
		if (!*to)
			break;
		route=route_it(strtrim(from),strtrim(to));
		if (route)
			printf(" via %s\n",route);
		else
			printf(" no way\n");
	}
}

int
main (int argc, char **argv)
{
	int ret;
	error_to[0]='\0';
	sprintf (gm_addr, "=%s", opts.gatemaster);

	configure (MROUTE, argc, argv,NULL);

	if (!strcasecmp(action,"route"))
		is_routing=1;

	open_log ("mroute", TRUE);

	if (argument_route_create)
	{
		exit(route_make("mroute", libdirname, conffilename));
	}
	if (route_open())
	{
		log(LOG_ERR,"cannot open routing database\n");
		exit(1);
	}
	if (argument_route_show)
	{
		ret=route_show(action);
	}
	else if (argument_route_test)
	{
		test_routing();
		ret=0;
	}
	else if (this_is_mail_flag)
	{
		ret=handle_rmail(argument_from,mail_to);
	}
	else if (argument_is_rnews)
		ret=handle_rnews();	
	else
		ret=handle_news();	
	if (got_cancels)
		cancel_end(0,1);
	exit (ret);
}
