
/* rfc2maus - konvertiert rfc (news,mail) nach Maus */

#include "ownstd.h"

#include <errno.h>
#include <ctype.h>
#include <fcntl.h>
#ifdef HAVE_SYS_STAT_H
#  include <sys/stat.h>
#endif

#include "mausconlib.h"
#include "metamsg.h"
#include "rfc.h"
#include "maus.h"
#include "groups.h"
#include "lib/rfcaddr.h"
#include "lib/xmalloc.h"
#ifdef HAVE_TCP
#ifdef FEATURE_NNTP
#include "lib/nntp.h"
#endif
#endif

/* Weil MAUS \r zwingend braucht */
#define LINEEND "\r\n"

char *global_help_text=
"rfc2maus: RFC -> MausTausch Konverter\n"
"Benutzung: rfc2maus [optionen] aktion >infile\n"
;

FILE *outputfile;

char *unlink_fname;

static void convert_messages (metamsg_t *meta);
static void write_cmd(void);
static BOOLEAN rfc2maus(FILE *, metamsg_t *meta);
static void convert_newsbatches (metamsg_t *meta);
static void	append_mausbatches(void);
static int read_and_convert_file (char *fname, buf_t *mbuf, metamsg_t *meta);

static int
read_and_convert_file (char *fname, buf_t *mbuf, metamsg_t *meta)
{
	char *transferidp;
	char *p;

	meta_clean_message(meta);

	/* Mail oder news? Erkennbar am Namen. */
	p=strrchr(fname,'/');
	if (!p)
		p=fname;
	else
		p++;
	transferidp=p;

	/* alles was mit "m" anfngt ist mail */
	if (*p=='m')
		meta->is_mail=1;

	/*
	 * Die Nachricht aus der Datei einlesen in den Puffer (der bei
	 * Bedarf vergrert wird, derhalber mbufsize und die indirekten
	 * Zeiger).
	 */
	if (rfc_read_message(fname,mbuf)!=0)
	{
		/* Pech gehabt, Nachricht nicht lesbar */
		struct stat sb;
		if (lstat(fname,&sb)==0)
		{
			/* wenn symbolischer Link, dann weg damit */
			if ((sb.st_mode & S_IFMT) == S_IFLNK)
				unlink(fname);
		}
		return 1;
	}

	/*
	 * Die Nachricht im Puffer ins Metamessageformat zerlegen.
	 */
	if (!strcmp (".old", fname + strlen (fname) - 4))
	{
		char *oldp;
		oldp=strrchr(fname,'.');
		*oldp='\0';
		meta->transfer_id=XSTRDUP(transferidp);
		*oldp='.';
	}
	else
		meta->transfer_id=XSTRDUP(transferidp);
	if (rfc_to_meta(mbuf,meta))
	{
		/* Fehler beim Umwandeln */
		return -1;
	}
	return 0;
}

static BOOLEAN
rfc2maus(FILE *outputfile, metamsg_t *msg)
{
	cancel_t *cancel;
	unsigned char *error_text = NULL;
	int errors_detected;

	errors_detected = 0;
	error_text=NULL;


	cancel=cancel_find_in_list(msg->message_id);
	if (cancel)
	{
		/* Eine Cancel-Nachricht auf die zu verarbeitende Nachricht ist 
		 * also schon eingetroffen. 
		 */		
		msglog("%s: canceled\n",msg->message_id);
		if (unlink_fname)
			unlink(unlink_fname);
		cancel->done=1;
		return TRUE;
	}

	/*
	 * Der Header ist in die Datenstrukturen konvertiert worden.
	 * Nun Vollstndigkeit prfen. Allzu viel ist nicht zu tun.
	 * - bei Mail wird die Angabe der Envelope-Adressen geprft, und
	 *   geprft, ob der Envelope-Absender durch das Gate schreiben darf.
	 * - bei News wird geprft, ob eine der Gruppen bekannt ist, ob
	 *   From oder Sender durch das Gate schreiben darf.
	 * Diese Prfungen mssen wohl beim Boxmodus anders gehandhabt werden
	 * (eine Nachricht, die bis hier kommt, ist als OK anzusehen?)
	 */
	 if (msg->is_mail)
	 {
	 	if (!msg->env_to)
	 	{
			errors_detected++;
			error_text = "Kein Empfnger in Envelope";
	 	}

	 	if (!msg->env_from)
	 	{
			errors_detected++;
			error_text = "Kein Absender im Envelope";
	 	}
	 	else if (!is_allowed(msg->env_from))
		{
			errors_detected++;
			error_text = "Envelope-Absender darf das Gate nicht benutzen";
		}
	 }
	 else
	 {
	 	/* News */
	 	if (msg->newsgroups)
	 	{
	 		metalist_t *ng;
	 		BOOLEAN found=FALSE;
	 		ng=msg->newsgroups;
	 		while(ng)
 			{
	 			grp_data_t *grp;
				grp = grp_suche_rfcname (ng->text,opts.pollsite);
				if (grp)
				{
					found=TRUE;
					if (grp->mausnet_distribution)
					{
						if (msg->distribution)
						{
							/* Wenn die Distribution nicht schon lokal ist */
							if (strcasecmp(msg->distribution, opts.usenet_dist_local))
							{
								free(msg->distribution);
								msg->distribution=XSTRDUP(grp->mausnet_distribution);
							}
						}
						else
							msg->distribution=XSTRDUP(grp->mausnet_distribution);
					}
				}
 				ng=ng->next;
	 		}
	 		if (!found)
	 		{
				/* 
				 * Dummyeintrag in der Namensliste erzeugen, damit ber
				 * einen Gruppenamen nur einmal gemeckert wird.
				 */
	 			grp_data_t *grp;
				grp=XMALLOC(sizeof(grp_data_t));
				*grp=leer_grp;
				grp->mausname=XSTRDUP(msg->newsgroups->text);
				grp->rfcname=XSTRDUP(msg->newsgroups->text);
				grp->boxen=XSTRDUP(opts.pollsite);
				grp->dont_save=1;
				grp_append_new(grp);

				error_text = "Kein Mausgruppenname gefunden";
				errors_detected++;
			}
	 	}
	 	else
	 	{
			errors_detected++;
			error_text = "Keine Gruppe angegeben";
	 	}

	 	if (msg->date==(time_t) 0 || msg->date==(time_t) -1)
	 	{
			errors_detected++;
			error_text = "Datum nicht angegeben oder nicht wandelbar";
	 	}

	 	if (!msg->head_from || !msg->head_from->text)
	 	{
	 		/* Keine From-Zeile? Laut 1036 verboten */
			errors_detected++;
			error_text = "From nicht angegeben";
	 	}
	 	else
	 	{
	 		/* news haben laut 1036-Nachfolger nur einen Sender/From */
	 		int ok=0;
	 		if (is_allowed(msg->head_from->text))
	 			ok=1;
	 		else if (msg->sender && msg->sender->text)
	 		{
	 			if (is_allowed(msg->sender->text))
	 				ok=1;
	 		}
	 		if (!ok)
	 		{
				errors_detected++;
				error_text = "From [& Sender] nicht erlaubt";
				if (opts.unlink_unallowed && unlink_fname)
					unlink(unlink_fname);
			}
		}
	}
	if (opts.workmode==USER && !msg->is_mail)
	{
		/* Wiedereinleitungen von Nachrichten vermeiden - das *sollte*
		 * zwar auch durch die Absenderkontrolle abgefangen werden,
		 * aber da es so leicht ist, es Newssystem zu verkonfigurieren,
		 * ist diese Paranoia zwecks Dupevermeidung sinnvoll.
		 */
		if (msg->path && strstr(opts.pollsite,msg->path))
		{
			errors_detected++;
			error_text = "Wiedereinleitung vermieden";
		}
	}

	if (errors_detected != 0)
	{
		/* Ob stderr der richtige Platz ist??? */
		if (msg->message_id)
			fprintf (stderr, "ID:%s\n", msg->message_id);

		fprintf (stderr, "%d errors detected\n", errors_detected);
		fprintf (stderr, "one of them: %s\n", error_text);
		return FALSE;
	}

	/***********************************************************************
	*
	*                    Nun Ausgabe im MausTausch-Format
	*
	************************************************************************/
	maus_export(outputfile,msg);

	return TRUE;
}

int
main (int argc, char **argv)
{
	/* ### 0 mu nicht gleich NULL sein! */
	metamsg_t *meta=XCALLOC(1,sizeof(metamsg_t));
	outputfile = stdout;

	configure (RFC2MAUS, argc, argv,NULL);
	open_log ("rfc2maus",TRUE);
	if (outputfilename)
	{
		outputfile = fopen (outputfilename, "w");
		if (!outputfile)
		{
			fprintf (stderr, "rfc2maus: %s: %s\n", outputfilename,
					 strerror (errno));
			exit (1);
		}
	}

	if (!grp_load_list())
		exit (1);				/* Fehlermeldung schon ausgegeben */

	/* Cancel-Liste ffnen und Lock halten */
	if (cancel_init (1))
		exit (1);				/* Fehlermeldung schon ausgegeben */
#ifdef HAVE_TCP
#ifdef FEATURE_NNTP
	if (opts.newshost)
		nntp_open(opts.newshost);
#endif
#endif

	convert_messages (meta);
	convert_newsbatches (meta);
	append_mausbatches();

	/* Alle Cancels lschen - die sind jetzt verarbeitet */
	write_cmd();
	cancel_end(1,0);

	fprintf(outputfile,"#"LINEEND);
	if (outputfilename)
		fclose(outputfile);

#ifdef HAVE_TCP
#ifdef FEATURE_NNTP
	if (opts.newshost)
		nntp_close();
#endif
#endif

	close_log ();
	exit (0);
}

static void 
write_cmd(void)
{
	char fname[PATH_MAX];
	char lockfname[PATH_MAX];
	int fatal_err=0;
	char str[1024];
	char *p;
	FILE *f;
	minfo_t *akt;
	cancel_t *cakt;

	sprintf(lockfname,"%s/%s/etc/LOCK",opts.spoolbase,action);
	while (lock_create (lockfname, &fatal_err))
	{
		if (fatal_err)
		{
			log(LOG_ERR,"cannot lock with %s",lockfname);
			return;
		}
		sleep(5);
	}

	/* 
	 * Eigentlich mte man jetzt die noch brigen Cancels verarbeiten,
	 * sie irgendwie in die MausNetinternen IDs umsetzen ... 
	 * Brute Hack Alert.
	 */
	cakt=cancel_anker();
	if (cakt)
	{
		sprintf(fname,"%s/%s/etc/status.msg",opts.spoolbase,action);
		f=fopen(fname,"a");
		if (f)
		{
			while(cakt)
			{
				if (!cakt->done)
				{
					if (opts.workmode != USER 
						|| strstr(cakt->id,opts.mausdomain))
					{
						fprintf(f,"#%s\n",cakt->id);
						fprintf(f,":BX\n");
					}
				}
				cakt=cakt->next;
			}
			fclose(f);
		}
	}

	if (!info_read_list())
	{
		akt=info_get_anker();
		fprintf(outputfile,"#CMD" LINEEND);
		while (akt)
		{
			if (akt->needs_crc && akt->subscribed)
			{
				if (time(NULL)>akt->last_date+akt->pause*86400)
					fprintf(outputfile,":%s %ld"LINEEND,akt->kuerzel,akt->last_crc);
			}
			akt=akt->next;
		}
	}

	sprintf(fname,"%s/%s/etc/infile.cmd",opts.spoolbase,action);
	f=fopen(fname,"r");
	if (f)
	{
		fprintf(outputfile,"#CMD" LINEEND);
		while (fgets (str, sizeof(str), f))
		{
			p=strpbrk(str,"\r\n");
			if (p)
				*p='\0';
			if (*str==':')
				fprintf(outputfile,"%s"LINEEND,str);
			else
				fprintf(outputfile,":%s"LINEEND,str);
		}
		fclose(f);
	}

	sprintf(fname,"%s/%s/etc/once.cmd",opts.spoolbase,action);
	f=fopen(fname,"r");
	if (f)
	{
		fprintf(outputfile,"#CMD" LINEEND);
		while (fgets (str, sizeof(str), f))
		{
			p=strpbrk(str,"\r\n");
			if (p)
				*p='\0';
			if (*str==':')
				fprintf(outputfile,"%s"LINEEND,str);
			else
				fprintf(outputfile,":%s"LINEEND,str);
		}
		fclose(f);
		log(LOG_INFO,"added once.cmd",NULL);
	}
	sprintf(fname,"%s/%s/etc/status.msg",opts.spoolbase,action);
	f=fopen(fname,"r");
	if (f)
	{
		while (fgets (str, sizeof(str), f))
		{
			p=strpbrk(str,"\r\n");
			if (p)
				*p='\0';
			fprintf(outputfile,"%s"LINEEND,str);
		}
		fclose(f);
	}
	lock_delete(lockfname);
}

static void
convert_messages (metamsg_t *meta)
{
	DIR *dir;
	struct dirent *entry;
	char dirname[PATH_MAX];
	buf_t mbuf={NULL,NULL,0};

	sprintf (dirname, "%s/%s/outgoing", opts.spoolbase, action);
	tzset();
	dir = opendir (dirname);
	if (!dir)
	{
		unsigned char g[256];
		unsigned char m[1024];
		sprintf (m, "%s: %s\n"
				 "(dadurch verzgert sich die Nachrichtenauslieferung)",
				 dirname, strerror (errno));
		sprintf (g, "-%s", opts.gatemaster);

		errormail (NULL, "error creating infile",
				   m, g, NULL);
		return;
	}

	entry = readdir (dir);
	while (entry)
	{
		/* Keine unsichtbaren Dateien */
		if (entry->d_name[0] != '.')
		{
			char zwi[PATH_MAX];
			char *p;
			int work_it=1;

			sprintf (zwi, "%s/%s/outgoing/%s", opts.spoolbase, action, entry->d_name);

			/* Nur Dateien ohne Endung und .old's wandeln - alles 
			 * andere ist als Mll anzusehen, z.B. emacs-Backups.
			 */
			p=strrchr(zwi,'/');
			if (!p)
				p=zwi;
			else 
				p++;
			if (strncasecmp(p,"mail",4) && !isdigit(*p))
				work_it=0;
			if ((p=strchr(p,'.'))!=NULL)
			{
				if (strcasecmp(p+1,"old"))
					work_it=0;
			}
			if (work_it)
			{
				if (read_and_convert_file (zwi,&mbuf,meta)==0)
					rfc2maus (outputfile, meta);
			}
		}
		entry = readdir (dir);
	}
	closedir (dir);
	if (mbuf.start)
		free(mbuf.start);
}

static void
append_mausbatches (void)
{
	DIR *dir;
	struct dirent *entry;
	char dirname[PATH_MAX];

	sprintf (dirname, "%s/%s/outgoing", opts.spoolbase, action);
	tzset();
	dir = opendir (dirname);
	if (!dir)
	{
		unsigned char g[256];
		unsigned char m[1024];
		sprintf (m, "%s: %s\n"
				 "(dadurch verzgert sich die Nachrichtenauslieferung)",
				 dirname, strerror (errno));
		sprintf (g, "-%s", opts.gatemaster);

		errormail (NULL, "error creating infile",
				   m, g, NULL);
		return;
	}

	entry = readdir (dir);
	while (entry)
	{
		if (entry->d_name[0] == 'm' && entry->d_name[1]=='t')
		{
			char buf[32768];
			char mausb[PATH_MAX];
			FILE *in;
			char *p;

			sprintf (mausb, "%s/%s/outgoing/%s", opts.spoolbase, action, entry->d_name);
			in=fopen(mausb,"r");
			if (!in)
			{
				log(LOG_NOTICE,"Trouble appending mausbatch %s", mausb);
			}
			else
			{
				while (fgets(buf,sizeof(buf),in))
				{
					p=strpbrk(buf,"\r\n");
					if (p)
						*p='\0';
					fprintf(outputfile,"%s\r\n",buf);
				}
				fclose(in);
			}
		}
		entry = readdir (dir);
	}
	closedir (dir);
}

static void
convert_newsbatches (metamsg_t *meta)
{
	DIR *dir;
	struct dirent *entry;
	char dirname[PATH_MAX];

	sprintf (dirname, "%s/%s/outgoing", opts.spoolbase, action);
	tzset();
	dir = opendir (dirname);
	if (!dir)
	{
		unsigned char g[256];
		unsigned char m[1024];
		sprintf (m, "%s: %s\n"
				 "(dadurch verzgert sich die Nachrichtenauslieferung)",
				 dirname, strerror (errno));
		sprintf (g, "-%s", opts.gatemaster);

		errormail (NULL, "error creating infile",
				   m, g, NULL);
		return;
	}

	entry = readdir (dir);
	while (entry)
	{
		/* Keine unsichtbaren Dateien */
		if (entry->d_name[0] == 'n' && entry->d_name[1]=='b')
		{
			char newsb[PATH_MAX];
			char mausb[PATH_MAX];
			char *p;
			FILE *f;

			sprintf (newsb, "%s/%s/outgoing/%s", opts.spoolbase, action, 
				entry->d_name);

			strcpy(mausb,newsb);
			p=strrchr(mausb,'/')+1;
			*p='m';
			p[1]='t';

			f=fopen(mausb,"w");
			if (f)
			{
				if (!rnews_batch(newsb, meta, f, rfc2maus))
					log(LOG_NOTICE,"Trouble converting newsbatch %s", newsb);
				/* dirty Hack im Gatewaymodus, um Rckmeldungen pro
				 * Batchfile zu erzwingen ...
				 */
				fprintf(f,"#%s\r\n",strrchr(mausb,'/')+1);
				fclose(f);
				/* und nun auf jeden Fall verschieben */
				sprintf (mausb, "%s/%s/sent2maus/%s", opts.spoolbase, 
					action, entry->d_name);
				rename(newsb,mausb);
			}
			else
				log(LOG_NOTICE,"Trouble writing mausbatch %s", mausb);
		}
		entry = readdir (dir);
	}
	closedir (dir);
}

