/* src/srfcmail.c: send metamsg as rfc822-mail */

#include "ownstd.h"
#ifdef HAVE_SYS_STAT_H
#  include <sys/stat.h>
#endif
#include <errno.h>
#include <ctype.h>
#include <conflib.h>
#include <stdarg.h>

#include "mausconlib.h"
#include "metamsg.h"
#include "smtp.h"
#include "srfcmail.h"

/*
   Zwei Methoden werden zur Zeit untersttzt, die erste ist "echtes"
   smtp, die zweite smtp via Batchfile. Das erste ist prinzipiell
   schneller und resourcensparender, das zweite problemloser, da
   smtliche Fehler vom rsmtp gehandhabt werden. 
   Mglicherweise gibt's Dupes, aber das sollte nur dann passieren,
   wenn der smtp-Daemon in ungluecklichen Momenten die 
   Verbindung unterbricht.
 */
#define CRLF "\r\n"


#ifdef FEATURE_SMTP
static int main_tried;
static FILE *mainf;
static FILE *primef;
static char primename[128];
#endif

static FILE *batchf;

/* 
 * Aufrufen, wenn es gebraucht wird, d.h., wenn der direkte smtp-Versuch
 * gescheitert ist. Falls auch das nicht klappt, weil das Batchfile
 * nicht geffnet werden kann -> Pech, Ende, Aus. Fr den Gatemodus
 * sollte man das noch mal berdenken, aber fr den Usermodus ist es
 * das beste.
 */
static int 
open_batchfile(void)
{
	char fname[PATH_MAX];
	char *pos;
	long i;
	struct stat st;
	sprintf (fname, "%s/%s/incoming/mailbatch.", opts.spoolbase, action);
	pos=fname+strlen(fname);
	i=0;
	do
	{
		i++;
		sprintf(pos,"%ld",i);
	} while (stat(fname,&st)==0 && st.st_size!=0);
	batchf = fopen (fname, "a");
	if (!batchf)
	{
		int err;

		err = errno;
		log (LOG_CRIT, "Cannot open Mailbatch: '%s': %s",
			 fname, strerror (err));
		exit (1);
	}
	/* Wenn Batchfile neu angelegt wird und nicht etwas 
	 * angehngt wird.
	 */
	if (ftell (batchf) <= 0)
	{
		fprintf (batchf, "HELO %s%s" CRLF,
				 opts.pollsite, opts.mausdomain);
	}
	return 0; /* OK */
}

int srfcmail(metamsg_t *msg)
{
	char *p;
	int ret;
	if (!msg)
	{
		log(LOG_WARNING,"srfcmail: no message");
		/* Wenn man nichts versendet, kann man auch keine Fehler machen */
		return 0;
	}
	if (!msg->env_to)
	{
		log(LOG_WARNING,"srfcmail: kein Empfaenger");
		return 1;
	}

#ifdef FEATURE_SMTP
	/* Zuerst der Versuch, die Nachricht direkt zu verschicken */
	if (opts.direct_smtp)
	{
		p=strchr(msg->env_to,'@');
		if (p)
		{
			if (strcasecmp(p+1,primename))
			{
				if (primef)
					smtp_close(primef);
				primef=smtp_open(p+1);
				if (primef)
				{
					strcpy(primename,p+1);
				}
				else
					*primename='\0';
			}
			if (primef)
			{
				ret=smtp_mail(primef,msg,0);
				if (ret==2)
				{
					smtp_close(primef);
					primef=0;
					*primename='\0';
				}
				else if (ret==0)
					return 0; /* verschickt */
			}
		}
	}

	/* Nun der Versuch, die Nachricht an den Hauptserver zu schicken 
	 * (an die eigene Kiste).
	 */
	if (!mainf && !main_tried)
	{
		mainf = smtp_open(opts.mailhost);
		main_tried=1;
	}
	if (mainf)
	{
		ret=smtp_mail(mainf,msg,0);
		if (ret==2)
		{
			smtp_close(mainf);
			mainf=0;
		}
		else if (ret==0)
			return 0; /* verschickt */
	}
#endif
	/* Letzter Versuch, das Batchfile */
	if (!batchf)
	{
		if (open_batchfile())
			return 1;
	}
	return smtp_mail(batchf,msg,1);
}

void
srfcmail_end_all(void)
{
#ifdef FEATURE_SMTP
	if (mainf)
	{
		smtp_close(mainf);
		mainf=NULL;
	}
	if (primef)
	{
		smtp_close(primef);
		primef=NULL;
	}
#endif
	if (batchf)
	{
		fprintf(batchf,"QUIT" CRLF);
		fclose(batchf);
		batchf=NULL;
	}
}
