/* srfcnews.c - send news in rfc-format */

#include "ownstd.h"
#ifdef HAVE_SYS_STAT_H
#  include <sys/stat.h>
#endif
#include <errno.h>
#include <ctype.h>
#include <conflib.h>
#include <stdarg.h>

#include "mausconlib.h"
#include "metamsg.h"
#include "rfc.h"
#include "info2msg.h"
#ifdef FEATURE_NNTP
#include "nntp.h"
#endif
#include "srfcnews.h"

#define BATCHLEN	1024*256

static FILE *newsbatchf;
#ifdef FEATURE_NNTP
static int nntp_tried;
static int nntp_ok;
#endif

static long batchcount=0;
static size_t batchlen=0;

static int 
close_batch(void)
{
	if (newsbatchf)
		fclose(newsbatchf);
	newsbatchf=NULL;
	return 0;
}

/*
 * Aufrufen, wenn es gebraucht wird, d.h., entweder NNTP auskonfiguriert
 * (inc. nicht eincompiliert) ist oder das bertragen einer Nachricht
 * gescheitert ist.
 */
static int
open_batchfile(void)
{
	char fname[PATH_MAX];
	char *pos;
	long i;
	struct stat st;
	sprintf (fname, "%s/%s/incoming/newsbatch.", opts.spoolbase, action);
	pos=fname+strlen(fname);
	i=batchcount;
	do
	{
		i++;
		sprintf(pos,"%ld",i);
	} while (stat(fname,&st)==0 && st.st_size!=0);
	batchcount=i;

	newsbatchf=fopen(fname,"w");
	if (!newsbatchf)
	{
		log (LOG_CRIT, "Cannot open Newsbatch: '%s': %s", fname, strerror (errno));
		return 1;
	}
	return 0;
}

int 
srfcnews(metamsg_t *msg)
{
	int ret;
	if (!msg)
	{
		log(LOG_WARNING,"srfcmail: no message");
		/* Wenn man nichts versendet, kann man auch keine Fehler machen */
		return 0;
	}
	if (!msg->newsgroups || !msg->newsgroups->text)
	{
		log(LOG_ERR,"srfcmail: keine Gruppe angegeben");
		return 1;
	}
	ret=1; /* try again */
#ifdef FEATURE_NNTP
	if (!nntp_tried)
	{
		if (!dont_use_nntp)
			nntp_ok=!nntp_open(opts.newshost);
		nntp_tried=1;
	}
	if (nntp_ok)
	{
		ret=nntp_send(msg);
		if (!ret || ret==2)
			return 0;
		else if (ret==3)
		{
			nntp_close();
			nntp_ok=0;
		}
	}
#endif
	if (!newsbatchf)
	{
		ret=open_batchfile();
		if (ret)	
			return ret;
	}
	else if (strlen(msg->body.start)+ftell(newsbatchf)>BATCHLEN)
	{
		if (newsbatchf)
			close_batch();
		ret=open_batchfile();
		if (ret)
			return ret;
	}
	/* 
	 * Das rnews-format benoetigt die Laenge der Nachricht inclusive
	 * body. Die haben wir noch gar nicht ...
	 * Die rfc_export-Routine soll aber nach Mglichkeit genutzt werden
	 * (Vermeidung doppelter Programmierung), also was tun? Eine 
	 * besonders tolle Mglichkeit ist mir nicht eingefallen, was jetzt
	 * kommt, ist definitiv ein Hack ;-)
	 */
	/* Viel Platz lassen :-) */
	{
		char *filler="                    ";
		long fillerpos;
		long newspos;
		long endpos;
		fprintf(newsbatchf,"#! rnews ");
		fillerpos=ftell(newsbatchf);
		fprintf(newsbatchf,"%s\n",filler);
		newspos=ftell(newsbatchf);
		rfc_print_message(msg,newsbatchf,"\n",0);
		endpos=ftell(newsbatchf);
		if (fseek(newsbatchf,fillerpos,SEEK_SET))
		{
			/* Tja ... das heit, der Batch ist kaputt. Ende und aus. */
			log (LOG_CRIT, "Cannot seek on newsbatchf: %s", 
				strerror (errno));
			exit(1);
		} 
		fprintf(newsbatchf,"%ld",endpos-newspos);
		/* Zurueck zum Ende */
		if (fseek(newsbatchf,endpos,SEEK_SET))
		{
			/* Auch das ist ein unrecoverable error */
			log (LOG_CRIT, "Cannot seek on newsbatchf: %s", 
				strerror (errno));
			exit(1);
		}
		batchlen+=endpos+fillerpos+9;
	}
	return 0;
}

int 
srfcnews_end_all(void)
{
#ifdef FEATURE_NNTP
	if (nntp_ok)
	{
		nntp_close();
		nntp_ok=0;
		nntp_tried=0;
	}
#endif
	if (newsbatchf)
	{
		fclose(newsbatchf);
	}
	return 0;
}
