/*
 * Mail2News auf Basis von MausConnect
 */

#include "ownstd.h"
#ifdef HAVE_SYS_STAT_H
#  include <sys/stat.h>
#endif
#include <errno.h>
#include <ctype.h>
#include <getopt.h>
#include <fnmatch.h>

#include "conflib.h"
#include "mausconlib.h"
#include "metamsg.h"
#include "maus.h"
#include "groups.h"
#include "xmalloc.h"

#define PROGNAME "mtdelay"

/* FILLME */
char *global_help_text=
	PROGNAME ": Mail2News-Gateways auf MausTauschbasis\n"
	;

struct eintrag {
	char *id;
	char *gateway;
	time_t zeit;
	metalist_t *groups;
	struct eintrag *next;
};

struct geintrag
{
	char *grp;
	unsigned int zahl;
	time_t gesamtzeit;
	struct geintrag *next;
};

struct eintrag *anker;
struct geintrag *ganker;

unsigned int arg_tage = 5; /* default */
unsigned int arg_anzeigezahl = 20;

/* ------------------------------------------------------------------------- */
metamsg_t meta;

static void work (FILE *in);
static void usage (const char *prog);
static void eintragen(time_t sekunden);
static void eintragen_gruppe(time_t sekunden);

static void 
usage (const char *prog)
{
	fprintf(stderr,
		"usage: %s [-i inputfile] [-d tage] \n",prog);
}

static void
eintragen(time_t sekunden)
{
	metalist_t *ng;
	struct eintrag *new;
	new=XMALLOC(sizeof(struct eintrag));
	new->id=XSTRDUP(meta.message_id);
	if (meta.x_gateway)
		new->gateway=XSTRDUP(meta.x_gateway);
	else
		new->gateway=NULL;
	new->zeit=sekunden;
	new->groups=NULL;
	new->next=anker;
	anker=new;
	for (ng=meta.newsgroups;ng;ng=ng->next)
	{
			meta_add_list(&new->groups,ng->text,NULL,0);
	}
}

static void
eintragen_gruppe(time_t sekunden)
{
	metalist_t *ng;
	for (ng=meta.newsgroups;ng;ng=ng->next)
	{
		struct geintrag *akt;
		for (akt=ganker;akt;akt=akt->next)
		{
			if (!strcasecmp(akt->grp,ng->text))
				break;
		}
		if (!akt)
		{
			struct geintrag *new=XMALLOC(sizeof(struct geintrag));
			new->grp=XSTRDUP(ng->text);
			new->gesamtzeit=sekunden;
			new->zahl=1;
			new->next=ganker;
			ganker=new;
		}
		else
		{
			akt->zahl++;
			akt->gesamtzeit+=sekunden;
		}
	}
}

static void
work (FILE *in)
{
	unsigned char line[32*1024];    /* Lesepuffer */
	int zurueckgestellt=0;
	int ende;
	struct geintrag *g;
	struct eintrag *e;
	time_t grenze;
	time_t jetzt=time(NULL);
	FILE *f;

	grenze=jetzt-arg_tage*86400L;


	do
	{
		ende=maus_import(in,&meta,line,sizeof(line),&zurueckgestellt);
		if (!meta.env_to && meta.newsgroups)
		{
			if (meta.date<=grenze)
				eintragen(jetzt-meta.date);
			eintragen_gruppe(jetzt-meta.date);
		} /* if news */
	} while (!ende);

	sprintf(line,"sort -n -r | head -%u",arg_anzeigezahl);
	printf("%s\n",line);
	f=popen(line,"w");
	if (!f)
	{
		log(LOG_CRIT,"cannot popen: %m");
		exit(1);
	}
	printf("Verzgerung nach Gruppen:\n");
	fflush(stdout);
	for (g=ganker;g;g=g->next)
	{
		float schnitt;
		schnitt=(g->gesamtzeit/(float) g->zahl)/86400L;
		fprintf(f,"%6.2f %s\n",schnitt,g->grp);
	}
	pclose(f);
	fflush(stdout);
	printf("Herausragend langsame Nachrichten:\n");
	fflush(stdout);

	f=popen(line,"w");
	if (!f)
	{
		log(LOG_CRIT,"cannot popen: %m");
		exit(1);
	}
	for (e=anker;e;e=e->next)
	{
		float schnitt;
		metalist_t *ng;
		schnitt=e->zeit/(float)86400L;
		fprintf(f,"%6.2f %s",schnitt,e->id);
		if (e->gateway)
			fprintf(f," [%s]",e->gateway);
		for (ng=e->groups;ng;ng=ng->next)
			fprintf(f," %s",ng->text);
		fprintf(f,"\n");
	}
	pclose(f);
	fflush(stdout);
}

int
main (int argc, char **argv)
{
	FILE *outputf;					/* Outfile.txt for Reading */
	FILE *inputf;

	int option_index = 0;
	char *libdirname = NULL;
	char *inputfilename = NULL;

	struct option opts[] =
	{
		/* Optionen mit Shortcuts */
		{"help",0, NULL, 'h'},
		{"input",1, NULL, 'i'},
		{"libdir", 1, NULL, 'l'},
		{"tage", 0, NULL, 't'},
		{"zahl", 0, NULL, 'z'},
		{NULL, 0, NULL, 0}
	};

	outputf=stdout;
	inputf=stdin;

	while (1)
	{
		int c;

		c = getopt_long (argc, argv, "hi:l:t:z:",
						 opts, &option_index);

		/* Detect the end of the options.   */
		if (c == -1)
			break;
		switch (c)
		{
		default: break; 
		case 'h': usage (argv[0]); exit (0); break;
		case 'i': inputfilename = optarg; break;
		case 'l': libdirname = optarg; break;
		case 't': arg_tage = atoi(optarg); break;
		case 'z': arg_anzeigezahl = atoi(optarg); break;
		case '?':
			fprintf(stderr,"%s: too many parameters\n",argv[0]);
			fprintf(stderr,"use %s --help\n",argv[0]);
			exit (2);
			break;
		}
	}							/* while(1) */

	if (!libdirname)
	{
		char *p;

		p = getenv ("MAUSGATELIBDIR");
		if (p)
			libdirname = p;
	}
	if (!libdirname)
		libdirname=DEFAULT_LIBDIR;

	if (do_conf (argv[0], libdirname, NULL, PROGNAME))
		exit (1);

	if (!grp_load_list())
		exit (1);

	open_log (PROGNAME,FALSE);

	if (inputfilename)
		inputf=xfopen(inputfilename,"r");

	work (inputf);

	close_log ();

	exit (0);
}

