/*
 * Skelett eines MausTausch-Filters auf Basis von MausConnect, bzw. der
 * mausconlib.
 */

#include "ownstd.h"
#ifdef HAVE_SYS_STAT_H
#  include <sys/stat.h>
#endif
#include <errno.h>
#include <ctype.h>
#include <conflib.h>
#include <getopt.h>

#include "mausconlib.h"
#include "metamsg.h"
#include "maus.h"
#include "groups.h"
#include "xmalloc.h"

#define PROGNAME "skeleton-mt-filter"

/* FILLME */
char *global_help_text=
	PROGNAME ": Grundstock eines Filters\n"
	;

/* ------------------------------------------------------------------------- */
metamsg_t meta;

static void work (FILE *in, FILE *out);
static void usage (const char *prog);

static void 
usage (const char *prog)
{
	fprintf(stderr,"usage: %s [-i inputfile] [-o outputfile] [-v]\n",
		prog);
}

static void
work (FILE *in, FILE *out)
{
	unsigned char line[32*1024];    /* Lesepuffer */
	int zurueckgestellt=0;

	int ende;

	do
	{
		ende=maus_import(in,&meta,line,sizeof(line),&zurueckgestellt);

#if 0
		if (meta.maus_special_id)
			dann ist es ein Infofile, welches es ist bekommt man mit
			info_find(meta.maus_special_id) heraus.
		else if (meta.env_to)
			dann ist es eine Mail.
		else if (meta.newsgroups)
			dann ist es News.
#endif
	} while (!ende);
}

int
main (int argc, char **argv)
{
	FILE *outputf;					/* Outfile.txt for Reading */
	FILE *inputf;

	int option_index = 0;
	int libdir_given = 0;
	char *libdirname = DEFAULT_LIBDIR;
	char *outputfilename = NULL;
	char *inputfilename = NULL;

	struct option opts[] =
	{
		/* Optionen mit Shortcuts */
		{"help",0, NULL, 'h'},
		{"input",1, NULL, 'i'},
		{"output", 1, NULL, 'o'},
		{"libdir", 1, NULL, 'l'},
		{"verbose", 0, NULL, 'v'},
		{NULL, 0, NULL, 0}
	};

	outputf=stdout;
	inputf=stdin;

	while (1)
	{
		int c;

		c = getopt_long (argc, argv, "hi:l:o:v",
						 opts, &option_index);

		/* Detect the end of the options.   */
		if (c == -1)
			break;
		switch (c)
		{
		default: 
			break; 
		case 'h':
			usage (argv[0]);
			exit (0);
			break;
		case 'i':
			inputfilename = optarg;
			break;
		case 'l':
			libdir_given = 1;
			libdirname = optarg;
			break;
		case 'o':
			outputfilename = optarg;
			break;
		case 'V':
			argument_value = optarg;
			break;
		case '?':
			fprintf(stderr,"%s: too many parameters\n",argv[0]);
			fprintf(stderr,"use %s --help\n",argv[0]);
			exit (2);
			break;
		}
	}							/* while(1) */

	if (!libdir_given)
	{
		char *p;

		p = getenv ("MAUSGATELIBDIR");
		if (p)
			libdirname = p;
	}

#if 0
	Keine weiteren Parameter erlaubt? 
	if (argv[optind])
	{
		fprintf(stderr,"%s: too many parameters\n",argv[0]);
		fprintf(stderr,"use %s --help\n",argv[0]);
		exit (2);
	}
	if (do_conf (argv[0], libdirname, NULL, PROGNAME))
		exit (1);

	...

#endif
#if 0
	Genau ein Parameter erlaubt, nmlich die Aktion? */
	if (!argv[optind] || argv[optind+1])
	{
		fprintf(stderr,"%s: too many parameters\n",argv[0]);
		fprintf(stderr,"use %s --help\n",argv[0]);
		exit (2);
	}
	if (do_conf (argv[0], libdirname, NULL, argv[optind]))
		exit(1);

	...

#endif

	open_log (PROGNAME,FALSE); /* kein msglog */

	if (inputfilename)
		inputf=xfopen(inputfilename,"r");
	if (outputfilename)
		outputf=xfopen(outputfilename,"r");

#if LOCK_AUF_GRUPPENNOTWENDIG
	if (!grp_lock())
		exit (1);
#endif
#if LADEN_DER_GRUPPEN_NOTWENDIG
	if (!grp_load_list())
		exit (1);
#endif
#if LADEN_DER_INFOFILES_NOTWENDIG
	if (info_read_list())
		exit(1);
#endif

	work (inputf,outputf);

#if LADEN_DER_GRUPPEN_NOTWENDIG
	if (!grp_save_list())
	{
		close_log ();
#if LADEN_DER_GRUPPEN_NOTWENDIG
		grp_unlock();
#endif
		exit (1);
	}
#endif
#if LADEN_DER_GRUPPEN_NOTWENDIG
	grp_unlock();
#endif

#if LADEN_UND_SCHREIBEN_DER_INFOFILES_NOTWENDIG
	info_write_list();
#endif
	close_log ();

	exit (0);
}

