/*
 * Mail2News auf Basis von MausConnect
 */

#include "ownstd.h"
#ifdef HAVE_SYS_STAT_H
#  include <sys/stat.h>
#endif
#include <errno.h>
#include <ctype.h>
#include <getopt.h>
#include <fnmatch.h>

#include "conflib.h"
#include "mausconlib.h"
#include "metamsg.h"
#include "maus.h"
#include "groups.h"
#include "xmalloc.h"

#define PROGNAME "mtmail2news"

/* FILLME */
char *global_help_text=
	PROGNAME ": Mail2News-Gateways auf MausTauschbasis\n"
	;

struct control {
	char *from;
	char *subject;
	char *to;
	char *newsgroups;
	char *sender;
	struct control *next;
};

struct control *anker;
struct control *last=NULL;
char *sender;
char *organisation;

/* ------------------------------------------------------------------------- */
metamsg_t meta;

static void work (FILE *in, FILE *out);
static void usage (const char *prog);
static BOOLEAN read_controlfile(const char *);
static void add_control(char *from, char *subject, char *to, char *newsgroups, 
	char *sender);
static BOOLEAN transform_id(char *id);

static void 
usage (const char *prog)
{
	fprintf(stderr,
		"usage: %s [-i inputfile] [-o outputfile] [-v]\n"
		" [-c controlfile | -n newsgroups]\n",
		prog);
}

static void
work (FILE *in, FILE *out)
{
	unsigned char line[32*1024];    /* Lesepuffer */
	int zurueckgestellt=0;

	int ende;

	do
	{
		ende=maus_import(in,&meta,line,sizeof(line),&zurueckgestellt);
		if (meta.env_to)
		{
			/* Mail erhalten */
			struct control *akt;
			BOOLEAN done=FALSE;
			for(akt=anker;akt;akt=akt->next)
			{
				BOOLEAN matched=TRUE;
				if (akt->from)
				{
					if (fnmatch(akt->from,meta.env_from,FNM_CASEFOLD))
						matched=FALSE;
				}
				if (akt->subject)
				{
					if (fnmatch(akt->subject,meta.subject,FNM_CASEFOLD))
						matched=FALSE;
				}
				if (akt->to)
				{
					if (fnmatch(akt->to,meta.env_to,FNM_CASEFOLD))
						matched=FALSE;
				}
				if (matched && akt->newsgroups)
				{
					char *s;
					char *p;
					meta.is_mail=FALSE;
					free(meta.env_to);
					meta.env_to=NULL;
					if (meta.env_from)
					{
						free(meta.env_from);
						meta.env_from=FALSE;
					}
					if (meta.sender)
						meta_free_list (&meta.sender);
					meta.sender=NULL;

					if (akt->sender)
						meta.sender=meta_create_element(akt->sender,NULL);
					else if (sender)
						meta.sender=meta_create_element(sender,NULL);

					if (organisation)
					{
						free(meta.organization);
						meta.organization=strdup(organisation);
					}

					s=akt->newsgroups;
					while (s && *s)
					{
						char c=0;
						p=strpbrk(s,"\t :,");
						if (p)
						{
							c=*p;
							*p=0;
						}
						meta_add_list(&meta.newsgroups,s,NULL,0);
						if (p)
						{
							*p=c;
							s=p+1;
							while (isspace(*s) || *s==':' || *s==',')
								s++;
						}
						else
							s=NULL;
					}
					if (transform_id(meta.message_id))
						maus_export(out,&meta);
					done=TRUE;
				}
				else if (matched && akt->newsgroups==NULL)
					done=TRUE;
				if (!done)
				{
					meta.env_to=XSTRDUP(opts.mailuser);
					meta.env_from=XSTRDUP(sender);
					meta.is_mail=TRUE;
					maus_export(out,&meta);
				}
			} /* for durch alle Regeln */
		} /* if mail */
	} while (!ende);
}

int
main (int argc, char **argv)
{
	FILE *outputf;					/* Outfile.txt for Reading */
	FILE *inputf;

	int option_index = 0;
	char *libdirname = NULL;
	char *outputfilename = NULL;
	char *newsgroups=NULL;
	char *inputfilename = NULL;
	char *controlfilename = NULL;
	char *arg_sender = NULL;
	char *arg_organisation = NULL;

	struct option opts[] =
	{
		/* Optionen mit Shortcuts */
		{"control-file", 1, NULL, 'c'},
		{"help",0, NULL, 'h'},
		{"input",1, NULL, 'i'},
		{"newsgroups",1, NULL, 'n'},
		{"organisation", 1, NULL, 'O'},
		{"output", 1, NULL, 'o'},
		{"libdir", 1, NULL, 'l'},
		{"sender", 1, NULL, 's'},
		{"verbose", 0, NULL, 'v'},
		{NULL, 0, NULL, 0}
	};

	outputf=stdout;
	inputf=stdin;

	while (1)
	{
		int c;

		c = getopt_long (argc, argv, "hi:l:n:o:O:s:v",
						 opts, &option_index);

		/* Detect the end of the options.   */
		if (c == -1)
			break;
		switch (c)
		{
		default: break; 
		case 'c': controlfilename = optarg; break;
		case 'h': usage (argv[0]); exit (0); break;
		case 'i': inputfilename = optarg; break;
		case 'l': libdirname = optarg; break;
		case 'n': newsgroups = optarg; break;
		case 'o': outputfilename = optarg; break;
		case 'O': arg_organisation = optarg; break;
		case 's': arg_sender = optarg; break;
		case '?':
			fprintf(stderr,"%s: too many parameters\n",argv[0]);
			fprintf(stderr,"use %s --help\n",argv[0]);
			exit (2);
			break;
		}
	}							/* while(1) */

	if (!libdirname)
	{
		char *p;

		p = getenv ("MAUSGATELIBDIR");
		if (p)
			libdirname = p;
	}
	if (!libdirname)
		libdirname=DEFAULT_LIBDIR;

	if (do_conf (argv[0], libdirname, NULL, PROGNAME))
		exit (1);

	open_log (PROGNAME,FALSE);

	if (!controlfilename && !newsgroups)
	{
		log(LOG_ERR,"Steuerdatei und Gruppe nicht angegeben");
		exit(2);
	}
	else if (controlfilename)
	{
		if (!read_controlfile(controlfilename))
			exit(1);
	}
	if (newsgroups)
	{
		add_control("*",NULL,NULL,newsgroups,NULL);
	}
	if (arg_organisation)
		organisation=arg_organisation;
	if (arg_sender)
		sender=arg_sender;
	if (!sender)
	{
		fprintf(stderr,"Kein Sender angegeben!\n");
		exit(2);
	}

	if (inputfilename)
		inputf=xfopen(inputfilename,"r");
	if (outputfilename)
		outputf=xfopen(outputfilename,"w");

#if LOCK_AUF_GRUPPENNOTWENDIG
	if (!grp_lock())
		exit (1);
#endif
	if (!grp_load_list())
		exit (1);
#if LADEN_DER_INFOFILES_NOTWENDIG
	if (info_read_list())
		exit(1);
#endif

	work (inputf,outputf);

#if LADEN_DER_GRUPPEN_NOTWENDIG
	if (!grp_save_list())
	{
		close_log ();
#if LADEN_DER_GRUPPEN_NOTWENDIG
		grp_unlock();
#endif
		exit (1);
	}
#endif
#if LADEN_DER_GRUPPEN_NOTWENDIG
	grp_unlock();
#endif

#if LADEN_UND_SCHREIBEN_DER_INFOFILES_NOTWENDIG
	info_write_list();
#endif
	close_log ();

	exit (0);
}

static void 
add_control(char *from, char *subject, char *to, char *newsgroups, 
	char *sender)
{
	struct control *n=XMALLOC(sizeof(struct control));
	if (from)
	{
		n->from=XSTRDUP(from);
		strtrim(n->from);
	}
	else
		n->from=NULL;
	if (subject)
	{
		n->subject=XSTRDUP(subject);
		strtrim(n->subject);
	}
	else
		n->subject=NULL;
	if (to)
	{
		n->to=XSTRDUP(to);
		strtrim(n->to);
	}
	else
		n->to=NULL;
	if (newsgroups)
	{
		n->newsgroups=XSTRDUP(newsgroups);
		strtrim(n->newsgroups);
	}
	else
		n->newsgroups=NULL;
	if (sender)
	{
		n->sender=XSTRDUP(sender);
		strtrim(n->sender);
	}
	else
		n->sender=NULL;
	if (last)
		last->next=n;
	else
		anker=n;
	last=n;
	n->next=NULL;
}

static BOOLEAN 
read_controlfile(const char *fname)
{
	int erg;
	conf_list_t *list=NULL;
	conf_list_t *akt;
	conf_t tab[]=
	{
		{"active",NULL,CL_LIST,&list,0},
		{"sender",NULL,CL_STRING,&sender,0},
		{"organisation",NULL,CL_STRING,&organisation,0},
		{NULL,NULL,0,NULL,0}
	};
	erg = cl_getconfstanza (fname, tab, "liste",1);
	if (erg)
	{
		/* tja, pech */
		log (LOG_ERR, "kann Liste nicht aus %s lesen", fname);
		return FALSE;
	}
	for (akt=list;akt; akt=akt->next)
	{
		char *from=NULL;
		char *subject=NULL;
		char *to=NULL;
		char *newsgroups=NULL;
		char *sender=NULL;
		conf_t tab2[]=
		{
			{"from",NULL,CL_STRING,&from,0},
			{"subject",NULL,CL_STRING,&from,0},
			{"to",NULL,CL_STRING,&to,0},
			{"newsgroups",NULL,CL_STRING,&newsgroups,0},
			{"sender",NULL,CL_STRING,&sender,0},
			{NULL,NULL,0,NULL,0}
		};
		erg = cl_getconfstanza (fname, tab2, akt->inhalt,1);
		if (erg)
		{
			/* tja, pech */
			log (LOG_ERR, "kann Stanza %s nicht aus %s lesen", 	
				akt->inhalt,fname);
		}
		else
		{
			add_control(from, subject, to, newsgroups, sender);
			cl_clearconf(tab2);
		}
	}
	cl_closeconf();
	return TRUE;
}

static BOOLEAN 
transform_id(char *id)
{
	char *p;
	char *q;
	char loc[256];
	char dom[256];
/*
 *	"<",  end at the next ">", and reject the message if
 *	no ">" is found or a second "<" is seen  before  the
 *	">".  Also reject the message if the message ID con-
 *	tains no "@" or more than one "@", or if it contains
 *	no  ".".   Also reject the message if the message ID
 *	contains non-ASCII characters, ASCII control charac-
 *	ters, or white space.
 */
	for (p=id;*p;p++)
	{
		if ((*p & 127) != *p || isspace(*p) || *p<32)
			return FALSE;
	}
	if (!strchr(meta.message_id,'.'))
		return FALSE;

	p=strchr(meta.message_id,'@');
	if (!p || strchr(p+1,'@') || !p[1])
		return FALSE;

	strcpy(dom,p+1);
	strcpy(loc,meta.message_id);
	p=strchr(loc,'@');
	*p='\0';
	if (!*loc)	
		return FALSE;
/*
 *   3.   In both  components,  transliterate  leading  dots
 *        (".", ASCII 46), trailing dots, and dots after the
 *        first in sequences  of  two  or  more  consecutive
 *        dots, into underscores (ASCII 95).
 */ 
	for(p=loc;*p=='.';p++)
		*p='_';
	for(p=dom;*p=='.';p++)
		*p='_';
	for (p=loc+strlen(loc)-1;p>loc && *p=='.';p--)
		*p='_';
	for (p=dom+strlen(dom)-1;p>dom && *p=='.';p--)
		*p='_';
	for (p=loc+1;*p;p++)
	{
		if (*p!='.')
			continue;
		for (q=p;*q=='.';q++)
			;
		if (q==p+1)
			continue;
		while (p<q)
			*p++='_';
	}
	for (p=dom+1;*p;p++)
	{
		if (*p!='.')
			continue;
		for (q=p;*q=='.';q++)
			;
		if (q==p+1)
			continue;
		while (p<q)
			*p++='_';
	}
/*
    4.   In both components, transliterate disallowed char-
         acters other than  dots  (see  the  definition  of
         <unquoted-char>  in  section  5.2)  to underscores
         (ASCII 95).
unquoted-char = <ASCII printable character except !()<>@,;:\".[]>
*/
	p=loc;
	while ((p=strpbrk(p,"\\!()<>@,;:\"[]"))!=NULL)
		*p='_';
	p=dom;
	while ((p=strpbrk(p,"\\!()<>@,;:\"[]"))!=NULL)
		*p='_';
	sprintf(id,"%s@%s",loc,dom);
	return TRUE;
}
